/* -*- mode: c++; indent-tabs-mode: nil -*- */
/*
    Pseudo_QC_String.qpp

    Qore Programming Language

    Copyright (C) 2003 - 2023 Qore Technologies, s.r.o.

    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
    to deal in the Software without restriction, including without limitation
    the rights to use, copy, modify, merge, publish, distribute, sublicense,
    and/or sell copies of the Software, and to permit persons to whom the
    Software is furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
    DEALINGS IN THE SOFTWARE.

    Note that the Qore library is released under a choice of three open-source
    licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
    information.
*/

#include <qore/Qore.h>
#include "qore/intern/ql_crypto.h"
#include "qore/intern/QoreLibIntern.h"

#include <cctype>
/* Qore class Qore::<string> */

qore_classid_t CID_PSEUDOSTRING;
QoreClass* QC_PSEUDOSTRING;

// int PseudoString::comparePartial(string ostr){}
static QoreValue PseudoString_comparePartial_Vs(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* ostr = HARD_QORE_VALUE_STRING(args, 0);
    const QoreStringNode* str = v.get<const QoreStringNode>();
# 959 "Pseudo_QC_String.qpp"
    // convert pattern encoding to string if necessary
    TempEncodingHelper temp(ostr, str->getEncoding(), xsink);
    if (*xsink)
        return QoreValue();

    int rc = strncmp(str->c_str(), temp->c_str(), temp->size());
    return !rc ? 0 : (rc < 0 ? -1 : 1);
}

// bool PseudoString::empty(){}
static QoreValue PseudoString_empty(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = v.get<const QoreStringNode>();
# 163 "Pseudo_QC_String.qpp"
    return str->empty();
}

// string PseudoString::encoding(){}
static QoreValue PseudoString_encoding(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = v.get<const QoreStringNode>();
# 178 "Pseudo_QC_String.qpp"
    return new QoreStringNode(str->getEncoding()->getCode());
}

// bool PseudoString::equalPartial(string ostr){}
static QoreValue PseudoString_equalPartial_Vs(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* ostr = HARD_QORE_VALUE_STRING(args, 0);
    const QoreStringNode* str = v.get<const QoreStringNode>();
# 984 "Pseudo_QC_String.qpp"
    return str->equalPartialSoft(ostr, xsink);
}

// bool PseudoString::equalPartialPath(string ostr){}
static QoreValue PseudoString_equalPartialPath_Vs(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* ostr = HARD_QORE_VALUE_STRING(args, 0);
    const QoreStringNode* str = v.get<const QoreStringNode>();
# 1002 "Pseudo_QC_String.qpp"
    return str->equalPartialPath(ostr, xsink);
}

// int PseudoString::find(softstring substr, softint pos = 0){}
static QoreValue PseudoString_find_vsvi(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* substr = HARD_QORE_VALUE_STRING(args, 0);
    int64 pos = HARD_QORE_VALUE_INT(args, 1);
    const QoreStringNode* str = v.get<const QoreStringNode>();
# 266 "Pseudo_QC_String.qpp"
    return str->index(*substr, pos, xsink);
}

// string PseudoString::getDecoded(int code = CD_ALL){}
static QoreValue PseudoString_getDecoded_Vi(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 code = HARD_QORE_VALUE_INT(args, 0);
    const QoreStringNode* str = v.get<const QoreStringNode>();
# 1181 "Pseudo_QC_String.qpp"
    QoreStringNodeHolder rv(new QoreStringNode(str->getEncoding()));
    return rv->concatDecode(xsink, *str, code) ? nullptr : rv.release();
}

// string PseudoString::getEncoded(int code = CE_XHTML){}
static QoreValue PseudoString_getEncoded_Vi(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 code = HARD_QORE_VALUE_INT(args, 0);
    const QoreStringNode* str = v.get<const QoreStringNode>();
# 1164 "Pseudo_QC_String.qpp"
    QoreStringNodeHolder rv(new QoreStringNode(str->getEncoding()));
    return rv->concatEncode(xsink, *str, code) ? nullptr : rv.release();
}

// *string PseudoString::getLine(int offset = 0, *string eol, bool trim = True, *reference<int> size){}
static QoreValue PseudoString_getLine_ViNsVbC15_reference_int_(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 offset = HARD_QORE_VALUE_INT(args, 0);
    const QoreStringNode* eol = get_param_value(args, 1).get<const QoreStringNode>();
    bool trim = HARD_QORE_VALUE_BOOL(args, 2);
    const ReferenceNode* size = get_param_value(args, 3).get<const ReferenceNode>();
    const QoreStringNode* str = v.get<const QoreStringNode>();
# 1076 "Pseudo_QC_String.qpp"
    if (str->empty())
        return QoreValue();

    if (offset < 0) {
        offset = str->size() + offset;
        if (offset < 0)
            offset = 0;
    } else if (offset >= (int64)str->size())
        return QoreValue();

    TempEncodingHelper eolstr;
    if (eol) {
        eolstr.set(eol, str->getEncoding(), xsink);
        if (*xsink)
            return QoreValue();

        if (eolstr->size() > str->size())
            return QoreValue();
    }

    int64 strsize = 0;

    const char* start = str->c_str() + offset;
    SimpleRefHolder<QoreStringNode> rv(new QoreStringNode(str->getEncoding()));
    if (eolstr) {
        const char* p = strstr(start, eolstr->c_str());
        if (p) {
            strsize = p - start + eolstr->size();
            if (trim) {
                if (p != start)
                rv->concat(start, p - start);
            } else
                rv->concat(start, strsize);
        } else {
            strsize = rv->size();
            rv->concat(start);
            strsize = rv->size() - strsize;
        }
    } else {
        while (*start) {
            if (*start == '\r') {
                ++strsize;
                if (!trim)
                rv->concat(*start);
                if (*(start + 1) == '\n') {
                ++strsize;
                if (!trim)
                    rv->concat(*start);
                }
                break;
            }
            if (*start == '\n') {
                ++strsize;
                if (!trim)
                rv->concat(*start);
                break;
            }
            rv->concat(*start);
            ++strsize;
            ++start;
        }
    }

    if (size) {
        QoreTypeSafeReferenceHelper r(size, xsink);
        if (!r)
            return QoreValue();

        if (r.assign(strsize))
            return QoreValue();
    }

    return rv.release();
}

// int PseudoString::getUnicode(int offset = 0){}
static QoreValue PseudoString_getUnicode_Vi(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 offset = HARD_QORE_VALUE_INT(args, 0);
    const QoreStringNode* str = v.get<const QoreStringNode>();
# 1039 "Pseudo_QC_String.qpp"
    return str->getUnicodePoint(offset, xsink);
}

// bool PseudoString::intp(){}
static QoreValue PseudoString_intp(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = v.get<const QoreStringNode>();
# 322 "Pseudo_QC_String.qpp"
    if (str->empty()) {
        return false;
    }
    char c = (*str)[0];
    if (c == '-') {
        c = (*str)[1];
    }
    return (isdigit(c)) ? true : false;
}

// bool PseudoString::isDataAscii(){}
static QoreValue PseudoString_isDataAscii(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = v.get<const QoreStringNode>();
# 881 "Pseudo_QC_String.qpp"
    return str->isDataAscii();
}

// bool PseudoString::isDataPrintableAscii(){}
static QoreValue PseudoString_isDataPrintableAscii(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = v.get<const QoreStringNode>();
# 896 "Pseudo_QC_String.qpp"
    return str->isDataPrintableAscii();
}

// int PseudoString::length(){}
static QoreValue PseudoString_length(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = v.get<const QoreStringNode>();
# 123 "Pseudo_QC_String.qpp"
    return str->length();
}

// string PseudoString::lwr(){}
static QoreValue PseudoString_lwr(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = v.get<const QoreStringNode>();
# 203 "Pseudo_QC_String.qpp"
    SimpleRefHolder<QoreStringNode> rv(new QoreStringNode(str->getEncoding()));
   if (do_tolower(*(*rv), *str, xsink))
      return QoreValue();

   return rv.release();
}

// bool PseudoString::regex(string regex, int options = 0){}
static QoreValue PseudoString_regex_VsVi(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* regex = HARD_QORE_VALUE_STRING(args, 0);
    int64 options = HARD_QORE_VALUE_INT(args, 1);
    const QoreStringNode* str = v.get<const QoreStringNode>();
# 599 "Pseudo_QC_String.qpp"
    QoreRegex qr(*regex, options, xsink);
    if (*xsink) {
        return QoreValue();
    }

    return qr.exec(str, xsink);
}

// *list<*string> PseudoString::regexExtract(string regex, int options = 0){}
static QoreValue PseudoString_regexExtract_VsVi(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* regex = HARD_QORE_VALUE_STRING(args, 0);
    int64 options = HARD_QORE_VALUE_INT(args, 1);
    const QoreStringNode* str = v.get<const QoreStringNode>();
# 638 "Pseudo_QC_String.qpp"
    QoreRegex qr(*regex, options, xsink);
    if (*xsink)
        return QoreValue();

    return qr.extractSubstrings(str, xsink);
}

// int PseudoString::rfind(softstring substr, softint pos = -1){}
static QoreValue PseudoString_rfind_vsvi(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* substr = HARD_QORE_VALUE_STRING(args, 0);
    int64 pos = HARD_QORE_VALUE_INT(args, 1);
    const QoreStringNode* str = v.get<const QoreStringNode>();
# 296 "Pseudo_QC_String.qpp"
    return str->rindex(*substr, (qore_offset_t)pos, xsink);
}

// int PseudoString::size(){}
static QoreValue PseudoString_size(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = v.get<const QoreStringNode>();
# 101 "Pseudo_QC_String.qpp"
    return str->size();
}

// bool PseudoString::sizep(){}
static QoreValue PseudoString_sizep(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1050 "Pseudo_QC_String.qpp"
    return true;
}

// list<string> PseudoString::split(string sep, bool with_separator = False){}
static QoreValue PseudoString_split_VsVb(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* sep = HARD_QORE_VALUE_STRING(args, 0);
    bool with_separator = HARD_QORE_VALUE_BOOL(args, 1);
    const QoreStringNode* str = v.get<const QoreStringNode>();
# 461 "Pseudo_QC_String.qpp"
    // convert pattern encoding to string if necessary
    TempEncodingHelper temp(sep, str->getEncoding(), xsink);
    if (*xsink)
        return QoreValue();

    return split_intern(temp->c_str(), temp->strlen(), str->c_str(), str->strlen(), str->getEncoding(),
        with_separator);
    return QoreValue();
}

// list<string> PseudoString::split(string sep, string quote, bool trim_unquoted = False){}
static QoreValue PseudoString_split_VsVsVb(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* sep = HARD_QORE_VALUE_STRING(args, 0);
    const QoreStringNode* quote = HARD_QORE_VALUE_STRING(args, 1);
    bool trim_unquoted = HARD_QORE_VALUE_BOOL(args, 2);
    const QoreStringNode* str = v.get<const QoreStringNode>();
# 505 "Pseudo_QC_String.qpp"
    return split_with_quote(xsink, sep, str, quote, trim_unquoted);
}

// list<string> PseudoString::splitRegex(string regex_sep, int options = 0, bool with_separator = False){}
static QoreValue PseudoString_splitRegex_VsViVb(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* regex_sep = HARD_QORE_VALUE_STRING(args, 0);
    int64 options = HARD_QORE_VALUE_INT(args, 1);
    bool with_separator = HARD_QORE_VALUE_BOOL(args, 2);
    const QoreStringNode* str = v.get<const QoreStringNode>();
# 533 "Pseudo_QC_String.qpp"
    QoreRegex qr(*regex_sep, options, xsink);
    if (*xsink) {
        return QoreValue();
    }

    return qr.extractWithPattern(*str, with_separator, xsink);
}

// list<string> PseudoString::splitRegex(string regex_sep, bool with_separator = False){}
static QoreValue PseudoString_splitRegex_VsVb(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* regex_sep = HARD_QORE_VALUE_STRING(args, 0);
    bool with_separator = HARD_QORE_VALUE_BOOL(args, 1);
    const QoreStringNode* str = v.get<const QoreStringNode>();
# 565 "Pseudo_QC_String.qpp"
    QoreRegex qr(*regex_sep, 0, xsink);
    if (*xsink) {
        return QoreValue();
    }

    return qr.extractWithPattern(*str, with_separator, xsink);
}

// int PseudoString::strlen(){}
static QoreValue PseudoString_strlen(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = v.get<const QoreStringNode>();
# 79 "Pseudo_QC_String.qpp"
    return str->strlen();
}

// bool PseudoString::strp(){}
static QoreValue PseudoString_strp(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 387 "Pseudo_QC_String.qpp"
    return true;
}

// string PseudoString::substr(softint start){}
static QoreValue PseudoString_substr_vi(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 start = HARD_QORE_VALUE_INT(args, 0);
    const QoreStringNode* str = v.get<const QoreStringNode>();
# 408 "Pseudo_QC_String.qpp"
    QoreStringNode* rv = str->substr(start, xsink);
    if (!rv)
        rv = new QoreStringNode(str->getEncoding());
    return rv;
}

// string PseudoString::substr(softint start, softint len){}
static QoreValue PseudoString_substr_vivi(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 start = HARD_QORE_VALUE_INT(args, 0);
    int64 len = HARD_QORE_VALUE_INT(args, 1);
    const QoreStringNode* str = v.get<const QoreStringNode>();
# 435 "Pseudo_QC_String.qpp"
    QoreStringNode* rv = str->substr(start, len, xsink);
    if (!rv)
        rv = new QoreStringNode(str->getEncoding());
    return rv;
}

// string PseudoString::toBase64(softint maxlinelen = -1){}
static QoreValue PseudoString_toBase64_vi(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 maxlinelen = HARD_QORE_VALUE_INT(args, 0);
    const QoreStringNode* str = v.get<const QoreStringNode>();
# 939 "Pseudo_QC_String.qpp"
    QoreStringNode *rv = new QoreStringNode;
    rv->concatBase64(str, maxlinelen);
    return rv;
}

// binary PseudoString::toBinary(){}
static QoreValue PseudoString_toBinary(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = v.get<const QoreStringNode>();
# 865 "Pseudo_QC_String.qpp"
    SimpleRefHolder<BinaryNode> b(new BinaryNode);
    b->append(str->c_str(), str->size());
    return b.release();
}

// string PseudoString::toHex(){}
static QoreValue PseudoString_toHex(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = v.get<const QoreStringNode>();
# 914 "Pseudo_QC_String.qpp"
    QoreStringNode *rv = new QoreStringNode;
    rv->concatHex(str);
    return rv;
}

// int PseudoString::toInt(int base = 10){}
static QoreValue PseudoString_toInt_Vi(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 base = HARD_QORE_VALUE_INT(args, 0);
    const QoreStringNode* str = v.get<const QoreStringNode>();
# 365 "Pseudo_QC_String.qpp"
    if (base < 0 || base == 1 || base > 36) {
        return xsink -> raiseException("INVALID-BASE", "base " QLLD " is invalid; base must be 0 or 2 - 36 inclusive",
            base);
    }
    if (!str->getEncoding()->isAsciiCompat()) {
        return xsink -> raiseException("UNSUPPORTED-ENCODING", "cannot convert string in non-ASCII-compatible "
            "encoding \"%s\" to an integer", str->getEncoding()->getCode());
    }
    return strtoll(str -> c_str(), 0, (int)base);
}

// string PseudoString::toMD5(){}
static QoreValue PseudoString_toMD5(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = v.get<const QoreStringNode>();
# 669 "Pseudo_QC_String.qpp"
    DigestHelper dh(*str);
    if (dh.doDigest(MD5_ERR, EVP_md5(), xsink))
        return QoreValue();

    return dh.getString();
}

// string PseudoString::toSHA1(){}
static QoreValue PseudoString_toSHA1(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = v.get<const QoreStringNode>();
# 697 "Pseudo_QC_String.qpp"
    DigestHelper dh(*str);
    if (dh.doDigest(SHA1_ERR, EVP_sha1(), xsink))
        return QoreValue();

    return dh.getString();
}

// string PseudoString::toSHA224(){}
static QoreValue PseudoString_toSHA224(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = v.get<const QoreStringNode>();
# 728 "Pseudo_QC_String.qpp"
    #if !defined(OPENSSL_NO_SHA256) && defined(HAVE_OPENSSL_SHA512)
    DigestHelper dh(*str);
    if (dh.doDigest(SHA224_ERR, EVP_sha224(), xsink))
        return QoreValue();

    return dh.getString();
#else
    missing_openssl_feature("SHA224", xsink);
    return QoreValue();
#endif
    return QoreValue();
}

// string PseudoString::toSHA256(){}
static QoreValue PseudoString_toSHA256(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = v.get<const QoreStringNode>();
# 764 "Pseudo_QC_String.qpp"
    #if !defined(OPENSSL_NO_SHA256) && defined(HAVE_OPENSSL_SHA512)
    DigestHelper dh(*str);
    if (dh.doDigest(SHA256_ERR, EVP_sha256(), xsink))
        return QoreValue();

    return dh.getString();
#else
    missing_openssl_feature("SHA256", xsink);
    return QoreValue();
#endif
    return QoreValue();
}

// string PseudoString::toSHA384(){}
static QoreValue PseudoString_toSHA384(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = v.get<const QoreStringNode>();
# 802 "Pseudo_QC_String.qpp"
    #if !defined(OPENSSL_NO_SHA512) && defined(HAVE_OPENSSL_SHA512)
    DigestHelper dh(*str);
    if (dh.doDigest(SHA384_ERR, EVP_sha384(), xsink))
        return QoreValue();

    return dh.getString();
#else
    missing_openssl_feature("SHA384", xsink);
    return QoreValue();
#endif
    return QoreValue();
}

// string PseudoString::toSHA512(){}
static QoreValue PseudoString_toSHA512(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = v.get<const QoreStringNode>();
# 840 "Pseudo_QC_String.qpp"
    #if !defined(OPENSSL_NO_SHA512) && defined(HAVE_OPENSSL_SHA512)
    DigestHelper dh(*str);
    if (dh.doDigest(SHA512_ERR, EVP_sha512(), xsink))
        return QoreValue();

    return dh.getString();
#else
    missing_openssl_feature("SHA512", xsink);
    return QoreValue();
#endif
    return QoreValue();
}

// int PseudoString::typeCode(){}
static QoreValue PseudoString_typeCode(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 57 "Pseudo_QC_String.qpp"
    return NT_STRING;
}

// string PseudoString::unaccent(){}
static QoreValue PseudoString_unaccent(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = v.get<const QoreStringNode>();
# 1019 "Pseudo_QC_String.qpp"
    SimpleRefHolder<QoreStringNode> rv(new QoreStringNode(str->getEncoding()));
    if (do_unaccent(*(*rv), *str, xsink))
        return QoreValue();

    return rv.release();
}

// string PseudoString::upr(){}
static QoreValue PseudoString_upr(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = v.get<const QoreStringNode>();
# 232 "Pseudo_QC_String.qpp"
    SimpleRefHolder<QoreStringNode> rv(new QoreStringNode(str->getEncoding()));
   if (do_toupper(*(*rv), *str, xsink))
      return QoreValue();

   return rv.release();
}

// bool PseudoString::val(){}
static QoreValue PseudoString_val(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = v.get<const QoreStringNode>();
# 308 "Pseudo_QC_String.qpp"
    return !str->empty();
}

// int PseudoString::width(){}
static QoreValue PseudoString_width(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = v.get<const QoreStringNode>();
# 149 "Pseudo_QC_String.qpp"
    return str->getCharWidth(xsink);
}

DLLLOCAL void preinitPseudoStringClass() {
    QC_PSEUDOSTRING = new QoreClass("<string>", "::Qore::<string>", QDOM_DEFAULT);
    CID_PSEUDOSTRING = QC_PSEUDOSTRING->getID();
    QC_PSEUDOSTRING->setSystem();
}

DLLLOCAL QoreClass* initPseudoStringClass() {
    if (!QC_PSEUDOSTRING)
        preinitPseudoStringClass();

    // set parent class
    assert(QC_PSEUDOVALUE);
    QC_PSEUDOSTRING->addBuiltinVirtualBaseClass(QC_PSEUDOVALUE);

    // int PseudoString::comparePartial(string ostr){}
    QC_PSEUDOSTRING->addMethod("comparePartial", (q_method_n_t)PseudoString_comparePartial_Vs, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, bigIntTypeInfo, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "ostr");

    // bool PseudoString::empty(){}
    QC_PSEUDOSTRING->addMethod("empty", (q_method_n_t)PseudoString_empty, Public, QCF_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

    // string PseudoString::encoding(){}
    QC_PSEUDOSTRING->addMethod("encoding", (q_method_n_t)PseudoString_encoding, Public, QCF_CONSTANT, QDOM_DEFAULT, stringTypeInfo);

    // bool PseudoString::equalPartial(string ostr){}
    QC_PSEUDOSTRING->addMethod("equalPartial", (q_method_n_t)PseudoString_equalPartial_Vs, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, boolTypeInfo, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "ostr");

    // bool PseudoString::equalPartialPath(string ostr){}
    QC_PSEUDOSTRING->addMethod("equalPartialPath", (q_method_n_t)PseudoString_equalPartialPath_Vs, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, boolTypeInfo, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "ostr");

    // int PseudoString::find(softstring substr, softint pos = 0){}
    QC_PSEUDOSTRING->addMethod("find", (q_method_n_t)PseudoString_find_vsvi, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, bigIntTypeInfo, 2, softStringTypeInfo, QORE_PARAM_NO_ARG, "substr", softBigIntTypeInfo, QoreSimpleValue().assign((int64)0), "pos");

    // string PseudoString::getDecoded(int code = CD_ALL){}
    QC_PSEUDOSTRING->addMethod("getDecoded", (q_method_n_t)PseudoString_getDecoded_Vi, Public, QCF_NO_FLAGS, QDOM_DEFAULT, stringTypeInfo, 1, bigIntTypeInfo, QoreSimpleValue().assign((int64)CD_ALL), "code");

    // string PseudoString::getEncoded(int code = CE_XHTML){}
    QC_PSEUDOSTRING->addMethod("getEncoded", (q_method_n_t)PseudoString_getEncoded_Vi, Public, QCF_NO_FLAGS, QDOM_DEFAULT, stringTypeInfo, 1, bigIntTypeInfo, QoreSimpleValue().assign((int64)CE_XHTML), "code");

    // *string PseudoString::getLine(int offset = 0, *string eol, bool trim = True, *reference<int> size){}
    QC_PSEUDOSTRING->addMethod("getLine", (q_method_n_t)PseudoString_getLine_ViNsVbC15_reference_int_, Public, QCF_CONSTANT, QDOM_DEFAULT, stringOrNothingTypeInfo, 4, bigIntTypeInfo, QoreSimpleValue().assign((int64)0), "offset", stringOrNothingTypeInfo, QORE_PARAM_NO_ARG, "eol", boolTypeInfo, QoreSimpleValue().assign(true), "trim", qore_get_complex_reference_or_nothing_type(bigIntTypeInfo), QORE_PARAM_NO_ARG, "size");

    // int PseudoString::getUnicode(int offset = 0){}
    QC_PSEUDOSTRING->addMethod("getUnicode", (q_method_n_t)PseudoString_getUnicode_Vi, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, bigIntTypeInfo, 1, bigIntTypeInfo, QoreSimpleValue().assign((int64)0), "offset");

    // bool PseudoString::intp(){}
    QC_PSEUDOSTRING->addMethod("intp", (q_method_n_t)PseudoString_intp, Public, QCF_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

    // bool PseudoString::isDataAscii(){}
    QC_PSEUDOSTRING->addMethod("isDataAscii", (q_method_n_t)PseudoString_isDataAscii, Public, QCF_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

    // bool PseudoString::isDataPrintableAscii(){}
    QC_PSEUDOSTRING->addMethod("isDataPrintableAscii", (q_method_n_t)PseudoString_isDataPrintableAscii, Public, QCF_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

    // int PseudoString::length(){}
    QC_PSEUDOSTRING->addMethod("length", (q_method_n_t)PseudoString_length, Public, QCF_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

    // string PseudoString::lwr(){}
    QC_PSEUDOSTRING->addMethod("lwr", (q_method_n_t)PseudoString_lwr, Public, QCF_CONSTANT, QDOM_DEFAULT, stringTypeInfo);

    // bool PseudoString::regex(string regex, int options = 0){}
    QC_PSEUDOSTRING->addMethod("regex", (q_method_n_t)PseudoString_regex_VsVi, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, boolTypeInfo, 2, stringTypeInfo, QORE_PARAM_NO_ARG, "regex", bigIntTypeInfo, QoreSimpleValue().assign((int64)0), "options");

    // *list<*string> PseudoString::regexExtract(string regex, int options = 0){}
    QC_PSEUDOSTRING->addMethod("regexExtract", (q_method_n_t)PseudoString_regexExtract_VsVi, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, qore_get_complex_list_or_nothing_type(stringOrNothingTypeInfo), 2, stringTypeInfo, QORE_PARAM_NO_ARG, "regex", bigIntTypeInfo, QoreSimpleValue().assign((int64)0), "options");

    // int PseudoString::rfind(softstring substr, softint pos = -1){}
    QC_PSEUDOSTRING->addMethod("rfind", (q_method_n_t)PseudoString_rfind_vsvi, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, bigIntTypeInfo, 2, softStringTypeInfo, QORE_PARAM_NO_ARG, "substr", softBigIntTypeInfo, QoreSimpleValue().assign((int64)-1), "pos");

    // int PseudoString::size(){}
    QC_PSEUDOSTRING->addMethod("size", (q_method_n_t)PseudoString_size, Public, QCF_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

    // bool PseudoString::sizep(){}
    QC_PSEUDOSTRING->addMethod("sizep", (q_method_n_t)PseudoString_sizep, Public, QCF_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

    // list<string> PseudoString::split(string sep, bool with_separator = False){}
    QC_PSEUDOSTRING->addMethod("split", (q_method_n_t)PseudoString_split_VsVb, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, qore_get_complex_list_type(stringTypeInfo), 2, stringTypeInfo, QORE_PARAM_NO_ARG, "sep", boolTypeInfo, QoreSimpleValue().assign(false), "with_separator");

    // list<string> PseudoString::split(string sep, string quote, bool trim_unquoted = False){}
    QC_PSEUDOSTRING->addMethod("split", (q_method_n_t)PseudoString_split_VsVsVb, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, qore_get_complex_list_type(stringTypeInfo), 3, stringTypeInfo, QORE_PARAM_NO_ARG, "sep", stringTypeInfo, QORE_PARAM_NO_ARG, "quote", boolTypeInfo, QoreSimpleValue().assign(false), "trim_unquoted");

    // list<string> PseudoString::splitRegex(string regex_sep, int options = 0, bool with_separator = False){}
    QC_PSEUDOSTRING->addMethod("splitRegex", (q_method_n_t)PseudoString_splitRegex_VsViVb, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, qore_get_complex_list_type(stringTypeInfo), 3, stringTypeInfo, QORE_PARAM_NO_ARG, "regex_sep", bigIntTypeInfo, QoreSimpleValue().assign((int64)0), "options", boolTypeInfo, QoreSimpleValue().assign(false), "with_separator");

    // list<string> PseudoString::splitRegex(string regex_sep, bool with_separator = False){}
    QC_PSEUDOSTRING->addMethod("splitRegex", (q_method_n_t)PseudoString_splitRegex_VsVb, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, qore_get_complex_list_type(stringTypeInfo), 2, stringTypeInfo, QORE_PARAM_NO_ARG, "regex_sep", boolTypeInfo, QoreSimpleValue().assign(false), "with_separator");

    // int PseudoString::strlen(){}
    QC_PSEUDOSTRING->addMethod("strlen", (q_method_n_t)PseudoString_strlen, Public, QCF_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

    // bool PseudoString::strp(){}
    QC_PSEUDOSTRING->addMethod("strp", (q_method_n_t)PseudoString_strp, Public, QCF_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

    // string PseudoString::substr(softint start){}
    QC_PSEUDOSTRING->addMethod("substr", (q_method_n_t)PseudoString_substr_vi, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, stringTypeInfo, 1, softBigIntTypeInfo, QORE_PARAM_NO_ARG, "start");

    // string PseudoString::substr(softint start, softint len){}
    QC_PSEUDOSTRING->addMethod("substr", (q_method_n_t)PseudoString_substr_vivi, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, stringTypeInfo, 2, softBigIntTypeInfo, QORE_PARAM_NO_ARG, "start", softBigIntTypeInfo, QORE_PARAM_NO_ARG, "len");

    // string PseudoString::toBase64(softint maxlinelen = -1){}
    QC_PSEUDOSTRING->addMethod("toBase64", (q_method_n_t)PseudoString_toBase64_vi, Public, QCF_CONSTANT, QDOM_DEFAULT, stringTypeInfo, 1, softBigIntTypeInfo, QoreSimpleValue().assign((int64)-1), "maxlinelen");

    // binary PseudoString::toBinary(){}
    QC_PSEUDOSTRING->addMethod("toBinary", (q_method_n_t)PseudoString_toBinary, Public, QCF_CONSTANT, QDOM_DEFAULT, binaryTypeInfo);

    // string PseudoString::toHex(){}
    QC_PSEUDOSTRING->addMethod("toHex", (q_method_n_t)PseudoString_toHex, Public, QCF_CONSTANT, QDOM_DEFAULT, stringTypeInfo);

    // int PseudoString::toInt(int base = 10){}
    QC_PSEUDOSTRING->addMethod("toInt", (q_method_n_t)PseudoString_toInt_Vi, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, bigIntTypeInfo, 1, bigIntTypeInfo, QoreSimpleValue().assign((int64)10), "base");

    // string PseudoString::toMD5(){}
    QC_PSEUDOSTRING->addMethod("toMD5", (q_method_n_t)PseudoString_toMD5, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, stringTypeInfo);

    // string PseudoString::toSHA1(){}
    QC_PSEUDOSTRING->addMethod("toSHA1", (q_method_n_t)PseudoString_toSHA1, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, stringTypeInfo);

    // string PseudoString::toSHA224(){}
    QC_PSEUDOSTRING->addMethod("toSHA224", (q_method_n_t)PseudoString_toSHA224, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, stringTypeInfo);

    // string PseudoString::toSHA256(){}
    QC_PSEUDOSTRING->addMethod("toSHA256", (q_method_n_t)PseudoString_toSHA256, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, stringTypeInfo);

    // string PseudoString::toSHA384(){}
    QC_PSEUDOSTRING->addMethod("toSHA384", (q_method_n_t)PseudoString_toSHA384, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, stringTypeInfo);

    // string PseudoString::toSHA512(){}
    QC_PSEUDOSTRING->addMethod("toSHA512", (q_method_n_t)PseudoString_toSHA512, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, stringTypeInfo);

    // int PseudoString::typeCode(){}
    QC_PSEUDOSTRING->addMethod("typeCode", (q_method_n_t)PseudoString_typeCode, Public, QCF_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

    // string PseudoString::unaccent(){}
    QC_PSEUDOSTRING->addMethod("unaccent", (q_method_n_t)PseudoString_unaccent, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, stringTypeInfo);

    // string PseudoString::upr(){}
    QC_PSEUDOSTRING->addMethod("upr", (q_method_n_t)PseudoString_upr, Public, QCF_CONSTANT, QDOM_DEFAULT, stringTypeInfo);

    // bool PseudoString::val(){}
    QC_PSEUDOSTRING->addMethod("val", (q_method_n_t)PseudoString_val, Public, QCF_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

    // int PseudoString::width(){}
    QC_PSEUDOSTRING->addMethod("width", (q_method_n_t)PseudoString_width, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, bigIntTypeInfo);

    return QC_PSEUDOSTRING;
}
