/* -*- mode: c++; indent-tabs-mode: nil -*- */
/*
    QC_File.qpp

    Qore Programming Language

    Copyright (C) 2003 - 2023 Qore Technologies, s.r.o.

    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
    to deal in the Software without restriction, including without limitation
    the rights to use, copy, modify, merge, publish, distribute, sublicense,
    and/or sell copies of the Software, and to permit persons to whom the
    Software is furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
    DEALINGS IN THE SOFTWARE.

    Note that the Qore library is released under a choice of three open-source
    licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
    information.
*/

#include <qore/Qore.h>
#include "qore/intern/QC_File.h"
#include "qore/intern/QC_Queue.h"
#include "qore/intern/QC_TermIOS.h"

#include <unistd.h>

extern QoreClass* QC_READONLYFILE;

#ifndef TCSANOW
#define TCSANOW 0
#endif

QoreFileHelper::QoreFileHelper(QoreObject* obj, ExceptionSink* xs) : QorePrivateObjectAccessHelper(xs) {
    File* f = reinterpret_cast<File*>(obj->getReferencedPrivateData(CID_FILE, xs));
    if (!f) {
        if (!*xsink)
            xsink->raiseException("FILE-ERROR", "expecting an object derived from File; got class '%s' instead", obj->getClassName());
        return;
    }
    ptr = (void*)f;
}

QoreFileHelper::~QoreFileHelper() {
   if (ptr)
      reinterpret_cast<File*>(ptr)->deref(xsink);
}

QoreFile* QoreFileHelper::operator*() const {
   return reinterpret_cast<File*>(ptr);
}

QoreFile* QoreFileHelper::operator->() const {
   return reinterpret_cast<File*>(ptr);
}

static void FILE_system_constructor(QoreObject* self, int fd, va_list args) {
    //printd(5, "FILE_system_constructor() self=%p, descriptor=%d\n", self, fd);
    File* f = new File(QCS_DEFAULT);
    f->makeSpecial(fd);
    self->setPrivate(CID_FILE, f);
#ifdef _Q_WINDOWS
    // we have to set binary mode on stdin/stdout/stderr so that \r\n is not automatically
    // translated to \n causing breakage on Windows
    // note that this only has to be done once per descriptor, meaning that the fix here
    // addresses stdin, stdout, stderr as well as stdin_stream, stdout_stream, and
    // stderr_stream
    _setmode(fd, O_BINARY);
#endif
}

#ifdef HAVE_STRUCT_FLOCK
static int lock_intern(struct flock& fl, const QoreListNode* args, ExceptionSink* xsink) {
    fl.l_type = (short)HARD_QORE_VALUE_INT(args, 0);
    fl.l_start = (off_t)HARD_QORE_VALUE_INT(args, 1);
    fl.l_len = (off_t)HARD_QORE_VALUE_INT(args, 2);
    if (fl.l_len < 0) {
        xsink->raiseException("FILE-LOCK-ERROR", "length of locked area cannot be negative (value passed=%d)", fl.l_len);
        return -1;
    }
    fl.l_whence = (short)HARD_QORE_VALUE_INT(args, 3);
    return 0;
}
#else
static int file_lock_error(const char* f, ExceptionSink* xsink) {
    xsink->raiseException("MISSING-FEATURE-ERROR", "this platform does not support UNIX-style file locking with fnctl(), therefore the File::%s() method is not available; for maximum portability, check Option::HAVE_FILE_LOCKING before calling this method", f);
    return 0;
}
#ifndef F_RDLCK
#define F_RDLCK 0
#endif
#ifndef F_UNLCK
#define F_UNLCK 0
#endif
#ifndef F_WRLCK
#define F_WRLCK 0
#endif
#endif

AbstractQoreNode* missing_method_error(const char* meth, const char* feat, ExceptionSink* xsink) {
    xsink->raiseException("MISSING-FEATURE-ERROR", "the %s() method is not available on this build; for maximum portability, check Option::HAVE_%s before calling this method", meth, feat);
    return nullptr;
}
TypedHashDecl* init_hashdecl_PipeInfo(QoreNamespace& ns) {
    TypedHashDecl* hd = new TypedHashDecl("PipeInfo", "::Qore::PipeInfo");
    hd->addMember("read", QC_READONLYFILE->getTypeInfo(), QoreValue());
    hd->addMember("write", QC_FILE->getTypeInfo(), QoreValue());
    ns.addSystemHashDecl(hd);
    return hd;
}

/* Qore class Qore::File */

qore_classid_t CID_FILE;
QoreClass* QC_FILE;

// nothing File::chown(softint uid, softint gid = -1){}
static QoreValue File_chown_vivi(QoreObject* self, File* f, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 uid = HARD_QORE_VALUE_INT(args, 0);
    int64 gid = HARD_QORE_VALUE_INT(args, 1);
# 972 "QC_File.qpp"
    #ifdef HAVE_CHOWN
   if (check_terminal_io(self, "File::chown", xsink))
      return QoreValue();

   f->chown( (uid_t)uid, (gid_t)gid, xsink);
   return QoreValue();
#else
   return missing_method_error("File::chown", "UNIX_FILEMGT", xsink);
#endif
    return QoreValue();
}

// File::constructor(*string encoding) {}
static void File_constructor_Ns(QoreObject* self, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* encoding = get_param_value(args, 0).get<const QoreStringNode>();
# 255 "QC_File.qpp"
const QoreEncoding *qe = encoding ? QEM.findCreate(encoding) : QCS_DEFAULT;

   SimpleRefHolder<File> f(new File(qe));
   if (f->isTty()) {
      xsink->raiseException("ILLEGAL-EXPRESSION", "File::constructor() cannot be called with a TTY target when 'no-terminal-io' is set");
      return;
   }

   self->setPrivate(CID_FILE, f.release());
}

// File::copy() {}
static void File_copy(QoreObject* self, QoreObject* old, File* f, ExceptionSink* xsink) {
# 281 "QC_File.qpp"
self->setPrivate(CID_FILE, new File(f->getEncoding()));
}

// int File::f_printf(string fmt, ...){}
static QoreValue File_f_printf_VsVV(QoreObject* self, File* f, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 739 "QC_File.qpp"
    if (check_terminal_io(self, "File::f_printf", xsink))
      return QoreValue();

   QoreStringNodeHolder str(q_sprintf(args, 1, 0, xsink));
   if (!str)
      return QoreValue();

   return f->write(*str, xsink);
}

// int File::f_printf(){}
static QoreValue File_f_printf(QoreObject* self, File* f, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 753 "QC_File.qpp"
    return 0;
}

// int File::f_vprintf(string fmt, any fmt_args){}
static QoreValue File_f_vprintf_VsVa(QoreObject* self, File* f, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 780 "QC_File.qpp"
    if (check_terminal_io(self, "File::f_vprintf", xsink))
      return QoreValue();

   QoreStringNodeHolder str(q_vsprintf(args, 1, 0, xsink));
   if (!str)
      return QoreValue();

   return f->write(*str, xsink);
}

// int File::f_vprintf(){}
static QoreValue File_f_vprintf(QoreObject* self, File* f, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 794 "QC_File.qpp"
    return 0;
}

// string File::getCharset(){}
static QoreValue File_getCharset(QoreObject* self, File* f, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 817 "QC_File.qpp"
    return new QoreStringNode(f->getEncoding()->getCode());
}

// hash File::getLockInfo(){}
static QoreValue File_getLockInfo(QoreObject* self, File* f, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 933 "QC_File.qpp"
    #ifdef HAVE_STRUCT_FLOCK
    if (check_terminal_io(self, "File::getLockInfo", xsink))
        return QoreValue();

    struct flock fl;
    if (f->getLockInfo(fl, xsink))
        return QoreValue();

    QoreHashNode *h = new QoreHashNode(autoTypeInfo);
    h->setKeyValue("start", fl.l_start, xsink);
    h->setKeyValue("len", fl.l_len, xsink);
    h->setKeyValue("pid", fl.l_pid, xsink);
    h->setKeyValue("type", fl.l_type, xsink);
    h->setKeyValue("whence", fl.l_whence, xsink);

    return h;
#else
    file_lock_error("getLockInfo", xsink);
    return QoreValue();
#endif
    return QoreValue();
}

// nothing File::getTerminalAttributes(TermIOS termios){}
static QoreValue File_getTerminalAttributes_C7TermIOS(QoreObject* self, File* f, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    HARD_QORE_VALUE_OBJ_DATA(termios, QoreTermIOS, args, 0, CID_TERMIOS, "File::getTerminalAttributes()", "TermIOS", xsink);
    if (*xsink)
        return 0;
# 1003 "QC_File.qpp"
    #ifdef HAVE_TERMIOS_H
   ReferenceHolder<QoreTermIOS> holder(termios, xsink);
   if (check_terminal_io(self, "File::getTerminalAttributes", xsink))
      return QoreValue();

   f->getTerminalAttributes(termios, xsink);
   return QoreValue();
#else
   return missing_method_error("File::getTerminalAttributes", "TERMIOS", xsink);
#endif
    return QoreValue();
}

// TermIOS File::getTerminalAttributes(){}
static QoreValue File_getTerminalAttributes(QoreObject* self, File* f, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1034 "QC_File.qpp"
    #ifdef HAVE_TERMIOS_H
   if (check_terminal_io(self, "File::getTerminalAttributes", xsink))
      return QoreValue();

   ReferenceHolder<QoreTermIOS> holder(new QoreTermIOS, xsink);
   f->getTerminalAttributes(*holder, xsink);
   if (*xsink)
      return QoreValue();
   return new QoreObject(QC_TERMIOS, getProgram(), holder.release());
#else
   return (QoreObject*)missing_method_error("File::getTerminalAttributes", "TERMIOS", xsink);
#endif
    return QoreValue();
}

// int File::lock(softint type = F_RDLCK, softint start = 0, softint len = 0, softint whence = SEEK_SET){}
static QoreValue File_lock_vivivivi(QoreObject* self, File* f, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 852 "QC_File.qpp"
    #ifdef HAVE_STRUCT_FLOCK
   if (check_terminal_io(self, "lock", xsink))
      return QoreValue();

   struct flock fl;
   if (lock_intern(fl, args, xsink))
      return QoreValue();
   return f->lock(fl, xsink);
#else
   file_lock_error("lock", xsink);
   return QoreValue();
#endif
    return QoreValue();
}

// nothing File::lockBlocking(softint type = F_RDLCK, softint start = 0, softint len = 0, softint whence = SEEK_SET){}
static QoreValue File_lockBlocking_vivivivi(QoreObject* self, File* f, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 896 "QC_File.qpp"
    #ifdef HAVE_STRUCT_FLOCK
   if (check_terminal_io(self, "File::lockBlocking", xsink))
      return QoreValue();

   struct flock fl;
   if (lock_intern(fl, args, xsink))
      return QoreValue();

   f->lockBlocking(fl, xsink);
   return QoreValue();
#else
   file_lock_error("lockBlocking", xsink);
   return QoreValue();
#endif
    return QoreValue();
}

// int File::open(string path, softint flags = O_RDONLY, softint mode = 0666, *string encoding){}
static QoreValue File_open_VsviviNs(QoreObject* self, File* f, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* path = HARD_QORE_VALUE_STRING(args, 0);
    int64 flags = HARD_QORE_VALUE_INT(args, 1);
    int64 mode = HARD_QORE_VALUE_INT(args, 2);
    const QoreStringNode* encoding = get_param_value(args, 3).get<const QoreStringNode>();
# 313 "QC_File.qpp"
    if (check_terminal_io(self, "File::open", xsink))
      return QoreValue();

   const QoreEncoding *qe = encoding ? QEM.findCreate(encoding) : f->getEncoding();
   return f->open(path->getBuffer(), flags, mode, qe);
}

// nothing File::open2(string path, softint flags = O_RDONLY, softint mode = 0666, *string encoding){}
static QoreValue File_open2_VsviviNs(QoreObject* self, File* f, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* path = HARD_QORE_VALUE_STRING(args, 0);
    int64 flags = HARD_QORE_VALUE_INT(args, 1);
    int64 mode = HARD_QORE_VALUE_INT(args, 2);
    const QoreStringNode* encoding = get_param_value(args, 3).get<const QoreStringNode>();
# 354 "QC_File.qpp"
    if (check_terminal_io(self, "File::open2", xsink))
      return QoreValue();

   const QoreEncoding *qe = encoding ? QEM.findCreate(encoding) : f->getEncoding();
   f->open2(xsink, path->getBuffer(), flags, mode, qe);
    return QoreValue();
}

// int File::print(string data){}
static QoreValue File_print_Vs(QoreObject* self, File* f, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* data = HARD_QORE_VALUE_STRING(args, 0);
# 454 "QC_File.qpp"
    if (check_terminal_io(self, "File::print", xsink))
      return QoreValue();

   return f->write(data, xsink);
}

// int File::printf(string fmt, ...){}
static QoreValue File_printf_VsVV(QoreObject* self, File* f, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 658 "QC_File.qpp"
    if (check_terminal_io(self, "File::printf", xsink))
      return QoreValue();

   QoreStringNodeHolder str(q_sprintf(args, 0, 0, xsink));
   if (!str)
      return QoreValue();

   return f->write(*str, xsink);
}

// int File::printf(){}
static QoreValue File_printf(QoreObject* self, File* f, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 672 "QC_File.qpp"
    return 0;
}

// nothing File::redirect(Qore::File file){}
static QoreValue File_redirect_C4File(QoreObject* self, File* f, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    HARD_QORE_VALUE_OBJ_DATA(file, File, args, 0, CID_FILE, "File::redirect()", "Qore::File", xsink);
    if (*xsink)
        return 0;
# 1120 "QC_File.qpp"
    ReferenceHolder<File> holder(file, xsink);

    if (check_terminal_io(self, "File::redirect", xsink) || check_terminal_io(obj_file, "File::redirect", xsink))
        return QoreValue();

    f->redirect(*file, xsink);
    return QoreValue();
}

// nothing File::setCharset(*string encoding){}
static QoreValue File_setCharset_Ns(QoreObject* self, File* f, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* encoding = get_param_value(args, 0).get<const QoreStringNode>();
# 804 "QC_File.qpp"
    if (check_terminal_io(self, "File::setCharset", xsink))
      return QoreValue();

   const QoreEncoding *qe = encoding ? QEM.findCreate(encoding) : QCS_DEFAULT;
   f->setEncoding(qe);
    return QoreValue();
}

// nothing File::setTerminalAttributes(softint action = TCSANOW, TermIOS termios){}
static QoreValue File_setTerminalAttributes_viC7TermIOS(QoreObject* self, File* f, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 action = HARD_QORE_VALUE_INT(args, 0);
    HARD_QORE_VALUE_OBJ_DATA(termios, QoreTermIOS, args, 1, CID_TERMIOS, "File::setTerminalAttributes()", "TermIOS", xsink);
    if (*xsink)
        return 0;
# 1087 "QC_File.qpp"
    #ifdef HAVE_TERMIOS_H
   ReferenceHolder<QoreTermIOS> holder(termios, xsink);
   if (check_terminal_io(self, "File::setTerminalAttributes", xsink))
      return QoreValue();

   f->setTerminalAttributes(action, termios, xsink);
   return QoreValue();
#else
   return missing_method_error("File::setTerminalAttributes", "TERMIOS", xsink);
#endif
    return QoreValue();
}

// int File::sync(){}
static QoreValue File_sync(QoreObject* self, File* f, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 373 "QC_File.qpp"
    if (check_terminal_io(self, "File::sync", xsink))
      return QoreValue();

   return f->sync();
}

// int File::vprintf(string fmt, any fmt_args){}
static QoreValue File_vprintf_VsVa(QoreObject* self, File* f, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 699 "QC_File.qpp"
    if (check_terminal_io(self, "File::vprintf", xsink))
      return QoreValue();

   QoreStringNodeHolder str(q_vsprintf(args, 0, 0, xsink));
   if (!str)
      return QoreValue();

   return f->write(*str, xsink);
}

// int File::vprintf(){}
static QoreValue File_vprintf(QoreObject* self, File* f, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 713 "QC_File.qpp"
    return 0;
}

// int File::write(binary data){}
static QoreValue File_write_Vy(QoreObject* self, File* f, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const BinaryNode* data = HARD_QORE_VALUE_BINARY(args, 0);
# 398 "QC_File.qpp"
    if (check_terminal_io(self, "File::write", xsink))
      return QoreValue();

   return f->write(data, xsink);
}

// int File::write(string data){}
static QoreValue File_write_Vs(QoreObject* self, File* f, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* data = HARD_QORE_VALUE_STRING(args, 0);
# 426 "QC_File.qpp"
    if (check_terminal_io(self, "File::write", xsink))
      return QoreValue();

   return f->write(data, xsink);
}

// int File::writei1(int c){}
static QoreValue File_writei1_Vi(QoreObject* self, File* f, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 c = HARD_QORE_VALUE_INT(args, 0);
# 479 "QC_File.qpp"
    if (check_terminal_io(self, "File::writei1", xsink))
      return QoreValue();

   return f->writei1(c, xsink);
}

// int File::writei2(int s){}
static QoreValue File_writei2_Vi(QoreObject* self, File* f, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 s = HARD_QORE_VALUE_INT(args, 0);
# 504 "QC_File.qpp"
    if (check_terminal_io(self, "File::writei2", xsink))
      return QoreValue();

   return f->writei2(s, xsink);
}

// int File::writei2LSB(int s){}
static QoreValue File_writei2LSB_Vi(QoreObject* self, File* f, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 s = HARD_QORE_VALUE_INT(args, 0);
# 579 "QC_File.qpp"
    if (check_terminal_io(self, "File::writei2LSB", xsink))
      return QoreValue();

   return f->writei2LSB(s, xsink);
}

// int File::writei4(int i){}
static QoreValue File_writei4_Vi(QoreObject* self, File* f, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 i = HARD_QORE_VALUE_INT(args, 0);
# 529 "QC_File.qpp"
    if (check_terminal_io(self, "File::writei4", xsink))
      return QoreValue();

   return f->writei4(i, xsink);
}

// int File::writei4LSB(int i){}
static QoreValue File_writei4LSB_Vi(QoreObject* self, File* f, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 i = HARD_QORE_VALUE_INT(args, 0);
# 604 "QC_File.qpp"
    if (check_terminal_io(self, "File::writei4LSB", xsink))
      return QoreValue();

   return f->writei4LSB(i, xsink);
}

// int File::writei8(int i){}
static QoreValue File_writei8_Vi(QoreObject* self, File* f, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 i = HARD_QORE_VALUE_INT(args, 0);
# 554 "QC_File.qpp"
    if (check_terminal_io(self, "File::writei8", xsink))
      return QoreValue();

   return f->writei8(i, xsink);
}

// int File::writei8LSB(int i){}
static QoreValue File_writei8LSB_Vi(QoreObject* self, File* f, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 i = HARD_QORE_VALUE_INT(args, 0);
# 629 "QC_File.qpp"
    if (check_terminal_io(self, "File::writei8LSB", xsink))
      return QoreValue();

   return f->writei8LSB(i, xsink);
}

// static hash<PipeInfo> File::getPipe(){}
static QoreValue static_File_getPipe(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1136 "QC_File.qpp"
#ifndef _Q_WINDOWS
    int pipefd[2];
    if (pipe(pipefd)) {
        xsink->raiseErrnoException("PIPE-ERROR", errno, "failed to create pipe");
        return QoreValue();
    }

    ReferenceHolder<QoreHashNode> rv(new QoreHashNode(hashdeclPipeInfo, xsink), xsink);

    ReferenceHolder<File> read(new File(QCS_DEFAULT), xsink);
    read->makeSpecial(pipefd[0]);
    ReferenceHolder<QoreObject> readobj(new QoreObject(QC_READONLYFILE, getProgram(), read.release()), xsink);

    ReferenceHolder<File> write(new File(QCS_DEFAULT), xsink);
    write->makeSpecial(pipefd[1]);
    ReferenceHolder<QoreObject> writeobj(new QoreObject(QC_FILE, getProgram(), write.release()), xsink);

    rv->setKeyValue("read", readobj.release(), xsink);
    rv->setKeyValue("write", writeobj.release(), xsink);

    return rv.release();
#else
    return missing_method_error("File::pipe", "UNIX_FILEMGT", xsink);
#endif
    return QoreValue();
}

DLLLOCAL void preinitFileClass() {
    QC_FILE = new QoreClass("File", "::Qore::File", QDOM_FILESYSTEM);
    CID_FILE = QC_FILE->getID();
    QC_FILE->setSystem();
}

DLLLOCAL QoreClass* initFileClass(QoreNamespace& ns) {
    if (!QC_FILE)
        preinitFileClass();

    // set parent class
    assert(QC_READONLYFILE);
    QC_FILE->addBuiltinVirtualBaseClass(QC_READONLYFILE);

    // set system constructor
    QC_FILE->setSystemConstructor(FILE_system_constructor);

    // nothing File::chown(softint uid, softint gid = -1){}
    QC_FILE->addMethod("chown", (q_method_n_t)File_chown_vivi, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 2, softBigIntTypeInfo, QORE_PARAM_NO_ARG, "uid", softBigIntTypeInfo, QoreSimpleValue().assign((int64)-1), "gid");

    // File::constructor(*string encoding) {}
    QC_FILE->addConstructor(File_constructor_Ns, Public, QCF_NO_FLAGS, QDOM_DEFAULT, 1, stringOrNothingTypeInfo, QORE_PARAM_NO_ARG, "encoding");

    // File::copy() {}
    QC_FILE->setCopy((q_copy_t)File_copy);

    // int File::f_printf(string fmt, ...){}
    QC_FILE->addMethod("f_printf", (q_method_n_t)File_f_printf_VsVV, Public, QCF_USES_EXTRA_ARGS, QDOM_DEFAULT, bigIntTypeInfo, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "fmt");

    // int File::f_printf(){}
    QC_FILE->addMethod("f_printf", (q_method_n_t)File_f_printf, Public, QCF_RUNTIME_NOOP, QDOM_DEFAULT, bigIntTypeInfo);

    // int File::f_vprintf(string fmt, any fmt_args){}
    QC_FILE->addMethod("f_vprintf", (q_method_n_t)File_f_vprintf_VsVa, Public, QCF_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo, 2, stringTypeInfo, QORE_PARAM_NO_ARG, "fmt", anyTypeInfo, QORE_PARAM_NO_ARG, "fmt_args");

    // int File::f_vprintf(){}
    QC_FILE->addMethod("f_vprintf", (q_method_n_t)File_f_vprintf, Public, QCF_RUNTIME_NOOP, QDOM_DEFAULT, bigIntTypeInfo);

    // string File::getCharset(){}
    QC_FILE->addMethod("getCharset", (q_method_n_t)File_getCharset, Public, QCF_CONSTANT, QDOM_DEFAULT, stringTypeInfo);

    // hash File::getLockInfo(){}
    QC_FILE->addMethod("getLockInfo", (q_method_n_t)File_getLockInfo, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, hashTypeInfo);

    // nothing File::getTerminalAttributes(TermIOS termios){}
    QC_FILE->addMethod("getTerminalAttributes", (q_method_n_t)File_getTerminalAttributes_C7TermIOS, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, QC_TERMIOS->getTypeInfo(), QORE_PARAM_NO_ARG, "termios");

    // TermIOS File::getTerminalAttributes(){}
    QC_FILE->addMethod("getTerminalAttributes", (q_method_n_t)File_getTerminalAttributes, Public, QCF_NO_FLAGS, QDOM_DEFAULT, QC_TERMIOS->getTypeInfo());

    // int File::lock(softint type = F_RDLCK, softint start = 0, softint len = 0, softint whence = SEEK_SET){}
    QC_FILE->addMethod("lock", (q_method_n_t)File_lock_vivivivi, Public, QCF_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo, 4, softBigIntTypeInfo, QoreSimpleValue().assign((int64)F_RDLCK), "type", softBigIntTypeInfo, QoreSimpleValue().assign((int64)0), "start", softBigIntTypeInfo, QoreSimpleValue().assign((int64)0), "len", softBigIntTypeInfo, QoreSimpleValue().assign((int64)SEEK_SET), "whence");

    // nothing File::lockBlocking(softint type = F_RDLCK, softint start = 0, softint len = 0, softint whence = SEEK_SET){}
    QC_FILE->addMethod("lockBlocking", (q_method_n_t)File_lockBlocking_vivivivi, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 4, softBigIntTypeInfo, QoreSimpleValue().assign((int64)F_RDLCK), "type", softBigIntTypeInfo, QoreSimpleValue().assign((int64)0), "start", softBigIntTypeInfo, QoreSimpleValue().assign((int64)0), "len", softBigIntTypeInfo, QoreSimpleValue().assign((int64)SEEK_SET), "whence");

    // int File::open(string path, softint flags = O_RDONLY, softint mode = 0666, *string encoding){}
    QC_FILE->addMethod("open", (q_method_n_t)File_open_VsviviNs, Public, QCF_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo, 4, stringTypeInfo, QORE_PARAM_NO_ARG, "path", softBigIntTypeInfo, QoreSimpleValue().assign((int64)O_RDONLY), "flags", softBigIntTypeInfo, QoreSimpleValue().assign((int64)0666), "mode", stringOrNothingTypeInfo, QORE_PARAM_NO_ARG, "encoding");

    // nothing File::open2(string path, softint flags = O_RDONLY, softint mode = 0666, *string encoding){}
    QC_FILE->addMethod("open2", (q_method_n_t)File_open2_VsviviNs, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 4, stringTypeInfo, QORE_PARAM_NO_ARG, "path", softBigIntTypeInfo, QoreSimpleValue().assign((int64)O_RDONLY), "flags", softBigIntTypeInfo, QoreSimpleValue().assign((int64)0666), "mode", stringOrNothingTypeInfo, QORE_PARAM_NO_ARG, "encoding");

    // int File::print(string data){}
    QC_FILE->addMethod("print", (q_method_n_t)File_print_Vs, Public, QCF_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "data");

    // int File::printf(string fmt, ...){}
    QC_FILE->addMethod("printf", (q_method_n_t)File_printf_VsVV, Public, QCF_USES_EXTRA_ARGS, QDOM_DEFAULT, bigIntTypeInfo, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "fmt");

    // int File::printf(){}
    QC_FILE->addMethod("printf", (q_method_n_t)File_printf, Public, QCF_RUNTIME_NOOP, QDOM_DEFAULT, bigIntTypeInfo);

    // nothing File::redirect(Qore::File file){}
    QC_FILE->addMethod("redirect", (q_method_n_t)File_redirect_C4File, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, QC_FILE->getTypeInfo(), QORE_PARAM_NO_ARG, "file");

    // nothing File::setCharset(*string encoding){}
    QC_FILE->addMethod("setCharset", (q_method_n_t)File_setCharset_Ns, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, stringOrNothingTypeInfo, QORE_PARAM_NO_ARG, "encoding");

    // nothing File::setTerminalAttributes(softint action = TCSANOW, TermIOS termios){}
    QC_FILE->addMethod("setTerminalAttributes", (q_method_n_t)File_setTerminalAttributes_viC7TermIOS, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 2, softBigIntTypeInfo, QoreSimpleValue().assign((int64)TCSANOW), "action", QC_TERMIOS->getTypeInfo(), QORE_PARAM_NO_ARG, "termios");

    // int File::sync(){}
    QC_FILE->addMethod("sync", (q_method_n_t)File_sync, Public, QCF_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo);

    // int File::vprintf(string fmt, any fmt_args){}
    QC_FILE->addMethod("vprintf", (q_method_n_t)File_vprintf_VsVa, Public, QCF_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo, 2, stringTypeInfo, QORE_PARAM_NO_ARG, "fmt", anyTypeInfo, QORE_PARAM_NO_ARG, "fmt_args");

    // int File::vprintf(){}
    QC_FILE->addMethod("vprintf", (q_method_n_t)File_vprintf, Public, QCF_RUNTIME_NOOP, QDOM_DEFAULT, bigIntTypeInfo);

    // int File::write(binary data){}
    QC_FILE->addMethod("write", (q_method_n_t)File_write_Vy, Public, QCF_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo, 1, binaryTypeInfo, QORE_PARAM_NO_ARG, "data");

    // int File::write(string data){}
    QC_FILE->addMethod("write", (q_method_n_t)File_write_Vs, Public, QCF_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "data");

    // int File::writei1(int c){}
    QC_FILE->addMethod("writei1", (q_method_n_t)File_writei1_Vi, Public, QCF_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo, 1, bigIntTypeInfo, QORE_PARAM_NO_ARG, "c");

    // int File::writei2(int s){}
    QC_FILE->addMethod("writei2", (q_method_n_t)File_writei2_Vi, Public, QCF_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo, 1, bigIntTypeInfo, QORE_PARAM_NO_ARG, "s");

    // int File::writei2LSB(int s){}
    QC_FILE->addMethod("writei2LSB", (q_method_n_t)File_writei2LSB_Vi, Public, QCF_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo, 1, bigIntTypeInfo, QORE_PARAM_NO_ARG, "s");

    // int File::writei4(int i){}
    QC_FILE->addMethod("writei4", (q_method_n_t)File_writei4_Vi, Public, QCF_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo, 1, bigIntTypeInfo, QORE_PARAM_NO_ARG, "i");

    // int File::writei4LSB(int i){}
    QC_FILE->addMethod("writei4LSB", (q_method_n_t)File_writei4LSB_Vi, Public, QCF_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo, 1, bigIntTypeInfo, QORE_PARAM_NO_ARG, "i");

    // int File::writei8(int i){}
    QC_FILE->addMethod("writei8", (q_method_n_t)File_writei8_Vi, Public, QCF_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo, 1, bigIntTypeInfo, QORE_PARAM_NO_ARG, "i");

    // int File::writei8LSB(int i){}
    QC_FILE->addMethod("writei8LSB", (q_method_n_t)File_writei8LSB_Vi, Public, QCF_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo, 1, bigIntTypeInfo, QORE_PARAM_NO_ARG, "i");

    // static hash<PipeInfo> File::getPipe(){}
    QC_FILE->addStaticMethod("getPipe", (q_func_n_t)static_File_getPipe, Public, QCF_NO_FLAGS, QDOM_DEFAULT, hashdeclPipeInfo->getTypeInfo(false));
# 130 "QC_File.qpp"
// the following constants were defined earlier but are system-specific; they are included in the build
// in case anyone is using them on the platforms where they are supported
#ifndef O_NOCTTY
#define O_NOCTTY 0
#endif
#ifndef O_NONBLOCK
#define O_NONBLOCK 0
#endif
#ifndef O_NDELAY
#define O_NDELAY 0
#endif
#ifndef O_SYNC
#define O_SYNC 0
#endif
#ifndef O_DIRECT
#define O_DIRECT 0
#endif
#ifndef O_DIRECTORY
#define O_DIRECTORY 0
#endif
#ifndef O_NOFOLLOW
#define O_NOFOLLOW 0
#endif
    ns.addConstant("O_ACCMODE", ((int64)O_ACCMODE));
    ns.addConstant("O_APPEND", ((int64)O_APPEND));
    ns.addConstant("O_CREAT", ((int64)O_CREAT));
    ns.addConstant("O_DIRECT", ((int64)O_DIRECT));
    ns.addConstant("O_DIRECTORY", ((int64)O_DIRECTORY));
    ns.addConstant("O_EXCL", ((int64)O_EXCL));
    ns.addConstant("O_NDELAY", ((int64)O_NDELAY));
    ns.addConstant("O_NOCTTY", ((int64)O_NOCTTY));
    ns.addConstant("O_NOFOLLOW", ((int64)O_NOFOLLOW));
    ns.addConstant("O_NONBLOCK", ((int64)O_NONBLOCK));
    ns.addConstant("O_RDONLY", ((int64)O_RDONLY));
    ns.addConstant("O_RDWR", ((int64)O_RDWR));
    ns.addConstant("O_SYNC", ((int64)O_SYNC));
    ns.addConstant("O_TRUNC", ((int64)O_TRUNC));
    ns.addConstant("O_WRONLY", ((int64)O_WRONLY));
    ns.addConstant("F_RDLCK", ((int64)F_RDLCK));
    ns.addConstant("F_UNLCK", ((int64)F_UNLCK));
    ns.addConstant("F_WRLCK", ((int64)F_WRLCK));
    ns.addConstant("SEEK_CUR", ((int64)SEEK_CUR));
    ns.addConstant("SEEK_END", ((int64)SEEK_END));
    ns.addConstant("SEEK_SET", ((int64)SEEK_SET));

    return QC_FILE;
}
