/* -*- mode: c++; indent-tabs-mode: nil -*- */
/*
    QC_HTTPClient.qpp

    Qore Programming Language

    Copyright (C) 2006 - 2023 Qore Technologies, s.r.o.

    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
    to deal in the Software without restriction, including without limitation
    the rights to use, copy, modify, merge, publish, distribute, sublicense,
    and/or sell copies of the Software, and to permit persons to whom the
    Software is furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
    DEALINGS IN THE SOFTWARE.

    Note that the Qore library is released under a choice of three open-source
    licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
    information.
*/

#include <qore/Qore.h>
#include <qore/QoreHttpClientObject.h>
#include "qore/intern/QC_HTTPClient.h"
#include "qore/intern/QC_Queue.h"
#include "qore/intern/QC_SocketPollOperation.h"
#include "qore/intern/ssl_constants.h"
#include "qore/minitest.hpp"
#include "qore/QoreSSLCertificate.h"
#include "qore/QoreSSLPrivateKey.h"

extern QoreClass* QC_SOCKET;

#ifdef DEBUG_TESTS
#  include "tests/QC_HTTPClient_tests.cpp"
#endif
/* Qore class Qore::HTTPClient */

qore_classid_t CID_HTTPCLIENT;
QoreClass* QC_HTTPCLIENT;

// nothing HTTPClient::addDefaultHeaders(hash<auto> hdr){}
static QoreValue HTTPClient_addDefaultHeaders_C10hash_auto_(QoreObject* self, QoreHttpClientObject* client, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreHashNode* hdr = HARD_QORE_VALUE_HASH(args, 0);
# 1740 "QC_HTTPClient.qpp"
    client->addDefaultHeaders(hdr);
    return QoreValue();
}

// nothing HTTPClient::clearProxyURL(){}
static QoreValue HTTPClient_clearProxyURL(QoreObject* self, QoreHttpClientObject* client, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1468 "QC_HTTPClient.qpp"
    client->clearProxyURL();
    return QoreValue();
}

// nothing HTTPClient::clearProxyUserPassword(){}
static QoreValue HTTPClient_clearProxyUserPassword(QoreObject* self, QoreHttpClientObject* client, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1729 "QC_HTTPClient.qpp"
    client->clearProxyUserPassword();
    return QoreValue();
}

// nothing HTTPClient::clearStats(){}
static QoreValue HTTPClient_clearStats(QoreObject* self, QoreHttpClientObject* client, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1905 "QC_HTTPClient.qpp"
    client->clearStats();
    return QoreValue();
}

// nothing HTTPClient::clearUserPassword(){}
static QoreValue HTTPClient_clearUserPassword(QoreObject* self, QoreHttpClientObject* client, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1677 "QC_HTTPClient.qpp"
    client->clearUserPassword();
    return QoreValue();
}

// nothing HTTPClient::clearWarningQueue(){}
static QoreValue HTTPClient_clearWarningQueue(QoreObject* self, QoreHttpClientObject* client, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1827 "QC_HTTPClient.qpp"
    client->clearWarningQueue(xsink);
    return QoreValue();
}

// nothing HTTPClient::connect(){}
static QoreValue HTTPClient_connect(QoreObject* self, QoreHttpClientObject* client, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 387 "QC_HTTPClient.qpp"
    client->connect(xsink);
    return QoreValue();
}

// HTTPClient::constructor(hash<auto> opts) {}
static void HTTPClient_constructor_C10hash_auto_(QoreObject* self, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreHashNode* opts = HARD_QORE_VALUE_HASH(args, 0);
# 263 "QC_HTTPClient.qpp"
ReferenceHolder<QoreHttpClientObject> client(new QoreHttpClientObject, xsink);
    if (client->setOptions(opts, xsink))
        return;

    self->setPrivate(CID_HTTPCLIENT, client.release());
}

// HTTPClient::constructor() {}
static void HTTPClient_constructor(QoreObject* self, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 278 "QC_HTTPClient.qpp"
self->setPrivate(CID_HTTPCLIENT, new QoreHttpClientObject);
}

// HTTPClient::copy() {}
static void HTTPClient_copy(QoreObject* self, QoreObject* old, QoreHttpClientObject* client, ExceptionSink* xsink) {
# 286 "QC_HTTPClient.qpp"
xsink->raiseException("HTTPCLIENT-COPY-ERROR", "copying HTTPClient objects is not yet supported");
}

// HTTPClient::destructor() {}
static void HTTPClient_destructor(QoreObject* self, QoreHttpClientObject* client, ExceptionSink* xsink) {
# 297 "QC_HTTPClient.qpp"
// have to clear callbacks before destroying
    client->cleanup(xsink);
    client->deref(xsink);
}

// nothing HTTPClient::disconnect(){}
static QoreValue HTTPClient_disconnect(QoreObject* self, QoreHttpClientObject* client, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 398 "QC_HTTPClient.qpp"
    client->disconnect();
    return QoreValue();
}

// *string HTTPClient::get(string path, *hash<auto> headers, *reference<hash<auto>> info){}
static QoreValue HTTPClient_get_VsC11_hash_auto_C22_reference_hash_auto__(QoreObject* self, QoreHttpClientObject* client, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* path = HARD_QORE_VALUE_STRING(args, 0);
    const QoreHashNode* headers = get_param_value(args, 1).get<const QoreHashNode>();
    const ReferenceNode* info = get_param_value(args, 2).get<const ReferenceNode>();
# 1108 "QC_HTTPClient.qpp"
    OptHashRefHelper ohrh(info, xsink);
    ReferenceHolder<AbstractQoreNode> rv(client->get(path->c_str(), headers, *ohrh, xsink), xsink);
    return *xsink ? nullptr : rv.release();
}

// string HTTPClient::getAssumedEncoding(){}
static QoreValue HTTPClient_getAssumedEncoding(QoreObject* self, QoreHttpClientObject* client, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 2077 "QC_HTTPClient.qpp"
    return client->getAssumedEncoding();
}

// int HTTPClient::getConnectTimeout(){}
static QoreValue HTTPClient_getConnectTimeout(QoreObject* self, QoreHttpClientObject* client, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1578 "QC_HTTPClient.qpp"
    return client->getConnectTimeout();
}

// *string HTTPClient::getConnectionPath(){}
static QoreValue HTTPClient_getConnectionPath(QoreObject* self, QoreHttpClientObject* client, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1794 "QC_HTTPClient.qpp"
    const char* p = client->getConnectionPath();
   return p ? new QoreStringNode(p) : nullptr;
}

// hash<string,string> HTTPClient::getDefaultHeaders(){}
static QoreValue HTTPClient_getDefaultHeaders(QoreObject* self, QoreHttpClientObject* client, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1751 "QC_HTTPClient.qpp"
    return client->getDefaultHeaders();
}

// string HTTPClient::getDefaultPath(){}
static QoreValue HTTPClient_getDefaultPath(QoreObject* self, QoreHttpClientObject* client, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1779 "QC_HTTPClient.qpp"
    const char* dp = client->getDefaultPath();
   return dp ? new QoreStringNode(dp) : nullptr;
}

// string HTTPClient::getEncoding(){}
static QoreValue HTTPClient_getEncoding(QoreObject* self, QoreHttpClientObject* client, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1336 "QC_HTTPClient.qpp"
    return new QoreStringNode(client->getEncoding()->getCode());
}

// bool HTTPClient::getEncodingPassthru(){}
static QoreValue HTTPClient_getEncodingPassthru(QoreObject* self, QoreHttpClientObject* client, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 2034 "QC_HTTPClient.qpp"
    return client->getEncodingPassthru();
}

// bool HTTPClient::getErrorPassthru(){}
static QoreValue HTTPClient_getErrorPassthru(QoreObject* self, QoreHttpClientObject* client, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1962 "QC_HTTPClient.qpp"
    return client->getErrorPassthru();
}

// string HTTPClient::getHTTPVersion(){}
static QoreValue HTTPClient_getHTTPVersion(QoreObject* self, QoreHttpClientObject* client, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 334 "QC_HTTPClient.qpp"
    return new QoreStringNode(client->getHTTPVersion());
}

// string HTTPClient::getHostHeaderValue(){}
static QoreValue HTTPClient_getHostHeaderValue(QoreObject* self, QoreHttpClientObject* client, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 2048 "QC_HTTPClient.qpp"
    return client->getHostHeaderValue();
}

// int HTTPClient::getMaxRedirects(){}
static QoreValue HTTPClient_getMaxRedirects(QoreObject* self, QoreHttpClientObject* client, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1519 "QC_HTTPClient.qpp"
    return client->getMaxRedirects();
}

// bool HTTPClient::getNoDelay(){}
static QoreValue HTTPClient_getNoDelay(QoreObject* self, QoreHttpClientObject* client, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1613 "QC_HTTPClient.qpp"
    return client->getNoDelay();
}

// bool HTTPClient::getPreEncodedUrls(){}
static QoreValue HTTPClient_getPreEncodedUrls(QoreObject* self, QoreHttpClientObject* client, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 2117 "QC_HTTPClient.qpp"
    return client->getPreEncodedUrls();
}

// *string HTTPClient::getProxyURL(){}
static QoreValue HTTPClient_getProxyURL(QoreObject* self, QoreHttpClientObject* client, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1440 "QC_HTTPClient.qpp"
    return client->getProxyURL();
}

// bool HTTPClient::getRedirectPassthru(){}
static QoreValue HTTPClient_getRedirectPassthru(QoreObject* self, QoreHttpClientObject* client, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1997 "QC_HTTPClient.qpp"
    return client->getRedirectPassthru();
}

// *string HTTPClient::getSafeProxyURL(){}
static QoreValue HTTPClient_getSafeProxyURL(QoreObject* self, QoreHttpClientObject* client, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1457 "QC_HTTPClient.qpp"
    return client->getSafeProxyURL();
}

// *string HTTPClient::getSafeURL(){}
static QoreValue HTTPClient_getSafeURL(QoreObject* self, QoreHttpClientObject* client, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1393 "QC_HTTPClient.qpp"
    return client->getSafeURL();
}

// int HTTPClient::getTimeout(){}
static QoreValue HTTPClient_getTimeout(QoreObject* self, QoreHttpClientObject* client, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1312 "QC_HTTPClient.qpp"
    return client->getTimeout();
}

// *string HTTPClient::getURL(){}
static QoreValue HTTPClient_getURL(QoreObject* self, QoreHttpClientObject* client, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1376 "QC_HTTPClient.qpp"
    return client->getURL();
}

// hash<auto> HTTPClient::getUsageInfo(){}
static QoreValue HTTPClient_getUsageInfo(QoreObject* self, QoreHttpClientObject* client, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1891 "QC_HTTPClient.qpp"
    return client->getUsageInfo();
}

// hash<auto> HTTPClient::head(string path, *hash<auto> headers, *reference<hash<auto>> info){}
static QoreValue HTTPClient_head_VsC11_hash_auto_C22_reference_hash_auto__(QoreObject* self, QoreHttpClientObject* client, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* path = HARD_QORE_VALUE_STRING(args, 0);
    const QoreHashNode* headers = get_param_value(args, 1).get<const QoreHashNode>();
    const ReferenceNode* info = get_param_value(args, 2).get<const ReferenceNode>();
# 1163 "QC_HTTPClient.qpp"
    OptHashRefHelper ohrh(info, xsink);
    ReferenceHolder<QoreHashNode> rv(client->head(path->c_str(), headers, *ohrh, xsink), xsink);
    return *xsink ? nullptr : rv.release();
}

// bool HTTPClient::isConnected(){}
static QoreValue HTTPClient_isConnected(QoreObject* self, QoreHttpClientObject* client, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1625 "QC_HTTPClient.qpp"
    return client->isConnected();
}

// bool HTTPClient::isProxySecure(){}
static QoreValue HTTPClient_isProxySecure(QoreObject* self, QoreHttpClientObject* client, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1493 "QC_HTTPClient.qpp"
    return client->isProxySecure();
}

// bool HTTPClient::isSecure(){}
static QoreValue HTTPClient_isSecure(QoreObject* self, QoreHttpClientObject* client, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 363 "QC_HTTPClient.qpp"
    return client->isSecure();
}

// *string HTTPClient::post(string path, string body, *hash<auto> headers, *reference<hash<auto>> info){}
static QoreValue HTTPClient_post_VsVsC11_hash_auto_C22_reference_hash_auto__(QoreObject* self, QoreHttpClientObject* client, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* path = HARD_QORE_VALUE_STRING(args, 0);
    const QoreStringNode* body = HARD_QORE_VALUE_STRING(args, 1);
    const QoreHashNode* headers = get_param_value(args, 2).get<const QoreHashNode>();
    const ReferenceNode* info = get_param_value(args, 3).get<const ReferenceNode>();
# 1222 "QC_HTTPClient.qpp"
    OptHashRefHelper ohrh(info, xsink);
    ReferenceHolder<AbstractQoreNode> rv(client->post(path->c_str(), headers, *body, *ohrh, xsink), xsink);
#ifdef DEBUG
    assert(!rv || rv->getType() == NT_STRING);
#endif
    return *xsink ? QoreValue() : rv.release();
}

// *string HTTPClient::post(string path, *binary body, *hash<auto> headers, *reference<hash<auto>> info){}
static QoreValue HTTPClient_post_VsNyC11_hash_auto_C22_reference_hash_auto__(QoreObject* self, QoreHttpClientObject* client, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* path = HARD_QORE_VALUE_STRING(args, 0);
    const BinaryNode* body = get_param_value(args, 1).get<const BinaryNode>();
    const QoreHashNode* headers = get_param_value(args, 2).get<const QoreHashNode>();
    const ReferenceNode* info = get_param_value(args, 3).get<const ReferenceNode>();
# 1283 "QC_HTTPClient.qpp"
    OptHashRefHelper ohrh(info, xsink);
    ReferenceHolder<AbstractQoreNode> rv(client->post(path->c_str(), headers, body ? body->getPtr() : 0, body ? body->size() : 0, *ohrh, xsink), xsink);
#ifdef DEBUG
    assert(!rv || rv->getType() == NT_STRING);
#endif
    return *xsink ? nullptr : rv.release();
}

// hash<auto> HTTPClient::send(string body, string method, *string path, *hash<auto> headers, softbool getbody = False, *reference<hash<auto>> info){}
static QoreValue HTTPClient_send_VsVsNsC11_hash_auto_vbC22_reference_hash_auto__(QoreObject* self, QoreHttpClientObject* client, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* body = HARD_QORE_VALUE_STRING(args, 0);
    const QoreStringNode* method = HARD_QORE_VALUE_STRING(args, 1);
    const QoreStringNode* path = get_param_value(args, 2).get<const QoreStringNode>();
    const QoreHashNode* headers = get_param_value(args, 3).get<const QoreHashNode>();
    bool getbody = HARD_QORE_VALUE_BOOL(args, 4);
    const ReferenceNode* info = get_param_value(args, 5).get<const ReferenceNode>();
# 548 "QC_HTTPClient.qpp"
    OptHashRefHelper ohrh(info, xsink);
    ReferenceHolder<QoreHashNode> rv(client->send(method->c_str(), path && !path->empty() ? path->c_str() : nullptr,
        headers, *body, getbody, *ohrh, xsink), xsink);
    return *xsink ? QoreValue() : rv.release();
}

// hash<auto> HTTPClient::send(*binary body, string method, *string path, *hash<auto> headers, softbool getbody = False, *reference<hash<auto>> info){}
static QoreValue HTTPClient_send_NyVsNsC11_hash_auto_vbC22_reference_hash_auto__(QoreObject* self, QoreHttpClientObject* client, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const BinaryNode* body = get_param_value(args, 0).get<const BinaryNode>();
    const QoreStringNode* method = HARD_QORE_VALUE_STRING(args, 1);
    const QoreStringNode* path = get_param_value(args, 2).get<const QoreStringNode>();
    const QoreHashNode* headers = get_param_value(args, 3).get<const QoreHashNode>();
    bool getbody = HARD_QORE_VALUE_BOOL(args, 4);
    const ReferenceNode* info = get_param_value(args, 5).get<const ReferenceNode>();
# 609 "QC_HTTPClient.qpp"
    OptHashRefHelper ohrh(info, xsink);
    ReferenceHolder<QoreHashNode> rv(client->send(method->c_str(), path && !path->empty() ? path->c_str() : nullptr,
        headers, body ? body->getPtr() : nullptr, body ? body->size() : 0, getbody, *ohrh, xsink), xsink);
    return *xsink ? QoreValue() : rv.release();
}

// nothing HTTPClient::send(Qore::OutputStream os, *data body, string method, *string path, *hash<auto> headers, timeout timeout_ms = 0, softbool getbody = False, *reference<hash<auto>> info, *code rcb){}
static QoreValue HTTPClient_send_C12OutputStreamNDVsNsC11_hash_auto_VtvbC22_reference_hash_auto__Nc(QoreObject* self, QoreHttpClientObject* client, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    HARD_QORE_VALUE_OBJ_DATA(os, OutputStream, args, 0, CID_OUTPUTSTREAM, "HTTPClient::send()", "Qore::OutputStream", xsink);
    if (*xsink)
        return 0;
    const AbstractQoreNode* body = get_param_value(args, 1).get<const AbstractQoreNode>();
    const QoreStringNode* method = HARD_QORE_VALUE_STRING(args, 2);
    const QoreStringNode* path = get_param_value(args, 3).get<const QoreStringNode>();
    const QoreHashNode* headers = get_param_value(args, 4).get<const QoreHashNode>();
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 5);
    bool getbody = HARD_QORE_VALUE_BOOL(args, 6);
    const ReferenceNode* info = get_param_value(args, 7).get<const ReferenceNode>();
    const ResolvedCallReferenceNode* rcb = get_param_value(args, 8).get<const ResolvedCallReferenceNode>();
# 882 "QC_HTTPClient.qpp"
    SimpleRefHolder<OutputStream> osHolder(os);
    OptHashRefHelper ohrh(info, xsink);
    const char* pstr = path && !path->empty() ? path->c_str() : nullptr;
    if (get_node_type(body) == NT_STRING) {
        client->sendWithOutputStream(method->c_str(), pstr, headers, *static_cast<const QoreStringNode*>(body), getbody,
            *ohrh, timeout_ms, rcb, self, os, xsink);
    } else {
        const BinaryNode* b = reinterpret_cast<const BinaryNode*>(body);
        client->sendWithOutputStream(method->c_str(), pstr, headers, b ? b->getPtr() : nullptr, b ? b->size() : 0,
            getbody, *ohrh, timeout_ms, rcb, self, os, xsink);
    }
    return QoreValue();
}

// nothing HTTPClient::sendChunked(Qore::OutputStream os, Qore::InputStream is, string method, int max_chunk_size = 4096, *string path, *hash<auto> headers, timeout timeout_ms = 0, softbool getbody = False, *reference<hash<auto>> info, *code rcb, *code tcb){}
static QoreValue HTTPClient_sendChunked_C12OutputStreamC11InputStreamVsViNsC11_hash_auto_VtvbC22_reference_hash_auto__NcNc(QoreObject* self, QoreHttpClientObject* client, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    HARD_QORE_VALUE_OBJ_DATA(os, OutputStream, args, 0, CID_OUTPUTSTREAM, "HTTPClient::sendChunked()", "Qore::OutputStream", xsink);
    if (*xsink)
        return 0;
    HARD_QORE_VALUE_OBJ_DATA(is, InputStream, args, 1, CID_INPUTSTREAM, "HTTPClient::sendChunked()", "Qore::InputStream", xsink);
    if (*xsink)
        return 0;
    const QoreStringNode* method = HARD_QORE_VALUE_STRING(args, 2);
    int64 max_chunk_size = HARD_QORE_VALUE_INT(args, 3);
    const QoreStringNode* path = get_param_value(args, 4).get<const QoreStringNode>();
    const QoreHashNode* headers = get_param_value(args, 5).get<const QoreHashNode>();
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 6);
    bool getbody = HARD_QORE_VALUE_BOOL(args, 7);
    const ReferenceNode* info = get_param_value(args, 8).get<const ReferenceNode>();
    const ResolvedCallReferenceNode* rcb = get_param_value(args, 9).get<const ResolvedCallReferenceNode>();
    const ResolvedCallReferenceNode* tcb = get_param_value(args, 10).get<const ResolvedCallReferenceNode>();
# 965 "QC_HTTPClient.qpp"
        SimpleRefHolder<OutputStream> osHolder(os);
    SimpleRefHolder<InputStream> isHolder(is);
    if (max_chunk_size < 1) {
        xsink->raiseException("HTTP-CHUNK-ERROR", "the max chunk size must be greater than 0; value given is: " QLLD,
            max_chunk_size);
        return QoreValue();
    }
    OptHashRefHelper ohrh(info, xsink);
    client->sendChunked(method->c_str(), path && !path->empty() ? path->c_str() : nullptr, headers, getbody, *ohrh, timeout_ms, rcb, self, os, is, (size_t)max_chunk_size, tcb, xsink);
    return QoreValue();
}

// nothing HTTPClient::sendWithCallbacks(code scb, code rcb, string method, *string path, *hash<auto> headers, timeout timeout_ms = 0, softbool getbody = False, *reference<hash<auto>> info){}
static QoreValue HTTPClient_sendWithCallbacks_VcVcVsNsC11_hash_auto_VtvbC22_reference_hash_auto__(QoreObject* self, QoreHttpClientObject* client, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const ResolvedCallReferenceNode* scb = get_param_value(args, 0).get<const ResolvedCallReferenceNode>();
    const ResolvedCallReferenceNode* rcb = get_param_value(args, 1).get<const ResolvedCallReferenceNode>();
    const QoreStringNode* method = HARD_QORE_VALUE_STRING(args, 2);
    const QoreStringNode* path = get_param_value(args, 3).get<const QoreStringNode>();
    const QoreHashNode* headers = get_param_value(args, 4).get<const QoreHashNode>();
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 5);
    bool getbody = HARD_QORE_VALUE_BOOL(args, 6);
    const ReferenceNode* info = get_param_value(args, 7).get<const ReferenceNode>();
# 1047 "QC_HTTPClient.qpp"
        OptHashRefHelper ohrh(info, xsink);
    client->sendWithCallbacks(method->c_str(), path && !path->empty() ? path->c_str() : nullptr, headers, scb,
        getbody, *ohrh, timeout_ms, rcb, self, xsink);
    return QoreValue();
}

// nothing HTTPClient::sendWithRecvCallback(code rcb, string body, string method, *string path, *hash<auto> headers, timeout timeout_ms = 0, softbool getbody = False, *reference<hash<auto>> info){}
static QoreValue HTTPClient_sendWithRecvCallback_VcVsVsNsC11_hash_auto_VtvbC22_reference_hash_auto__(QoreObject* self, QoreHttpClientObject* client, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const ResolvedCallReferenceNode* rcb = get_param_value(args, 0).get<const ResolvedCallReferenceNode>();
    const QoreStringNode* body = HARD_QORE_VALUE_STRING(args, 1);
    const QoreStringNode* method = HARD_QORE_VALUE_STRING(args, 2);
    const QoreStringNode* path = get_param_value(args, 3).get<const QoreStringNode>();
    const QoreHashNode* headers = get_param_value(args, 4).get<const QoreHashNode>();
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 5);
    bool getbody = HARD_QORE_VALUE_BOOL(args, 6);
    const ReferenceNode* info = get_param_value(args, 7).get<const ReferenceNode>();
# 747 "QC_HTTPClient.qpp"
    OptHashRefHelper ohrh(info, xsink);
    // issue 1813: convert string to socket's character encoding as documented
    TempEncodingHelper btmp(body, client->getEncoding(), xsink);
    if (*xsink)
        return QoreValue();
    client->sendWithRecvCallback(method->c_str(), path && !path->empty() ? path->c_str() : 0, headers, btmp->c_str(), btmp->size(), getbody, *ohrh, timeout_ms, rcb, self, xsink);
    return QoreValue();
}

// nothing HTTPClient::sendWithRecvCallback(code rcb, *binary body, string method, *string path, *hash<auto> headers, timeout timeout_ms = 0, softbool getbody = False, *reference<hash<auto>> info){}
static QoreValue HTTPClient_sendWithRecvCallback_VcNyVsNsC11_hash_auto_VtvbC22_reference_hash_auto__(QoreObject* self, QoreHttpClientObject* client, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const ResolvedCallReferenceNode* rcb = get_param_value(args, 0).get<const ResolvedCallReferenceNode>();
    const BinaryNode* body = get_param_value(args, 1).get<const BinaryNode>();
    const QoreStringNode* method = HARD_QORE_VALUE_STRING(args, 2);
    const QoreStringNode* path = get_param_value(args, 3).get<const QoreStringNode>();
    const QoreHashNode* headers = get_param_value(args, 4).get<const QoreHashNode>();
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 5);
    bool getbody = HARD_QORE_VALUE_BOOL(args, 6);
    const ReferenceNode* info = get_param_value(args, 7).get<const ReferenceNode>();
# 819 "QC_HTTPClient.qpp"
    OptHashRefHelper ohrh(info, xsink);
    client->sendWithRecvCallback(method->c_str(), path && !path->empty() ? path->c_str() : nullptr, headers, body ? body->getPtr() : nullptr, body ? body->size() : 0, getbody, *ohrh, timeout_ms, rcb, self, xsink);
    return QoreValue();
}

// hash<auto> HTTPClient::sendWithSendCallback(code scb, string method, *string path, *hash<auto> headers, timeout timeout_ms = 0, softbool getbody = False, *reference<hash> info){}
static QoreValue HTTPClient_sendWithSendCallback_VcVsNsC11_hash_auto_VtvbC16_reference_hash_(QoreObject* self, QoreHttpClientObject* client, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const ResolvedCallReferenceNode* scb = get_param_value(args, 0).get<const ResolvedCallReferenceNode>();
    const QoreStringNode* method = HARD_QORE_VALUE_STRING(args, 1);
    const QoreStringNode* path = get_param_value(args, 2).get<const QoreStringNode>();
    const QoreHashNode* headers = get_param_value(args, 3).get<const QoreHashNode>();
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 4);
    bool getbody = HARD_QORE_VALUE_BOOL(args, 5);
    const ReferenceNode* info = get_param_value(args, 6).get<const ReferenceNode>();
# 676 "QC_HTTPClient.qpp"
    OptHashRefHelper ohrh(info, xsink);
    ReferenceHolder<QoreHashNode> rv(client->sendWithSendCallback(method->c_str(), path && !path->empty() ? path->c_str() : 0, headers, scb, getbody, *ohrh, timeout_ms, xsink), xsink);
    return *xsink ? QoreValue() : rv.release();
}

// nothing HTTPClient::setAssumedEncoding(*string encoding){}
static QoreValue HTTPClient_setAssumedEncoding_Ns(QoreObject* self, QoreHttpClientObject* client, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* encoding = get_param_value(args, 0).get<const QoreStringNode>();
# 2062 "QC_HTTPClient.qpp"
    client->setAssumedEncoding(encoding && !encoding->empty() ? encoding->c_str() : "ISO-8859-1");
    return QoreValue();
}

// nothing HTTPClient::setConnectTimeout(timeout timeout_ms = -1){}
static QoreValue HTTPClient_setConnectTimeout_Vt(QoreObject* self, QoreHttpClientObject* client, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 0);
# 1566 "QC_HTTPClient.qpp"
    client->setConnectTimeout((int)timeout_ms);
    return QoreValue();
}

// nothing HTTPClient::setConnectionPath(*string uri_path){}
static QoreValue HTTPClient_setConnectionPath_Ns(QoreObject* self, QoreHttpClientObject* client, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* uri_path = get_param_value(args, 0).get<const QoreStringNode>();
# 1809 "QC_HTTPClient.qpp"
    TempEncodingHelper path;
    if (uri_path && !path.set(uri_path, QCS_DEFAULT, xsink)) {
        return QoreValue();
    }
    client->setConnectionPath(uri_path ? path->c_str() : nullptr);
    return QoreValue();
}

// nothing HTTPClient::setDefaultPath(*string path){}
static QoreValue HTTPClient_setDefaultPath_Ns(QoreObject* self, QoreHttpClientObject* client, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* path = get_param_value(args, 0).get<const QoreStringNode>();
# 1763 "QC_HTTPClient.qpp"
    TempEncodingHelper npath;
    if (path && !npath.set(path, QCS_DEFAULT, xsink)) {
        return QoreValue();
    }
    client->setDefaultPath(path ? npath->c_str() : nullptr);
    return QoreValue();
}

// nothing HTTPClient::setEncoding(string encoding){}
static QoreValue HTTPClient_setEncoding_Vs(QoreObject* self, QoreHttpClientObject* client, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* encoding = HARD_QORE_VALUE_STRING(args, 0);
# 1324 "QC_HTTPClient.qpp"
    client->setEncoding(QEM.findCreate(encoding));
    return QoreValue();
}

// bool HTTPClient::setEncodingPassthru(bool set = True){}
static QoreValue HTTPClient_setEncodingPassthru_Vb(QoreObject* self, QoreHttpClientObject* client, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    bool set = HARD_QORE_VALUE_BOOL(args, 0);
# 2017 "QC_HTTPClient.qpp"
    return client->setEncodingPassthru(set);
}

// bool HTTPClient::setErrorPassthru(bool set = True){}
static QoreValue HTTPClient_setErrorPassthru_Vb(QoreObject* self, QoreHttpClientObject* client, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    bool set = HARD_QORE_VALUE_BOOL(args, 0);
# 1944 "QC_HTTPClient.qpp"
    return client->setErrorPassthru(set);
}

// nothing HTTPClient::setEventQueue(){}
static QoreValue HTTPClient_setEventQueue(QoreObject* self, QoreHttpClientObject* client, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1530 "QC_HTTPClient.qpp"
    client->setEventQueue(xsink, nullptr, QoreValue(), false);
    return QoreValue();
}

// nothing HTTPClient::setEventQueue(Qore::Thread::Queue queue, auto arg, *bool with_data){}
static QoreValue HTTPClient_setEventQueue_C5QueueVANb(QoreObject* self, QoreHttpClientObject* client, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    HARD_QORE_VALUE_OBJ_DATA(queue, Queue, args, 0, CID_QUEUE, "HTTPClient::setEventQueue()", "Qore::Thread::Queue", xsink);
    if (*xsink)
        return 0;
    QoreValue arg = get_param_value(args, 1);
    bool with_data = HARD_QORE_VALUE_BOOL(args, 2);
# 1550 "QC_HTTPClient.qpp"
    ReferenceHolder<Queue> q(queue, xsink);
    if (queue->getMax() != -1)
        return xsink->raiseException("QUEUE-ERROR", "the Queue object passed as a maximum size of %d entr%s, which could cause internal I/O to freeze in the HttpClient object if used; use a Queue object with no maximum size for I/O monitoring", queue->getMax(), queue->getMax() == 1 ? "y" : "ies");
    // pass reference from QoreObject::getReferencedPrivateData() to function
    client->setEventQueue(xsink, q.release(), arg.refSelf(), with_data);
    return QoreValue();
}

// nothing HTTPClient::setHTTPVersion(string ver){}
static QoreValue HTTPClient_setHTTPVersion_Vs(QoreObject* self, QoreHttpClientObject* client, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* ver = HARD_QORE_VALUE_STRING(args, 0);
# 322 "QC_HTTPClient.qpp"
    client->setHTTPVersion(ver->c_str(), xsink);
    return QoreValue();
}

// nothing HTTPClient::setMaxRedirects(softint mr = 0){}
static QoreValue HTTPClient_setMaxRedirects_vi(QoreObject* self, QoreHttpClientObject* client, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 mr = HARD_QORE_VALUE_INT(args, 0);
# 1507 "QC_HTTPClient.qpp"
    client->setMaxRedirects((int)mr);
    return QoreValue();
}

// int HTTPClient::setNoDelay(softbool b = True){}
static QoreValue HTTPClient_setNoDelay_vb(QoreObject* self, QoreHttpClientObject* client, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    bool b = HARD_QORE_VALUE_BOOL(args, 0);
# 1600 "QC_HTTPClient.qpp"
    return client->setNoDelay(b);
}

// nothing HTTPClient::setPersistent(){}
static QoreValue HTTPClient_setPersistent(QoreObject* self, QoreHttpClientObject* client, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1922 "QC_HTTPClient.qpp"
    client->setPersistent(xsink);
    return QoreValue();
}

// bool HTTPClient::setPreEncodedUrls(bool set = True){}
static QoreValue HTTPClient_setPreEncodedUrls_Vb(QoreObject* self, QoreHttpClientObject* client, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    bool set = HARD_QORE_VALUE_BOOL(args, 0);
# 2099 "QC_HTTPClient.qpp"
    return client->setPreEncodedUrls(set);
}

// nothing HTTPClient::setProxySecure(softbool b = True){}
static QoreValue HTTPClient_setProxySecure_vb(QoreObject* self, QoreHttpClientObject* client, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    bool b = HARD_QORE_VALUE_BOOL(args, 0);
# 1481 "QC_HTTPClient.qpp"
    client->setProxySecure(b);
    return QoreValue();
}

// nothing HTTPClient::setProxyURL(){}
static QoreValue HTTPClient_setProxyURL(QoreObject* self, QoreHttpClientObject* client, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1405 "QC_HTTPClient.qpp"
    client->clearProxyURL();
    return QoreValue();
}

// nothing HTTPClient::setProxyURL(string url){}
static QoreValue HTTPClient_setProxyURL_Vs(QoreObject* self, QoreHttpClientObject* client, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* url = HARD_QORE_VALUE_STRING(args, 0);
# 1422 "QC_HTTPClient.qpp"
    TempEncodingHelper nurl(url, QCS_DEFAULT, xsink);
    if (*xsink) {
        return QoreValue();
    }
    client->setProxyURL(nurl->c_str(), xsink);
    return QoreValue();
}

// nothing HTTPClient::setProxyUserPassword(string user, string pass){}
static QoreValue HTTPClient_setProxyUserPassword_VsVs(QoreObject* self, QoreHttpClientObject* client, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* user = HARD_QORE_VALUE_STRING(args, 0);
    const QoreStringNode* pass = HARD_QORE_VALUE_STRING(args, 1);
# 1692 "QC_HTTPClient.qpp"
    TempEncodingHelper nuser(user, QCS_DEFAULT, xsink);
    if (*xsink) {
        return QoreValue();
    }
    TempEncodingHelper npass(pass, QCS_DEFAULT, xsink);
    if (*xsink) {
        return QoreValue();
    }

    client->setProxyUserPassword(nuser->c_str(), npass->c_str());
    return QoreValue();
}

// nothing HTTPClient::setProxyUserPassword(){}
static QoreValue HTTPClient_setProxyUserPassword(QoreObject* self, QoreHttpClientObject* client, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1717 "QC_HTTPClient.qpp"
    client->clearProxyUserPassword();
    return QoreValue();
}

// bool HTTPClient::setRedirectPassthru(bool set = True){}
static QoreValue HTTPClient_setRedirectPassthru_Vb(QoreObject* self, QoreHttpClientObject* client, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    bool set = HARD_QORE_VALUE_BOOL(args, 0);
# 1980 "QC_HTTPClient.qpp"
    return client->setRedirectPassthru(set);
}

// nothing HTTPClient::setSecure(softbool secure = True){}
static QoreValue HTTPClient_setSecure_vb(QoreObject* self, QoreHttpClientObject* client, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    bool secure = HARD_QORE_VALUE_BOOL(args, 0);
# 350 "QC_HTTPClient.qpp"
    client->setSecure(secure);
    return QoreValue();
}

// nothing HTTPClient::setTimeout(timeout timeout_ms = 0){}
static QoreValue HTTPClient_setTimeout_Vt(QoreObject* self, QoreHttpClientObject* client, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 0);
# 1300 "QC_HTTPClient.qpp"
    client->setTimeout((int)timeout_ms);
    return QoreValue();
}

// nothing HTTPClient::setURL(string url){}
static QoreValue HTTPClient_setURL_Vs(QoreObject* self, QoreHttpClientObject* client, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* url = HARD_QORE_VALUE_STRING(args, 0);
# 1357 "QC_HTTPClient.qpp"
    TempEncodingHelper nurl(url, QCS_DEFAULT, xsink);
    if (*xsink) {
        return QoreValue();
    }
    client->setURL(nurl->c_str(), xsink);
    return QoreValue();
}

// nothing HTTPClient::setUserPassword(string user, string pass){}
static QoreValue HTTPClient_setUserPassword_VsVs(QoreObject* self, QoreHttpClientObject* client, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* user = HARD_QORE_VALUE_STRING(args, 0);
    const QoreStringNode* pass = HARD_QORE_VALUE_STRING(args, 1);
# 1640 "QC_HTTPClient.qpp"
    TempEncodingHelper nuser(user, QCS_DEFAULT, xsink);
    if (*xsink) {
        return QoreValue();
    }
    TempEncodingHelper npass(pass, QCS_DEFAULT, xsink);
    if (*xsink) {
        return QoreValue();
    }

    client->setUserPassword(nuser->c_str(), npass->c_str());
    return QoreValue();
}

// nothing HTTPClient::setUserPassword(){}
static QoreValue HTTPClient_setUserPassword(QoreObject* self, QoreHttpClientObject* client, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1665 "QC_HTTPClient.qpp"
    client->clearUserPassword();
    return QoreValue();
}

// nothing HTTPClient::setWarningQueue(int warning_ms, int warning_bs, Queue queue, auto arg, timeout min_ms = 1s){}
static QoreValue HTTPClient_setWarningQueue_ViViC5QueueVAVt(QoreObject* self, QoreHttpClientObject* client, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 warning_ms = HARD_QORE_VALUE_INT(args, 0);
    int64 warning_bs = HARD_QORE_VALUE_INT(args, 1);
    HARD_QORE_VALUE_OBJ_DATA(queue, Queue, args, 2, CID_QUEUE, "HTTPClient::setWarningQueue()", "Queue", xsink);
    if (*xsink)
        return 0;
    QoreValue arg = get_param_value(args, 3);
    int64 min_ms = HARD_QORE_VALUE_INT(args, 4);
# 1864 "QC_HTTPClient.qpp"
    ReferenceHolder<Queue> q(queue, xsink);
   if (queue->getMax() != -1)
      return xsink->raiseException("QUEUE-ERROR", "the Queue object passed as a maximum size of %d entr%s, which could cause internal I/O to freeze in the Socket object if used; use a Queue object with no maximum size for I/O monitoring", queue->getMax(), queue->getMax() == 1 ? "y" : "ies");
   // pass reference from QoreObject::getReferencedPrivateData() to function
   client->setWarningQueue(xsink, warning_ms, warning_bs, q.release(), arg.refSelf(), min_ms);
    return QoreValue();
}

// AbstractPollOperation HTTPClient::startPollConnect(){}
static QoreValue HTTPClient_startPollConnect(QoreObject* self, QoreHttpClientObject* client, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 308 "QC_HTTPClient.qpp"
    return client->startPollConnect(xsink, self);
}

// AbstractPollOperation HTTPClient::startPollSendRecv(string method, *string path, string body, *hash<auto> headers){}
static QoreValue HTTPClient_startPollSendRecv_VsNsVsC11_hash_auto_(QoreObject* self, QoreHttpClientObject* client, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* method = HARD_QORE_VALUE_STRING(args, 0);
    const QoreStringNode* path = get_param_value(args, 1).get<const QoreStringNode>();
    const QoreStringNode* body = HARD_QORE_VALUE_STRING(args, 2);
    const QoreHashNode* headers = get_param_value(args, 3).get<const QoreHashNode>();
# 434 "QC_HTTPClient.qpp"
    return client->startPollSendRecv(xsink, self, method, path, body, body->c_str(), body->size(), headers,
        body->getEncoding());
    return QoreValue();
}

// AbstractPollOperation HTTPClient::startPollSendRecv(string method, *string path, *binary body, *hash<auto> headers){}
static QoreValue HTTPClient_startPollSendRecv_VsNsNyC11_hash_auto_(QoreObject* self, QoreHttpClientObject* client, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* method = HARD_QORE_VALUE_STRING(args, 0);
    const QoreStringNode* path = get_param_value(args, 1).get<const QoreStringNode>();
    const BinaryNode* body = get_param_value(args, 2).get<const BinaryNode>();
    const QoreHashNode* headers = get_param_value(args, 3).get<const QoreHashNode>();
# 471 "QC_HTTPClient.qpp"
    return client->startPollSendRecv(xsink, self, method, path, body, body ? body->getPtr() : nullptr,
        body ? body->size() : 0, headers);
    return QoreValue();
}

DLLLOCAL void preinitHTTPClientClass() {
    QC_HTTPCLIENT = new QoreClass("HTTPClient", "::Qore::HTTPClient", QDOM_NETWORK);
    CID_HTTPCLIENT = QC_HTTPCLIENT->getID();
    QC_HTTPCLIENT->setSystem();
}

DLLLOCAL QoreClass* initHTTPClientClass(QoreNamespace& ns) {
    if (!QC_HTTPCLIENT)
        preinitHTTPClientClass();

    // set parent class
    assert(QC_SOCKET);
    QC_HTTPCLIENT->addBuiltinVirtualBaseClass(QC_SOCKET);

    // nothing HTTPClient::addDefaultHeaders(hash<auto> hdr){}
    QC_HTTPCLIENT->addMethod("addDefaultHeaders", (q_method_n_t)HTTPClient_addDefaultHeaders_C10hash_auto_, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, autoHashTypeInfo, QORE_PARAM_NO_ARG, "hdr");

    // nothing HTTPClient::clearProxyURL(){}
    QC_HTTPCLIENT->addMethod("clearProxyURL", (q_method_n_t)HTTPClient_clearProxyURL, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

    // nothing HTTPClient::clearProxyUserPassword(){}
    QC_HTTPCLIENT->addMethod("clearProxyUserPassword", (q_method_n_t)HTTPClient_clearProxyUserPassword, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

    // nothing HTTPClient::clearStats(){}
    QC_HTTPCLIENT->addMethod("clearStats", (q_method_n_t)HTTPClient_clearStats, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

    // nothing HTTPClient::clearUserPassword(){}
    QC_HTTPCLIENT->addMethod("clearUserPassword", (q_method_n_t)HTTPClient_clearUserPassword, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

    // nothing HTTPClient::clearWarningQueue(){}
    QC_HTTPCLIENT->addMethod("clearWarningQueue", (q_method_n_t)HTTPClient_clearWarningQueue, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

    // nothing HTTPClient::connect(){}
    QC_HTTPCLIENT->addMethod("connect", (q_method_n_t)HTTPClient_connect, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

    // HTTPClient::constructor(hash<auto> opts) {}
    QC_HTTPCLIENT->addConstructor(HTTPClient_constructor_C10hash_auto_, Public, QCF_NO_FLAGS, QDOM_DEFAULT, 1, autoHashTypeInfo, QORE_PARAM_NO_ARG, "opts");

    // HTTPClient::constructor() {}
    QC_HTTPCLIENT->addConstructor(HTTPClient_constructor, Public, QCF_NO_FLAGS, QDOM_DEFAULT);

    // HTTPClient::copy() {}
    QC_HTTPCLIENT->setCopy((q_copy_t)HTTPClient_copy);

    // HTTPClient::destructor() {}
    QC_HTTPCLIENT->setDestructor((q_destructor_t)HTTPClient_destructor);

    // nothing HTTPClient::disconnect(){}
    QC_HTTPCLIENT->addMethod("disconnect", (q_method_n_t)HTTPClient_disconnect, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

    // *string HTTPClient::get(string path, *hash<auto> headers, *reference<hash<auto>> info){}
    QC_HTTPCLIENT->addMethod("get", (q_method_n_t)HTTPClient_get_VsC11_hash_auto_C22_reference_hash_auto__, Public, QCF_NO_FLAGS, QDOM_DEFAULT, stringOrNothingTypeInfo, 3, stringTypeInfo, QORE_PARAM_NO_ARG, "path", autoHashOrNothingTypeInfo, QORE_PARAM_NO_ARG, "headers", qore_get_complex_reference_or_nothing_type(autoHashTypeInfo), QORE_PARAM_NO_ARG, "info");

    // string HTTPClient::getAssumedEncoding(){}
    QC_HTTPCLIENT->addMethod("getAssumedEncoding", (q_method_n_t)HTTPClient_getAssumedEncoding, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, stringTypeInfo);

    // int HTTPClient::getConnectTimeout(){}
    QC_HTTPCLIENT->addMethod("getConnectTimeout", (q_method_n_t)HTTPClient_getConnectTimeout, Public, QCF_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

    // *string HTTPClient::getConnectionPath(){}
    QC_HTTPCLIENT->addMethod("getConnectionPath", (q_method_n_t)HTTPClient_getConnectionPath, Public, QCF_CONSTANT, QDOM_DEFAULT, stringOrNothingTypeInfo);

    // hash<string,string> HTTPClient::getDefaultHeaders(){}
    QC_HTTPCLIENT->addMethod("getDefaultHeaders", (q_method_n_t)HTTPClient_getDefaultHeaders, Public, QCF_CONSTANT, QDOM_DEFAULT, qore_get_complex_hash_type(stringTypeInfo));

    // string HTTPClient::getDefaultPath(){}
    QC_HTTPCLIENT->addMethod("getDefaultPath", (q_method_n_t)HTTPClient_getDefaultPath, Public, QCF_CONSTANT, QDOM_DEFAULT, stringTypeInfo);

    // string HTTPClient::getEncoding(){}
    QC_HTTPCLIENT->addMethod("getEncoding", (q_method_n_t)HTTPClient_getEncoding, Public, QCF_CONSTANT, QDOM_DEFAULT, stringTypeInfo);

    // bool HTTPClient::getEncodingPassthru(){}
    QC_HTTPCLIENT->addMethod("getEncodingPassthru", (q_method_n_t)HTTPClient_getEncodingPassthru, Public, QCF_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

    // bool HTTPClient::getErrorPassthru(){}
    QC_HTTPCLIENT->addMethod("getErrorPassthru", (q_method_n_t)HTTPClient_getErrorPassthru, Public, QCF_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

    // string HTTPClient::getHTTPVersion(){}
    QC_HTTPCLIENT->addMethod("getHTTPVersion", (q_method_n_t)HTTPClient_getHTTPVersion, Public, QCF_CONSTANT, QDOM_DEFAULT, stringTypeInfo);

    // string HTTPClient::getHostHeaderValue(){}
    QC_HTTPCLIENT->addMethod("getHostHeaderValue", (q_method_n_t)HTTPClient_getHostHeaderValue, Public, QCF_CONSTANT, QDOM_DEFAULT, stringTypeInfo);

    // int HTTPClient::getMaxRedirects(){}
    QC_HTTPCLIENT->addMethod("getMaxRedirects", (q_method_n_t)HTTPClient_getMaxRedirects, Public, QCF_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

    // bool HTTPClient::getNoDelay(){}
    QC_HTTPCLIENT->addMethod("getNoDelay", (q_method_n_t)HTTPClient_getNoDelay, Public, QCF_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

    // bool HTTPClient::getPreEncodedUrls(){}
    QC_HTTPCLIENT->addMethod("getPreEncodedUrls", (q_method_n_t)HTTPClient_getPreEncodedUrls, Public, QCF_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

    // *string HTTPClient::getProxyURL(){}
    QC_HTTPCLIENT->addMethod("getProxyURL", (q_method_n_t)HTTPClient_getProxyURL, Public, QCF_CONSTANT, QDOM_DEFAULT, stringOrNothingTypeInfo);

    // bool HTTPClient::getRedirectPassthru(){}
    QC_HTTPCLIENT->addMethod("getRedirectPassthru", (q_method_n_t)HTTPClient_getRedirectPassthru, Public, QCF_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

    // *string HTTPClient::getSafeProxyURL(){}
    QC_HTTPCLIENT->addMethod("getSafeProxyURL", (q_method_n_t)HTTPClient_getSafeProxyURL, Public, QCF_CONSTANT, QDOM_DEFAULT, stringOrNothingTypeInfo);

    // *string HTTPClient::getSafeURL(){}
    QC_HTTPCLIENT->addMethod("getSafeURL", (q_method_n_t)HTTPClient_getSafeURL, Public, QCF_CONSTANT, QDOM_DEFAULT, stringOrNothingTypeInfo);

    // int HTTPClient::getTimeout(){}
    QC_HTTPCLIENT->addMethod("getTimeout", (q_method_n_t)HTTPClient_getTimeout, Public, QCF_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

    // *string HTTPClient::getURL(){}
    QC_HTTPCLIENT->addMethod("getURL", (q_method_n_t)HTTPClient_getURL, Public, QCF_CONSTANT, QDOM_DEFAULT, stringOrNothingTypeInfo);

    // hash<auto> HTTPClient::getUsageInfo(){}
    QC_HTTPCLIENT->addMethod("getUsageInfo", (q_method_n_t)HTTPClient_getUsageInfo, Public, QCF_CONSTANT, QDOM_DEFAULT, autoHashTypeInfo);

    // hash<auto> HTTPClient::head(string path, *hash<auto> headers, *reference<hash<auto>> info){}
    QC_HTTPCLIENT->addMethod("head", (q_method_n_t)HTTPClient_head_VsC11_hash_auto_C22_reference_hash_auto__, Public, QCF_NO_FLAGS, QDOM_DEFAULT, autoHashTypeInfo, 3, stringTypeInfo, QORE_PARAM_NO_ARG, "path", autoHashOrNothingTypeInfo, QORE_PARAM_NO_ARG, "headers", qore_get_complex_reference_or_nothing_type(autoHashTypeInfo), QORE_PARAM_NO_ARG, "info");

    // bool HTTPClient::isConnected(){}
    QC_HTTPCLIENT->addMethod("isConnected", (q_method_n_t)HTTPClient_isConnected, Public, QCF_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

    // bool HTTPClient::isProxySecure(){}
    QC_HTTPCLIENT->addMethod("isProxySecure", (q_method_n_t)HTTPClient_isProxySecure, Public, QCF_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

    // bool HTTPClient::isSecure(){}
    QC_HTTPCLIENT->addMethod("isSecure", (q_method_n_t)HTTPClient_isSecure, Public, QCF_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

    // *string HTTPClient::post(string path, string body, *hash<auto> headers, *reference<hash<auto>> info){}
    QC_HTTPCLIENT->addMethod("post", (q_method_n_t)HTTPClient_post_VsVsC11_hash_auto_C22_reference_hash_auto__, Public, QCF_NO_FLAGS, QDOM_DEFAULT, stringOrNothingTypeInfo, 4, stringTypeInfo, QORE_PARAM_NO_ARG, "path", stringTypeInfo, QORE_PARAM_NO_ARG, "body", autoHashOrNothingTypeInfo, QORE_PARAM_NO_ARG, "headers", qore_get_complex_reference_or_nothing_type(autoHashTypeInfo), QORE_PARAM_NO_ARG, "info");

    // *string HTTPClient::post(string path, *binary body, *hash<auto> headers, *reference<hash<auto>> info){}
    QC_HTTPCLIENT->addMethod("post", (q_method_n_t)HTTPClient_post_VsNyC11_hash_auto_C22_reference_hash_auto__, Public, QCF_NO_FLAGS, QDOM_DEFAULT, stringOrNothingTypeInfo, 4, stringTypeInfo, QORE_PARAM_NO_ARG, "path", binaryOrNothingTypeInfo, QORE_PARAM_NO_ARG, "body", autoHashOrNothingTypeInfo, QORE_PARAM_NO_ARG, "headers", qore_get_complex_reference_or_nothing_type(autoHashTypeInfo), QORE_PARAM_NO_ARG, "info");

    // hash<auto> HTTPClient::send(string body, string method, *string path, *hash<auto> headers, softbool getbody = False, *reference<hash<auto>> info){}
    QC_HTTPCLIENT->addMethod("send", (q_method_n_t)HTTPClient_send_VsVsNsC11_hash_auto_vbC22_reference_hash_auto__, Public, QCF_NO_FLAGS, QDOM_DEFAULT, autoHashTypeInfo, 6, stringTypeInfo, QORE_PARAM_NO_ARG, "body", stringTypeInfo, QORE_PARAM_NO_ARG, "method", stringOrNothingTypeInfo, QORE_PARAM_NO_ARG, "path", autoHashOrNothingTypeInfo, QORE_PARAM_NO_ARG, "headers", softBoolTypeInfo, QoreSimpleValue().assign(false), "getbody", qore_get_complex_reference_or_nothing_type(autoHashTypeInfo), QORE_PARAM_NO_ARG, "info");

    // hash<auto> HTTPClient::send(*binary body, string method, *string path, *hash<auto> headers, softbool getbody = False, *reference<hash<auto>> info){}
    QC_HTTPCLIENT->addMethod("send", (q_method_n_t)HTTPClient_send_NyVsNsC11_hash_auto_vbC22_reference_hash_auto__, Public, QCF_NO_FLAGS, QDOM_DEFAULT, autoHashTypeInfo, 6, binaryOrNothingTypeInfo, QORE_PARAM_NO_ARG, "body", stringTypeInfo, QORE_PARAM_NO_ARG, "method", stringOrNothingTypeInfo, QORE_PARAM_NO_ARG, "path", autoHashOrNothingTypeInfo, QORE_PARAM_NO_ARG, "headers", softBoolTypeInfo, QoreSimpleValue().assign(false), "getbody", qore_get_complex_reference_or_nothing_type(autoHashTypeInfo), QORE_PARAM_NO_ARG, "info");

    // nothing HTTPClient::send(Qore::OutputStream os, *data body, string method, *string path, *hash<auto> headers, timeout timeout_ms = 0, softbool getbody = False, *reference<hash<auto>> info, *code rcb){}
    QC_HTTPCLIENT->addMethod("send", (q_method_n_t)HTTPClient_send_C12OutputStreamNDVsNsC11_hash_auto_VtvbC22_reference_hash_auto__Nc, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 9, QC_OUTPUTSTREAM->getTypeInfo(), QORE_PARAM_NO_ARG, "os", dataOrNothingTypeInfo, QORE_PARAM_NO_ARG, "body", stringTypeInfo, QORE_PARAM_NO_ARG, "method", stringOrNothingTypeInfo, QORE_PARAM_NO_ARG, "path", autoHashOrNothingTypeInfo, QORE_PARAM_NO_ARG, "headers", timeoutTypeInfo, QoreSimpleValue().assign((int64)0), "timeout_ms", softBoolTypeInfo, QoreSimpleValue().assign(false), "getbody", qore_get_complex_reference_or_nothing_type(autoHashTypeInfo), QORE_PARAM_NO_ARG, "info", codeOrNothingTypeInfo, QORE_PARAM_NO_ARG, "rcb");

    // nothing HTTPClient::sendChunked(Qore::OutputStream os, Qore::InputStream is, string method, int max_chunk_size = 4096, *string path, *hash<auto> headers, timeout timeout_ms = 0, softbool getbody = False, *reference<hash<auto>> info, *code rcb, *code tcb){}
    QC_HTTPCLIENT->addMethod("sendChunked", (q_method_n_t)HTTPClient_sendChunked_C12OutputStreamC11InputStreamVsViNsC11_hash_auto_VtvbC22_reference_hash_auto__NcNc, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 11, QC_OUTPUTSTREAM->getTypeInfo(), QORE_PARAM_NO_ARG, "os", QC_INPUTSTREAM->getTypeInfo(), QORE_PARAM_NO_ARG, "is", stringTypeInfo, QORE_PARAM_NO_ARG, "method", bigIntTypeInfo, QoreSimpleValue().assign((int64)4096), "max_chunk_size", stringOrNothingTypeInfo, QORE_PARAM_NO_ARG, "path", autoHashOrNothingTypeInfo, QORE_PARAM_NO_ARG, "headers", timeoutTypeInfo, QoreSimpleValue().assign((int64)0), "timeout_ms", softBoolTypeInfo, QoreSimpleValue().assign(false), "getbody", qore_get_complex_reference_or_nothing_type(autoHashTypeInfo), QORE_PARAM_NO_ARG, "info", codeOrNothingTypeInfo, QORE_PARAM_NO_ARG, "rcb", codeOrNothingTypeInfo, QORE_PARAM_NO_ARG, "tcb");

    // nothing HTTPClient::sendWithCallbacks(code scb, code rcb, string method, *string path, *hash<auto> headers, timeout timeout_ms = 0, softbool getbody = False, *reference<hash<auto>> info){}
    QC_HTTPCLIENT->addMethod("sendWithCallbacks", (q_method_n_t)HTTPClient_sendWithCallbacks_VcVcVsNsC11_hash_auto_VtvbC22_reference_hash_auto__, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 8, codeTypeInfo, QORE_PARAM_NO_ARG, "scb", codeTypeInfo, QORE_PARAM_NO_ARG, "rcb", stringTypeInfo, QORE_PARAM_NO_ARG, "method", stringOrNothingTypeInfo, QORE_PARAM_NO_ARG, "path", autoHashOrNothingTypeInfo, QORE_PARAM_NO_ARG, "headers", timeoutTypeInfo, QoreSimpleValue().assign((int64)0), "timeout_ms", softBoolTypeInfo, QoreSimpleValue().assign(false), "getbody", qore_get_complex_reference_or_nothing_type(autoHashTypeInfo), QORE_PARAM_NO_ARG, "info");

    // nothing HTTPClient::sendWithRecvCallback(code rcb, string body, string method, *string path, *hash<auto> headers, timeout timeout_ms = 0, softbool getbody = False, *reference<hash<auto>> info){}
    QC_HTTPCLIENT->addMethod("sendWithRecvCallback", (q_method_n_t)HTTPClient_sendWithRecvCallback_VcVsVsNsC11_hash_auto_VtvbC22_reference_hash_auto__, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 8, codeTypeInfo, QORE_PARAM_NO_ARG, "rcb", stringTypeInfo, QORE_PARAM_NO_ARG, "body", stringTypeInfo, QORE_PARAM_NO_ARG, "method", stringOrNothingTypeInfo, QORE_PARAM_NO_ARG, "path", autoHashOrNothingTypeInfo, QORE_PARAM_NO_ARG, "headers", timeoutTypeInfo, QoreSimpleValue().assign((int64)0), "timeout_ms", softBoolTypeInfo, QoreSimpleValue().assign(false), "getbody", qore_get_complex_reference_or_nothing_type(autoHashTypeInfo), QORE_PARAM_NO_ARG, "info");

    // nothing HTTPClient::sendWithRecvCallback(code rcb, *binary body, string method, *string path, *hash<auto> headers, timeout timeout_ms = 0, softbool getbody = False, *reference<hash<auto>> info){}
    QC_HTTPCLIENT->addMethod("sendWithRecvCallback", (q_method_n_t)HTTPClient_sendWithRecvCallback_VcNyVsNsC11_hash_auto_VtvbC22_reference_hash_auto__, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 8, codeTypeInfo, QORE_PARAM_NO_ARG, "rcb", binaryOrNothingTypeInfo, QORE_PARAM_NO_ARG, "body", stringTypeInfo, QORE_PARAM_NO_ARG, "method", stringOrNothingTypeInfo, QORE_PARAM_NO_ARG, "path", autoHashOrNothingTypeInfo, QORE_PARAM_NO_ARG, "headers", timeoutTypeInfo, QoreSimpleValue().assign((int64)0), "timeout_ms", softBoolTypeInfo, QoreSimpleValue().assign(false), "getbody", qore_get_complex_reference_or_nothing_type(autoHashTypeInfo), QORE_PARAM_NO_ARG, "info");

    // hash<auto> HTTPClient::sendWithSendCallback(code scb, string method, *string path, *hash<auto> headers, timeout timeout_ms = 0, softbool getbody = False, *reference<hash> info){}
    QC_HTTPCLIENT->addMethod("sendWithSendCallback", (q_method_n_t)HTTPClient_sendWithSendCallback_VcVsNsC11_hash_auto_VtvbC16_reference_hash_, Public, QCF_NO_FLAGS, QDOM_DEFAULT, autoHashTypeInfo, 7, codeTypeInfo, QORE_PARAM_NO_ARG, "scb", stringTypeInfo, QORE_PARAM_NO_ARG, "method", stringOrNothingTypeInfo, QORE_PARAM_NO_ARG, "path", autoHashOrNothingTypeInfo, QORE_PARAM_NO_ARG, "headers", timeoutTypeInfo, QoreSimpleValue().assign((int64)0), "timeout_ms", softBoolTypeInfo, QoreSimpleValue().assign(false), "getbody", qore_get_complex_reference_or_nothing_type(hashTypeInfo), QORE_PARAM_NO_ARG, "info");

    // nothing HTTPClient::setAssumedEncoding(*string encoding){}
    QC_HTTPCLIENT->addMethod("setAssumedEncoding", (q_method_n_t)HTTPClient_setAssumedEncoding_Ns, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, stringOrNothingTypeInfo, QORE_PARAM_NO_ARG, "encoding");

    // nothing HTTPClient::setConnectTimeout(timeout timeout_ms = -1){}
    QC_HTTPCLIENT->addMethod("setConnectTimeout", (q_method_n_t)HTTPClient_setConnectTimeout_Vt, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, timeoutTypeInfo, QoreSimpleValue().assign((int64)-1), "timeout_ms");

    // nothing HTTPClient::setConnectionPath(*string uri_path){}
    QC_HTTPCLIENT->addMethod("setConnectionPath", (q_method_n_t)HTTPClient_setConnectionPath_Ns, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, stringOrNothingTypeInfo, QORE_PARAM_NO_ARG, "uri_path");

    // nothing HTTPClient::setDefaultPath(*string path){}
    QC_HTTPCLIENT->addMethod("setDefaultPath", (q_method_n_t)HTTPClient_setDefaultPath_Ns, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, stringOrNothingTypeInfo, QORE_PARAM_NO_ARG, "path");

    // nothing HTTPClient::setEncoding(string encoding){}
    QC_HTTPCLIENT->addMethod("setEncoding", (q_method_n_t)HTTPClient_setEncoding_Vs, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "encoding");

    // bool HTTPClient::setEncodingPassthru(bool set = True){}
    QC_HTTPCLIENT->addMethod("setEncodingPassthru", (q_method_n_t)HTTPClient_setEncodingPassthru_Vb, Public, QCF_NO_FLAGS, QDOM_DEFAULT, boolTypeInfo, 1, boolTypeInfo, QoreSimpleValue().assign(true), "set");

    // bool HTTPClient::setErrorPassthru(bool set = True){}
    QC_HTTPCLIENT->addMethod("setErrorPassthru", (q_method_n_t)HTTPClient_setErrorPassthru_Vb, Public, QCF_NO_FLAGS, QDOM_DEFAULT, boolTypeInfo, 1, boolTypeInfo, QoreSimpleValue().assign(true), "set");

    // nothing HTTPClient::setEventQueue(){}
    QC_HTTPCLIENT->addMethod("setEventQueue", (q_method_n_t)HTTPClient_setEventQueue, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

    // nothing HTTPClient::setEventQueue(Qore::Thread::Queue queue, auto arg, *bool with_data){}
    QC_HTTPCLIENT->addMethod("setEventQueue", (q_method_n_t)HTTPClient_setEventQueue_C5QueueVANb, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 3, QC_QUEUE->getTypeInfo(), QORE_PARAM_NO_ARG, "queue", autoTypeInfo, QORE_PARAM_NO_ARG, "arg", boolOrNothingTypeInfo, QORE_PARAM_NO_ARG, "with_data");

    // nothing HTTPClient::setHTTPVersion(string ver){}
    QC_HTTPCLIENT->addMethod("setHTTPVersion", (q_method_n_t)HTTPClient_setHTTPVersion_Vs, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "ver");

    // nothing HTTPClient::setMaxRedirects(softint mr = 0){}
    QC_HTTPCLIENT->addMethod("setMaxRedirects", (q_method_n_t)HTTPClient_setMaxRedirects_vi, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, softBigIntTypeInfo, QoreSimpleValue().assign((int64)0), "mr");

    // int HTTPClient::setNoDelay(softbool b = True){}
    QC_HTTPCLIENT->addMethod("setNoDelay", (q_method_n_t)HTTPClient_setNoDelay_vb, Public, QCF_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo, 1, softBoolTypeInfo, QoreSimpleValue().assign(true), "b");

    // nothing HTTPClient::setPersistent(){}
    QC_HTTPCLIENT->addMethod("setPersistent", (q_method_n_t)HTTPClient_setPersistent, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

    // bool HTTPClient::setPreEncodedUrls(bool set = True){}
    QC_HTTPCLIENT->addMethod("setPreEncodedUrls", (q_method_n_t)HTTPClient_setPreEncodedUrls_Vb, Public, QCF_NO_FLAGS, QDOM_DEFAULT, boolTypeInfo, 1, boolTypeInfo, QoreSimpleValue().assign(true), "set");

    // nothing HTTPClient::setProxySecure(softbool b = True){}
    QC_HTTPCLIENT->addMethod("setProxySecure", (q_method_n_t)HTTPClient_setProxySecure_vb, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, softBoolTypeInfo, QoreSimpleValue().assign(true), "b");

    // nothing HTTPClient::setProxyURL(){}
    QC_HTTPCLIENT->addMethod("setProxyURL", (q_method_n_t)HTTPClient_setProxyURL, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

    // nothing HTTPClient::setProxyURL(string url){}
    QC_HTTPCLIENT->addMethod("setProxyURL", (q_method_n_t)HTTPClient_setProxyURL_Vs, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "url");

    // nothing HTTPClient::setProxyUserPassword(string user, string pass){}
    QC_HTTPCLIENT->addMethod("setProxyUserPassword", (q_method_n_t)HTTPClient_setProxyUserPassword_VsVs, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 2, stringTypeInfo, QORE_PARAM_NO_ARG, "user", stringTypeInfo, QORE_PARAM_NO_ARG, "pass");

    // nothing HTTPClient::setProxyUserPassword(){}
    QC_HTTPCLIENT->addMethod("setProxyUserPassword", (q_method_n_t)HTTPClient_setProxyUserPassword, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

    // bool HTTPClient::setRedirectPassthru(bool set = True){}
    QC_HTTPCLIENT->addMethod("setRedirectPassthru", (q_method_n_t)HTTPClient_setRedirectPassthru_Vb, Public, QCF_NO_FLAGS, QDOM_DEFAULT, boolTypeInfo, 1, boolTypeInfo, QoreSimpleValue().assign(true), "set");

    // nothing HTTPClient::setSecure(softbool secure = True){}
    QC_HTTPCLIENT->addMethod("setSecure", (q_method_n_t)HTTPClient_setSecure_vb, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, softBoolTypeInfo, QoreSimpleValue().assign(true), "secure");

    // nothing HTTPClient::setTimeout(timeout timeout_ms = 0){}
    QC_HTTPCLIENT->addMethod("setTimeout", (q_method_n_t)HTTPClient_setTimeout_Vt, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, timeoutTypeInfo, QoreSimpleValue().assign((int64)0), "timeout_ms");

    // nothing HTTPClient::setURL(string url){}
    QC_HTTPCLIENT->addMethod("setURL", (q_method_n_t)HTTPClient_setURL_Vs, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "url");

    // nothing HTTPClient::setUserPassword(string user, string pass){}
    QC_HTTPCLIENT->addMethod("setUserPassword", (q_method_n_t)HTTPClient_setUserPassword_VsVs, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 2, stringTypeInfo, QORE_PARAM_NO_ARG, "user", stringTypeInfo, QORE_PARAM_NO_ARG, "pass");

    // nothing HTTPClient::setUserPassword(){}
    QC_HTTPCLIENT->addMethod("setUserPassword", (q_method_n_t)HTTPClient_setUserPassword, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

    // nothing HTTPClient::setWarningQueue(int warning_ms, int warning_bs, Queue queue, auto arg, timeout min_ms = 1s){}
    QC_HTTPCLIENT->addMethod("setWarningQueue", (q_method_n_t)HTTPClient_setWarningQueue_ViViC5QueueVAVt, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 5, bigIntTypeInfo, QORE_PARAM_NO_ARG, "warning_ms", bigIntTypeInfo, QORE_PARAM_NO_ARG, "warning_bs", QC_QUEUE->getTypeInfo(), QORE_PARAM_NO_ARG, "queue", autoTypeInfo, QORE_PARAM_NO_ARG, "arg", timeoutTypeInfo, QoreSimpleValue().assign(DateTimeNode::makeRelative(0, 0, 0, 0, 0, 1, 0)), "min_ms");

    // AbstractPollOperation HTTPClient::startPollConnect(){}
    QC_HTTPCLIENT->addMethod("startPollConnect", (q_method_n_t)HTTPClient_startPollConnect, Public, QCF_NO_FLAGS, QDOM_DEFAULT, QC_ABSTRACTPOLLOPERATION->getTypeInfo());

    // AbstractPollOperation HTTPClient::startPollSendRecv(string method, *string path, string body, *hash<auto> headers){}
    QC_HTTPCLIENT->addMethod("startPollSendRecv", (q_method_n_t)HTTPClient_startPollSendRecv_VsNsVsC11_hash_auto_, Public, QCF_NO_FLAGS, QDOM_DEFAULT, QC_ABSTRACTPOLLOPERATION->getTypeInfo(), 4, stringTypeInfo, QORE_PARAM_NO_ARG, "method", stringOrNothingTypeInfo, QORE_PARAM_NO_ARG, "path", stringTypeInfo, QORE_PARAM_NO_ARG, "body", autoHashOrNothingTypeInfo, QORE_PARAM_NO_ARG, "headers");

    // AbstractPollOperation HTTPClient::startPollSendRecv(string method, *string path, *binary body, *hash<auto> headers){}
    QC_HTTPCLIENT->addMethod("startPollSendRecv", (q_method_n_t)HTTPClient_startPollSendRecv_VsNsNyC11_hash_auto_, Public, QCF_NO_FLAGS, QDOM_DEFAULT, QC_ABSTRACTPOLLOPERATION->getTypeInfo(), 4, stringTypeInfo, QORE_PARAM_NO_ARG, "method", stringOrNothingTypeInfo, QORE_PARAM_NO_ARG, "path", binaryOrNothingTypeInfo, QORE_PARAM_NO_ARG, "body", autoHashOrNothingTypeInfo, QORE_PARAM_NO_ARG, "headers");

    return QC_HTTPCLIENT;
}
