/* -*- mode: c++; indent-tabs-mode: nil -*- */
/*
    QC_Program.qpp

    Qore Programming Language

    Copyright (C) 2003 - 2023 Qore Technologies, s.r.o.

    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
    to deal in the Software without restriction, including without limitation
    the rights to use, copy, modify, merge, publish, distribute, sublicense,
    and/or sell copies of the Software, and to permit persons to whom the
    Software is furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
    DEALINGS IN THE SOFTWARE.

    Note that the Qore library is released under a choice of three open-source
    licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
    information.
*/

#include <qore/Qore.h>
#include "qore/intern/QC_Program.h"
#include "qore/intern/QC_ProgramControl.h"
#include "qore/intern/QC_TimeZone.h"
#include "qore/intern/QC_Breakpoint.h"
#include "qore/intern/QC_Expression.h"
#include "qore/intern/qore_program_private.h"
#include "qore/intern/QoreObjectIntern.h"
#include "qore/intern/ModuleInfo.h"
#include "qore/ParseOptionMap.h"

#include "qore/intern/ModuleInfo.h"
extern QoreModuleManager QMM;

#ifndef WARN_MODULES
// needed so that the Qore default argument value in sinatures below will match a C++ value
#define WARN_MODULES QP_WARN_MODULES
#endif
TypedHashDecl* init_hashdecl_SourceLocationInfo(QoreNamespace& ns) {
    TypedHashDecl* hd = new TypedHashDecl("SourceLocationInfo", "::Qore::SourceLocationInfo");
    hd->addMember("builtin", boolTypeInfo, QoreValue());
    hd->addMember("endline", bigIntTypeInfo, QoreValue());
    hd->addMember("file", stringOrNothingTypeInfo, QoreValue());
    hd->addMember("line", bigIntTypeInfo, QoreValue());
    hd->addMember("offset", bigIntTypeInfo, QoreValue());
    hd->addMember("source", stringOrNothingTypeInfo, QoreValue());
    ns.addSystemHashDecl(hd);
    return hd;
}

/* Qore class Qore::Program */

qore_classid_t CID_PROGRAM;
QoreClass* QC_PROGRAM;

// auto Program::callFunction(string name, ...){}
static QoreValue Program_callFunction_VsVV(QoreObject* self, QoreProgram* p, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* name = HARD_QORE_VALUE_STRING(args, 0);
# 1213 "QC_Program.qpp"
    TempEncodingHelper tmp(name, QCS_DEFAULT, xsink);
    if (!tmp)
        return QoreValue();

    ReferenceHolder<QoreListNode> vargs(xsink);
    if (args->size() > 1)
        vargs = args->copyListFrom(1);

    return p->callFunction(tmp->getBuffer(), *vargs, xsink);
}

// auto Program::callFunctionArgs(string name, *softlist<auto> vargs){}
static QoreValue Program_callFunctionArgs_VsC15_softlist_auto_(QoreObject* self, QoreProgram* p, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* name = HARD_QORE_VALUE_STRING(args, 0);
    const QoreListNode* vargs = get_param_value(args, 1).get<const QoreListNode>();
# 1244 "QC_Program.qpp"
    TempEncodingHelper tmp(name, QCS_DEFAULT, xsink);
    if (!tmp)
        return QoreValue();
    return p->callFunction(tmp->getBuffer(), vargs, xsink);
}

// auto Program::callStaticMethod(string class_name, string method, ...){}
static QoreValue Program_callStaticMethod_VsVsVV(QoreObject* self, QoreProgram* p, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* class_name = HARD_QORE_VALUE_STRING(args, 0);
    const QoreStringNode* method = HARD_QORE_VALUE_STRING(args, 1);
# 1276 "QC_Program.qpp"
    ReferenceHolder<QoreListNode> call_args(xsink);
    // if there are arguments to pass
    if (num_args(args) > 2) {
        // create argument list by copying current list
        call_args = args->copyListFrom(2);
    }
    return q_call_static_method_args(p, class_name, method, *call_args, xsink);
}

// auto Program::callStaticMethodArgs(string class_name, string method, *softlist<auto> call_args){}
static QoreValue Program_callStaticMethodArgs_VsVsC15_softlist_auto_(QoreObject* self, QoreProgram* p, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* class_name = HARD_QORE_VALUE_STRING(args, 0);
    const QoreStringNode* method = HARD_QORE_VALUE_STRING(args, 1);
    const QoreListNode* call_args = get_param_value(args, 2).get<const QoreListNode>();
# 1311 "QC_Program.qpp"
    return q_call_static_method_args(p, class_name, method, call_args, xsink);
}

// Program::constructor(softint po = PO_DEFAULT) {}
static void Program_constructor_vi(QoreObject* self, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 po = HARD_QORE_VALUE_INT(args, 0);
# 951 "QC_Program.qpp"
if (po & PO_SYSTEM_OPS) {
        xsink->raiseException("PROGRAM-OPTION-ERROR", "parse options (0x" QLLX ") contain restricted options that can only be set by the system", po);
        return;
    }

    QoreProgram* pgm = new QoreProgram(getProgram(), po);
    //pgm->registerQoreObject(self, xsink);
    assert(!xsink->isEvent());

    self->setPrivate(CID_PROGRAM, pgm);
    printd(5, "Program::constructor() pgm: %p, pgmid: %d, self: %p\n", pgm, pgm->getProgramId(), self);
}

// Program::copy() {}
static void Program_copy(QoreObject* self, QoreObject* old, QoreProgram* p, ExceptionSink* xsink) {
# 969 "QC_Program.qpp"
xsink->raiseException("PROGRAM-COPY-ERROR", "copying Program objects is currently unsupported");
}

// nothing Program::define(string def, auto val){}
static QoreValue Program_define_VsVA(QoreObject* self, QoreProgram* p, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* def = HARD_QORE_VALUE_STRING(args, 0);
    QoreValue val = get_param_value(args, 1);
# 1798 "QC_Program.qpp"
    TempEncodingHelper t(def, QCS_DEFAULT, xsink);
   if (t)
      qore_program_private::runTimeDefine(p, t->getBuffer(), val.refSelf(), xsink);
    return QoreValue();
}

// Program::destructor() {}
static void Program_destructor(QoreObject* self, QoreProgram* p, ExceptionSink* xsink) {
# 976 "QC_Program.qpp"
printd(5, "Program::destructor() pgm: %p pgmid: %d self: %p\n", p, p->getProgramId(), self);
    // inherited system object destructor is not executed
    //p->unregisterQoreObject(self, xsink);

    p->waitForTerminationAndDeref(xsink);
}

// nothing Program::disableParseOptions(softint opt){}
static QoreValue Program_disableParseOptions_vi(QoreObject* self, QoreProgram* p, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 opt = HARD_QORE_VALUE_INT(args, 0);
# 1597 "QC_Program.qpp"
    if (opt & PO_SYSTEM_OPS)
      return xsink->raiseException("PROGRAM-OPTION-ERROR", "parse options (0x" QLLX ") contain restricted options that can only be set by the system", opt);

   p->disableParseOptions(opt, xsink);
    return QoreValue();
}

// bool Program::existsFunction(string name){}
static QoreValue Program_existsFunction_Vs(QoreObject* self, QoreProgram* p, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* name = HARD_QORE_VALUE_STRING(args, 0);
# 1328 "QC_Program.qpp"
    TempEncodingHelper tmp(name, QCS_DEFAULT, xsink);
    if (!tmp)
        return false;
    return p->existsFunction(tmp->getBuffer());
}

// list<hash<auto>> Program::findFunctionVariants(string function){}
static QoreValue Program_findFunctionVariants_Vs(QoreObject* self, QoreProgram* p, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* function = HARD_QORE_VALUE_STRING(args, 0);
# 2377 "QC_Program.qpp"
    ReferenceHolder<QoreListNode> vl(p->runtimeFindCallVariants(function->c_str(), xsink), xsink);
    if (!vl)
        return xsink->raiseException("PROGRAM-ERROR", "cannot find any function \"%s\"", function->c_str());
    return vl.release();
}

// *hash<auto> Program::getAllDefines(){}
static QoreValue Program_getAllDefines(QoreObject* self, QoreProgram* p, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1864 "QC_Program.qpp"
    return qore_program_private::runTimeGetAllDefines(p);
}

// code Program::getCallReference(string identifier){}
static QoreValue Program_getCallReference_Vs(QoreObject* self, QoreProgram* p, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* identifier = HARD_QORE_VALUE_STRING(args, 0);
# 2459 "QC_Program.qpp"
    QoreProgramContextHelper pch(p);
    return get_call_reference_intern(runtime_get_stack_object(), identifier, xsink);
}

// auto Program::getDefine(string def){}
static QoreValue Program_getDefine_Vs(QoreObject* self, QoreProgram* p, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* def = HARD_QORE_VALUE_STRING(args, 0);
# 1847 "QC_Program.qpp"
    TempEncodingHelper t(def, QCS_DEFAULT, xsink);
   return t ? qore_program_private::runTimeGetDefine(p, t->c_str()) : QoreValue();
}

// Expression Program::getExpression(string source, string label){}
static QoreValue Program_getExpression_VsVs(QoreObject* self, QoreProgram* p, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* source = HARD_QORE_VALUE_STRING(args, 0);
    const QoreStringNode* label = HARD_QORE_VALUE_STRING(args, 1);
# 2470 "QC_Program.qpp"
    ReferenceHolder<QoreExpression> exp(new QoreExpression(*p, *source, *label, xsink), xsink);
    if (!*xsink) {
        return new QoreObject(QC_EXPRESSION, p, exp.release());
    }
    return QoreValue();
}

// list<string> Program::getFeatureList(){}
static QoreValue Program_getFeatureList(QoreObject* self, QoreProgram* p, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 2483 "QC_Program.qpp"
    return p->getFeatureList();
}

// auto Program::getGlobalVariable(string varname, *reference<bool> rexists){}
static QoreValue Program_getGlobalVariable_VsC16_reference_bool_(QoreObject* self, QoreProgram* p, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* varname = HARD_QORE_VALUE_STRING(args, 0);
    const ReferenceNode* rexists = get_param_value(args, 1).get<const ReferenceNode>();
# 1701 "QC_Program.qpp"
    TempEncodingHelper t(varname, QCS_DEFAULT, xsink);
    if (!t)
        return QoreValue();

    bool found;
    ValueHolder rv(p->getGlobalVariableValue(t->c_str(), found), xsink);

    if (rexists) {
        QoreTypeSafeReferenceHelper r(rexists, xsink);
        if (!r)
            return QoreValue();

        if (r.assign(found))
            return QoreValue();
    }

    return rv.release();
}

// hash<auto> Program::getGlobalVars(){}
static QoreValue Program_getGlobalVars(QoreObject* self, QoreProgram* p, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 2339 "QC_Program.qpp"
    return p->getGlobalVars();
}

// list<string> Program::getParseOptionStringList(){}
static QoreValue Program_getParseOptionStringList(QoreObject* self, QoreProgram* p, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1554 "QC_Program.qpp"
    return parse_option_bitfield_to_string_list(p->getParseOptions64(), xsink);
}

// int Program::getParseOptions(){}
static QoreValue Program_getParseOptions(QoreObject* self, QoreProgram* p, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1540 "QC_Program.qpp"
    return p->getParseOptions64();
}

// ProgramControl Program::getProgram(){}
static QoreValue Program_getProgram(QoreObject* self, QoreProgram* p, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 2401 "QC_Program.qpp"
    return QoreProgram::getQoreObject(p);
}

// int Program::getProgramId(){}
static QoreValue Program_getProgramId(QoreObject* self, QoreProgram* p, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1875 "QC_Program.qpp"
    return p->getProgramId();
}

// *string Program::getScriptDir(){}
static QoreValue Program_getScriptDir(QoreObject* self, QoreProgram* p, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1645 "QC_Program.qpp"
    return p->getScriptDir();
}

// *string Program::getScriptName(){}
static QoreValue Program_getScriptName(QoreObject* self, QoreProgram* p, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1659 "QC_Program.qpp"
    return p->getScriptName();
}

// *string Program::getScriptPath(){}
static QoreValue Program_getScriptPath(QoreObject* self, QoreProgram* p, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1673 "QC_Program.qpp"
    return p->getScriptPath();
}

// list<int> Program::getThreadList(){}
static QoreValue Program_getThreadList(QoreObject* self, QoreProgram* p, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 2394 "QC_Program.qpp"
    return p->getThreadList();
}

// TimeZone Program::getTimeZone(){}
static QoreValue Program_getTimeZone(QoreObject* self, QoreProgram* p, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1785 "QC_Program.qpp"
    return new QoreObject(QC_TIMEZONE, 0, new TimeZoneData(p->currentTZ()));
}

// softlist<string> Program::getUserFunctionList(){}
static QoreValue Program_getUserFunctionList(QoreObject* self, QoreProgram* p, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1528 "QC_Program.qpp"
    return p->getUserFunctionList();
}

// nothing Program::importClass(string cls, *string new_name, *softbool inject, int module_visibility = CSP_UNCHANGED){}
static QoreValue Program_importClass_VsNsnbVi(QoreObject* self, QoreProgram* p, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* cls = HARD_QORE_VALUE_STRING(args, 0);
    const QoreStringNode* new_name = get_param_value(args, 1).get<const QoreStringNode>();
    bool inject = HARD_QORE_VALUE_BOOL(args, 2);
    int64 module_visibility = HARD_QORE_VALUE_INT(args, 3);
# 1460 "QC_Program.qpp"
    TempEncodingHelper tmp(cls, QCS_DEFAULT, xsink);
    if (!tmp) {
        assert(*xsink);
        return QoreValue();
    }

    TempEncodingHelper nn;
    if (new_name) {
        nn.set(new_name, QCS_DEFAULT, xsink);
        if (!nn) {
            assert(*xsink);
            return QoreValue();
        }

        if (nn->empty())
            return xsink->raiseException("CLASS-IMPORT-ERROR", "the new class name while importing class '%s' may not be empty", tmp->c_str());
    }
    qore_program_private::get(*p)->importClass(xsink, *qore_program_private::get(*getProgram()), tmp->c_str(), new_name ? nn->c_str() : nullptr, inject, static_cast<q_setpub_t>(module_visibility));
    return QoreValue();
}

// nothing Program::importFunction(string func){}
static QoreValue Program_importFunction_Vs(QoreObject* self, QoreProgram* p, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* func = HARD_QORE_VALUE_STRING(args, 0);
# 1363 "QC_Program.qpp"
    TempEncodingHelper tmp(func, QCS_DEFAULT, xsink);
    if (!tmp)
        return QoreValue();
    qore_program_private::exportFunction(getProgram(), xsink, p, tmp->getBuffer());
    return QoreValue();
}

// nothing Program::importFunction(string func, string new_name, *softbool inject){}
static QoreValue Program_importFunction_VsVsnb(QoreObject* self, QoreProgram* p, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* func = HARD_QORE_VALUE_STRING(args, 0);
    const QoreStringNode* new_name = HARD_QORE_VALUE_STRING(args, 1);
    bool inject = HARD_QORE_VALUE_BOOL(args, 2);
# 1398 "QC_Program.qpp"
    TempEncodingHelper tfunc(func, QCS_DEFAULT, xsink);
    if (!tfunc)
        return QoreValue();
    TempEncodingHelper tnew_name(new_name, QCS_DEFAULT, xsink);
    if (!tnew_name)
        return QoreValue();
    if (tnew_name->empty())
        xsink->raiseException("FUNCTION-IMPORT-ERROR", "the new function name while importing function '%s' may not be empty", func);

    qore_program_private::exportFunction(getProgram(), xsink, p, tfunc->getBuffer(), tnew_name->getBuffer(), inject);
    return QoreValue();
}

// nothing Program::importGlobalVariable(string varname, bool readonly = False){}
static QoreValue Program_importGlobalVariable_VsVb(QoreObject* self, QoreProgram* p, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* varname = HARD_QORE_VALUE_STRING(args, 0);
    bool readonly = HARD_QORE_VALUE_BOOL(args, 1);
# 1425 "QC_Program.qpp"
    TempEncodingHelper tmp(varname, QCS_DEFAULT, xsink);
    if (!tmp)
        return QoreValue();
    qore_program_private::exportGlobalVariable(getProgram(), tmp->getBuffer(), readonly, p, xsink);
    return QoreValue();
}

// nothing Program::importHashDecl(string name, *string new_name){}
static QoreValue Program_importHashDecl_VsNs(QoreObject* self, QoreProgram* p, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* name = HARD_QORE_VALUE_STRING(args, 0);
    const QoreStringNode* new_name = get_param_value(args, 1).get<const QoreStringNode>();
# 1497 "QC_Program.qpp"
    TempEncodingHelper tmp(name, QCS_DEFAULT, xsink);
    if (!tmp) {
        assert(*xsink);
        return QoreValue();
    }

    TempEncodingHelper nn;
    if (new_name) {
        nn.set(new_name, QCS_DEFAULT, xsink);
        if (!nn) {
            assert(*xsink);
            return QoreValue();
        }

        if (nn->empty())
            return xsink->raiseException("HASHDECL-IMPORT-ERROR", "the new hashdecl name while importing hashdecl '%s' may not be empty", tmp->c_str());
    }
    qore_program_private::get(*p)->importHashDecl(xsink, *qore_program_private::get(*getProgram()), tmp->c_str(), new_name ? nn->c_str() : nullptr);
    return QoreValue();
}

// nothing Program::importSystemApi(){}
static QoreValue Program_importSystemApi(QoreObject* self, QoreProgram* p, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 2301 "QC_Program.qpp"
    qore_program_private::runtimeImportSystemApi(*p, xsink);
    return QoreValue();
}

// nothing Program::importSystemClasses(){}
static QoreValue Program_importSystemClasses(QoreObject* self, QoreProgram* p, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 2237 "QC_Program.qpp"
    qore_program_private::runtimeImportSystemClasses(*p, xsink);
    return QoreValue();
}

// nothing Program::importSystemConstants(){}
static QoreValue Program_importSystemConstants(QoreObject* self, QoreProgram* p, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 2269 "QC_Program.qpp"
    qore_program_private::runtimeImportSystemConstants(*p, xsink);
    return QoreValue();
}

// nothing Program::importSystemFunctions(){}
static QoreValue Program_importSystemFunctions(QoreObject* self, QoreProgram* p, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 2285 "QC_Program.qpp"
    qore_program_private::runtimeImportSystemFunctions(*p, xsink);
    return QoreValue();
}

// nothing Program::importSystemHashDecls(){}
static QoreValue Program_importSystemHashDecls(QoreObject* self, QoreProgram* p, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 2253 "QC_Program.qpp"
    qore_program_private::runtimeImportSystemHashDecls(*p, xsink);
    return QoreValue();
}

// bool Program::isDefined(string def){}
static QoreValue Program_isDefined_Vs(QoreObject* self, QoreProgram* p, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* def = HARD_QORE_VALUE_STRING(args, 0);
# 1830 "QC_Program.qpp"
    TempEncodingHelper t(def, QCS_DEFAULT, xsink);
   return t ? qore_program_private::runTimeIsDefined(p, t->getBuffer()) : false;
}

// nothing Program::issueModuleCmd(string module, string cmd){}
static QoreValue Program_issueModuleCmd_VsVs(QoreObject* self, QoreProgram* p, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* module = HARD_QORE_VALUE_STRING(args, 0);
    const QoreStringNode* cmd = HARD_QORE_VALUE_STRING(args, 1);
# 2421 "QC_Program.qpp"
    TempEncodingHelper tmp_module(module, QCS_DEFAULT, xsink);
    if (!tmp_module) {
        return QoreValue();
    }

    TempEncodingHelper tmp_cmd(cmd, QCS_DEFAULT, xsink);
    if (!tmp_cmd) {
        return QoreValue();
    }

    QMM.issueRuntimeCmd(tmp_module->c_str(), p, **tmp_cmd, xsink);
    return QoreValue();
}

// nothing Program::loadApplyToPrivateUserModule(string name, int warning_mask = WARN_MODULES){}
static QoreValue Program_loadApplyToPrivateUserModule_VsVi(QoreObject* self, QoreProgram* p, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* name = HARD_QORE_VALUE_STRING(args, 0);
    int64 warning_mask = HARD_QORE_VALUE_INT(args, 1);
# 2157 "QC_Program.qpp"
    // remove the QoreProgram private data from the object
    printd(5, "Program::loadApplyToPrivateUserModule pgm: %p, pgmid: %d, self: %p\n", p, p->getProgramId(), self);
#ifdef DEBUG
    QoreProgram* pgm = (QoreProgram*)
#endif
    qore_object_private::get(*self)->getAndRemovePrivateData(CID_PROGRAM, xsink);
#ifdef DEBUG
    assert(pgm == p);
#endif
    if (!*xsink) {
        QMM.runTimeLoadModule(*xsink, *xsink, name->c_str(), 0, p, QMLO_INJECT | QMLO_PRIVATE, warning_mask);
        // getAndRemovePrivateData removed private data, so destructor is not executed. We must remove reference to this object
        //p->unregisterQoreObject(self, xsink);
        self->doDelete(xsink);
    }
    return QoreValue();
}

// *hash<ExceptionInfo> Program::loadApplyToPrivateUserModuleWarn(string name, int warning_mask = WARN_MODULES){}
static QoreValue Program_loadApplyToPrivateUserModuleWarn_VsVi(QoreObject* self, QoreProgram* p, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* name = HARD_QORE_VALUE_STRING(args, 0);
    int64 warning_mask = HARD_QORE_VALUE_INT(args, 1);
# 2202 "QC_Program.qpp"
    // remove the QoreProgram private data from the object
    printd(5, "Program::loadApplyToPrivateUserModule pgm: %p, pgmid: %d, self: %p\n", p, p->getProgramId(), self);
#ifdef DEBUG
    QoreProgram* pgm = (QoreProgram*)
#endif
    qore_object_private::get(*self)->getAndRemovePrivateData(CID_PROGRAM, xsink);
#ifdef DEBUG
    assert(pgm == p);
#endif
    if (!*xsink) {
        ExceptionSink wsink;
        QMM.runTimeLoadModule(*xsink, wsink, name->c_str(), 0, p, QMLO_INJECT | QMLO_PRIVATE, warning_mask);
        // getAndRemovePrivateData removed private data, so destructor is not executed. We must remove reference to this object
        //p->unregisterQoreObject(self, xsink);
        self->doDelete(xsink);
        if (!*xsink && wsink) {
            return wsink.catchException()->makeExceptionObjectAndDelete(xsink);
        }
    }
    return QoreValue();
}

// nothing Program::loadApplyToUserModule(string name, *softbool reinject, int warning_mask = WARN_MODULES, bool reexport = False){}
static QoreValue Program_loadApplyToUserModule_VsnbViVb(QoreObject* self, QoreProgram* p, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* name = HARD_QORE_VALUE_STRING(args, 0);
    bool reinject = HARD_QORE_VALUE_BOOL(args, 1);
    int64 warning_mask = HARD_QORE_VALUE_INT(args, 2);
    bool reexport = HARD_QORE_VALUE_BOOL(args, 3);
# 2058 "QC_Program.qpp"
    // remove the QoreProgram private data from the object
    printd(5, "Program::loadApplyToUserModule pgm: %p, pgmid: %d, self: %p\n", p, p->getProgramId(), self);
#ifdef DEBUG
    QoreProgram* pgm = (QoreProgram*)
#endif
    qore_object_private::get(*self)->getAndRemovePrivateData(CID_PROGRAM, xsink);
#ifdef DEBUG
    assert(pgm == p);
#endif
    if (!*xsink) {
        QMM.runTimeLoadModule(*xsink, *xsink, name->c_str(), 0, p, QMLO_INJECT | (reinject ? QMLO_REINJECT : 0),
            warning_mask, reexport);
        // getAndRemovePrivateData removed private data, so destructor is not executed. We must remove reference to this object
        //p->unregisterQoreObject(self, xsink);
        self->doDelete(xsink);
    }
    return QoreValue();
}

// *hash<ExceptionInfo> Program::loadApplyToUserModuleWarn(string name, *softbool reinject, int warning_mask = WARN_MODULES, bool reexport = False){}
static QoreValue Program_loadApplyToUserModuleWarn_VsnbViVb(QoreObject* self, QoreProgram* p, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* name = HARD_QORE_VALUE_STRING(args, 0);
    bool reinject = HARD_QORE_VALUE_BOOL(args, 1);
    int64 warning_mask = HARD_QORE_VALUE_INT(args, 2);
    bool reexport = HARD_QORE_VALUE_BOOL(args, 3);
# 2108 "QC_Program.qpp"
    // remove the QoreProgram private data from the object
    printd(5, "Program::loadApplyToUserModule pgm: %p, pgmid: %d, self: %p\n", p, p->getProgramId(), self);
#ifdef DEBUG
    QoreProgram* pgm = (QoreProgram*)
#endif
    qore_object_private::get(*self)->getAndRemovePrivateData(CID_PROGRAM, xsink);
#ifdef DEBUG
    assert(pgm == p);
#endif
    if (!*xsink) {
        ExceptionSink wsink;
        QMM.runTimeLoadModule(*xsink, wsink, name->c_str(), 0, p, QMLO_INJECT | (reinject ? QMLO_REINJECT : 0),
            warning_mask, reexport);
        // getAndRemovePrivateData removed private data, so destructor is not executed. We must remove reference to this object
        //p->unregisterQoreObject(self, xsink);
        self->doDelete(xsink);
        if (!*xsink && wsink) {
            return wsink.catchException()->makeExceptionObjectAndDelete(xsink);
        }
    }
    return QoreValue();
}

// nothing Program::loadModule(string name, int warning_mask = WARN_MODULES){}
static QoreValue Program_loadModule_VsVi(QoreObject* self, QoreProgram* p, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* name = HARD_QORE_VALUE_STRING(args, 0);
    int64 warning_mask = HARD_QORE_VALUE_INT(args, 1);
# 1909 "QC_Program.qpp"
    QMM.runTimeLoadModule(*xsink, *xsink, name->c_str(), p, nullptr, QMLO_NONE, warning_mask);
    return QoreValue();
}

// *hash<ExceptionInfo> Program::loadModuleWarn(string name, int warning_mask = WARN_MODULES){}
static QoreValue Program_loadModuleWarn_VsVi(QoreObject* self, QoreProgram* p, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* name = HARD_QORE_VALUE_STRING(args, 0);
    int64 warning_mask = HARD_QORE_VALUE_INT(args, 1);
# 1943 "QC_Program.qpp"
    ExceptionSink wsink;
    QMM.runTimeLoadModule(*xsink, wsink, name->c_str(), p, nullptr, QMLO_NONE, warning_mask);
    if (!*xsink && wsink) {
        return wsink.catchException()->makeExceptionObjectAndDelete(xsink);
    }
    return QoreValue();
}

// nothing Program::loadUserModuleWithProgram(string name, Qore::Program pgm, int warning_mask = WARN_MODULES){}
static QoreValue Program_loadUserModuleWithProgram_VsC7ProgramVi(QoreObject* self, QoreProgram* p, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* name = HARD_QORE_VALUE_STRING(args, 0);
    TAKE_HARD_QORE_VALUE_OBJ_DATA(pgm, QoreProgram, args, 1, CID_PROGRAM, "Program::loadUserModuleWithProgram()", "Qore::Program", xsink);
    if (*xsink)
        return 0;
    int64 warning_mask = HARD_QORE_VALUE_INT(args, 2);
# 1984 "QC_Program.qpp"
    QMM.runTimeLoadModule(*xsink, *xsink, name->c_str(), p, pgm, QMLO_INJECT, warning_mask);
    return QoreValue();
}

// *hash<ExceptionInfo> Program::loadUserModuleWithProgramWarn(string name, Qore::Program pgm, int warning_mask = WARN_MODULES){}
static QoreValue Program_loadUserModuleWithProgramWarn_VsC7ProgramVi(QoreObject* self, QoreProgram* p, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* name = HARD_QORE_VALUE_STRING(args, 0);
    TAKE_HARD_QORE_VALUE_OBJ_DATA(pgm, QoreProgram, args, 1, CID_PROGRAM, "Program::loadUserModuleWithProgramWarn()", "Qore::Program", xsink);
    if (*xsink)
        return 0;
    int64 warning_mask = HARD_QORE_VALUE_INT(args, 2);
# 2020 "QC_Program.qpp"
    ExceptionSink wsink;
    QMM.runTimeLoadModule(*xsink, wsink, name->c_str(), p, pgm, QMLO_INJECT, warning_mask);
    if (!*xsink && wsink) {
        return wsink.catchException()->makeExceptionObjectAndDelete(xsink);
    }
    return QoreValue();
}

// nothing Program::lockOptions(){}
static QoreValue Program_lockOptions(QoreObject* self, QoreProgram* p, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1684 "QC_Program.qpp"
    p->lockOptions();
    return QoreValue();
}

// *hash<ExceptionInfo> Program::parse(string code, string label, *softint warning_mask, *string source, *softint offset, softbool format_label = True){}
static QoreValue Program_parse_VsVsniNsnivb(QoreObject* self, QoreProgram* p, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* code = HARD_QORE_VALUE_STRING(args, 0);
    const QoreStringNode* label = HARD_QORE_VALUE_STRING(args, 1);
    int64 warning_mask = HARD_QORE_VALUE_INT(args, 2);
    const QoreStringNode* source = get_param_value(args, 3).get<const QoreStringNode>();
    int64 offset = HARD_QORE_VALUE_INT(args, 4);
# 1037 "QC_Program.qpp"
    if (warning_mask) {
        ExceptionSink wsink;
        p->parse(code, label, xsink, &wsink, warning_mask, source, offset);
        if (!wsink.isException())
            return QoreValue();

        QoreException *e = wsink.catchException();
        return e->makeExceptionObjectAndDelete(xsink);
    }

    p->parse(code, label, xsink, 0, 0, source, offset);
    return QoreValue();
}

// nothing Program::parseCommit(){}
static QoreValue Program_parseCommit(QoreObject* self, QoreProgram* p, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1136 "QC_Program.qpp"
    p->parseCommit(xsink);
    return QoreValue();
}

// *hash<ExceptionInfo> Program::parseCommit(int warning_mask){}
static QoreValue Program_parseCommit_Vi(QoreObject* self, QoreProgram* p, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 warning_mask = HARD_QORE_VALUE_INT(args, 0);
# 1168 "QC_Program.qpp"
    ExceptionSink wsink;
    p->parseCommit(xsink, &wsink, warning_mask);
    if (!wsink.isException())
        return QoreValue();

    QoreException *e = wsink.catchException();
    return e->makeExceptionObjectAndDelete(xsink);
}

// *hash<ExceptionInfo> Program::parsePending(string code, string label, *softint warning_mask, *string source, *softint offset, softbool format_label = True){}
static QoreValue Program_parsePending_VsVsniNsnivb(QoreObject* self, QoreProgram* p, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* code = HARD_QORE_VALUE_STRING(args, 0);
    const QoreStringNode* label = HARD_QORE_VALUE_STRING(args, 1);
    int64 warning_mask = HARD_QORE_VALUE_INT(args, 2);
    const QoreStringNode* source = get_param_value(args, 3).get<const QoreStringNode>();
    int64 offset = HARD_QORE_VALUE_INT(args, 4);
# 1096 "QC_Program.qpp"
    if (warning_mask) {
        ExceptionSink wsink;
        p->parsePending(code, label, xsink, &wsink, warning_mask, source, offset);
        if (!wsink.isException())
            return QoreValue();

        QoreException *e = wsink.catchException();
        return e->makeExceptionObjectAndDelete(xsink);
    }

    p->parsePending(code, label, xsink, 0, 0, source, offset);
    return QoreValue();
}

// nothing Program::parseRollback(){}
static QoreValue Program_parseRollback(QoreObject* self, QoreProgram* p, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1190 "QC_Program.qpp"
    p->parseRollback(xsink);
    return QoreValue();
}

// nothing Program::replaceParseOptions(softint opt){}
static QoreValue Program_replaceParseOptions_vi(QoreObject* self, QoreProgram* p, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 opt = HARD_QORE_VALUE_INT(args, 0);
# 1619 "QC_Program.qpp"
    p->replaceParseOptions(opt, xsink);
    return QoreValue();
}

// auto Program::run(){}
static QoreValue Program_run(QoreObject* self, QoreProgram* p, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1343 "QC_Program.qpp"
    return p->run(xsink);
}

// nothing Program::setGlobalVarValue(string name, auto value){}
static QoreValue Program_setGlobalVarValue_VsVA(QoreObject* self, QoreProgram* p, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* name = HARD_QORE_VALUE_STRING(args, 0);
    QoreValue value = get_param_value(args, 1);
# 2360 "QC_Program.qpp"
    p->setGlobalVarValue(name->c_str(), value.refSelf(), xsink);
    return QoreValue();
}

// nothing Program::setParseOptions(softint opt = PO_DEFAULT){}
static QoreValue Program_setParseOptions_vi(QoreObject* self, QoreProgram* p, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 opt = HARD_QORE_VALUE_INT(args, 0);
# 1574 "QC_Program.qpp"
    if (opt & PO_SYSTEM_OPS)
      return xsink->raiseException("PROGRAM-OPTION-ERROR", "parse options (0x" QLLX ") contain restricted options that can only be set by the system", opt);

   p->setParseOptions(opt, xsink);
    return QoreValue();
}

// nothing Program::setScriptPath(*string path){}
static QoreValue Program_setScriptPath_Ns(QoreObject* self, QoreProgram* p, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* path = get_param_value(args, 0).get<const QoreStringNode>();
# 1631 "QC_Program.qpp"
    p->setScriptPath(path ? path->c_str() : nullptr);
    return QoreValue();
}

// bool Program::setThreadInit(*code init){}
static QoreValue Program_setThreadInit_Nc(QoreObject* self, QoreProgram* p, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const ResolvedCallReferenceNode* init = get_param_value(args, 0).get<const ResolvedCallReferenceNode>();
# 2323 "QC_Program.qpp"
    return qore_program_private::setThreadInit(*p, init, xsink);
}

// nothing Program::setTimeZone(TimeZone zone){}
static QoreValue Program_setTimeZone_C8TimeZone(QoreObject* self, QoreProgram* p, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    HARD_QORE_VALUE_OBJ_DATA(zone, TimeZoneData, args, 0, CID_TIMEZONE, "Program::setTimeZone()", "TimeZone", xsink);
    if (*xsink)
        return 0;
# 1773 "QC_Program.qpp"
    p->setTZ(zone->get());
    return QoreValue();
}

// nothing Program::setTimeZoneRegion(string region){}
static QoreValue Program_setTimeZoneRegion_Vs(QoreObject* self, QoreProgram* p, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* region = HARD_QORE_VALUE_STRING(args, 0);
# 1733 "QC_Program.qpp"
    const AbstractQoreZoneInfo *zone = QTZM.findLoadRegion(region->getBuffer(), xsink);
   if (!*xsink)
      p->setTZ(zone);
    return QoreValue();
}

// nothing Program::setTimeZoneUTCOffset(softint seconds_east){}
static QoreValue Program_setTimeZoneUTCOffset_vi(QoreObject* self, QoreProgram* p, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 seconds_east = HARD_QORE_VALUE_INT(args, 0);
# 1758 "QC_Program.qpp"
    const AbstractQoreZoneInfo *zone = QTZM.findCreateOffsetZone(seconds_east);
   p->setTZ(zone);
    return QoreValue();
}

// nothing Program::undefine(string def){}
static QoreValue Program_undefine_Vs(QoreObject* self, QoreProgram* p, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* def = HARD_QORE_VALUE_STRING(args, 0);
# 1812 "QC_Program.qpp"
    TempEncodingHelper t(def, QCS_DEFAULT, xsink);
   if (t)
      qore_program_private::runTimeUnDefine(p, t->getBuffer(), xsink);
    return QoreValue();
}

DLLLOCAL void preinitProgramClass() {
    QC_PROGRAM = new QoreClass("Program", "::Qore::Program", QDOM_DEFAULT);
    CID_PROGRAM = QC_PROGRAM->getID();
    QC_PROGRAM->setSystem();
}

DLLLOCAL QoreClass* initProgramClass(QoreNamespace& ns) {
    if (!QC_PROGRAM)
        preinitProgramClass();

    // auto Program::callFunction(string name, ...){}
    QC_PROGRAM->addMethod("callFunction", (q_method_n_t)Program_callFunction_VsVV, Public, QCF_USES_EXTRA_ARGS, QDOM_DEFAULT, autoTypeInfo, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "name");

    // auto Program::callFunctionArgs(string name, *softlist<auto> vargs){}
    QC_PROGRAM->addMethod("callFunctionArgs", (q_method_n_t)Program_callFunctionArgs_VsC15_softlist_auto_, Public, QCF_NO_FLAGS, QDOM_DEFAULT, autoTypeInfo, 2, stringTypeInfo, QORE_PARAM_NO_ARG, "name", softAutoListOrNothingTypeInfo, QORE_PARAM_NO_ARG, "vargs");

    // auto Program::callStaticMethod(string class_name, string method, ...){}
    QC_PROGRAM->addMethod("callStaticMethod", (q_method_n_t)Program_callStaticMethod_VsVsVV, Public, QCF_USES_EXTRA_ARGS, QDOM_DEFAULT, autoTypeInfo, 2, stringTypeInfo, QORE_PARAM_NO_ARG, "class_name", stringTypeInfo, QORE_PARAM_NO_ARG, "method");

    // auto Program::callStaticMethodArgs(string class_name, string method, *softlist<auto> call_args){}
    QC_PROGRAM->addMethod("callStaticMethodArgs", (q_method_n_t)Program_callStaticMethodArgs_VsVsC15_softlist_auto_, Public, QCF_NO_FLAGS, QDOM_DEFAULT, autoTypeInfo, 3, stringTypeInfo, QORE_PARAM_NO_ARG, "class_name", stringTypeInfo, QORE_PARAM_NO_ARG, "method", softAutoListOrNothingTypeInfo, QORE_PARAM_NO_ARG, "call_args");

    // Program::constructor(softint po = PO_DEFAULT) {}
    QC_PROGRAM->addConstructor(Program_constructor_vi, Public, QCF_NO_FLAGS, QDOM_EMBEDDED_LOGIC, 1, softBigIntTypeInfo, QoreSimpleValue().assign((int64)PO_DEFAULT), "po");

    // Program::copy() {}
    QC_PROGRAM->setCopy((q_copy_t)Program_copy);

    // nothing Program::define(string def, auto val){}
    QC_PROGRAM->addMethod("define", (q_method_n_t)Program_define_VsVA, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 2, stringTypeInfo, QORE_PARAM_NO_ARG, "def", autoTypeInfo, QORE_PARAM_NO_ARG, "val");

    // Program::destructor() {}
    QC_PROGRAM->setDestructor((q_destructor_t)Program_destructor);

    // nothing Program::disableParseOptions(softint opt){}
    QC_PROGRAM->addMethod("disableParseOptions", (q_method_n_t)Program_disableParseOptions_vi, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, softBigIntTypeInfo, QORE_PARAM_NO_ARG, "opt");

    // bool Program::existsFunction(string name){}
    QC_PROGRAM->addMethod("existsFunction", (q_method_n_t)Program_existsFunction_Vs, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, boolTypeInfo, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "name");

    // list<hash<auto>> Program::findFunctionVariants(string function){}
    QC_PROGRAM->addMethod("findFunctionVariants", (q_method_n_t)Program_findFunctionVariants_Vs, Public, QCF_NO_FLAGS, QDOM_DEFAULT, qore_get_complex_list_type(autoHashTypeInfo), 1, stringTypeInfo, QORE_PARAM_NO_ARG, "function");

    // *hash<auto> Program::getAllDefines(){}
    QC_PROGRAM->addMethod("getAllDefines", (q_method_n_t)Program_getAllDefines, Public, QCF_CONSTANT, QDOM_DEFAULT, autoHashOrNothingTypeInfo);

    // code Program::getCallReference(string identifier){}
    QC_PROGRAM->addMethod("getCallReference", (q_method_n_t)Program_getCallReference_Vs, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, codeTypeInfo, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "identifier");

    // auto Program::getDefine(string def){}
    QC_PROGRAM->addMethod("getDefine", (q_method_n_t)Program_getDefine_Vs, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, autoTypeInfo, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "def");

    // Expression Program::getExpression(string source, string label){}
    QC_PROGRAM->addMethod("getExpression", (q_method_n_t)Program_getExpression_VsVs, Public, QCF_NO_FLAGS, QDOM_DEFAULT, QC_EXPRESSION->getTypeInfo(), 2, stringTypeInfo, QORE_PARAM_NO_ARG, "source", stringTypeInfo, QORE_PARAM_NO_ARG, "label");

    // list<string> Program::getFeatureList(){}
    QC_PROGRAM->addMethod("getFeatureList", (q_method_n_t)Program_getFeatureList, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, qore_get_complex_list_type(stringTypeInfo));

    // auto Program::getGlobalVariable(string varname, *reference<bool> rexists){}
    QC_PROGRAM->addMethod("getGlobalVariable", (q_method_n_t)Program_getGlobalVariable_VsC16_reference_bool_, Public, QCF_NO_FLAGS, QDOM_DEFAULT, autoTypeInfo, 2, stringTypeInfo, QORE_PARAM_NO_ARG, "varname", qore_get_complex_reference_or_nothing_type(boolTypeInfo), QORE_PARAM_NO_ARG, "rexists");

    // hash<auto> Program::getGlobalVars(){}
    QC_PROGRAM->addMethod("getGlobalVars", (q_method_n_t)Program_getGlobalVars, Public, QCF_CONSTANT, QDOM_DEFAULT, autoHashTypeInfo);

    // list<string> Program::getParseOptionStringList(){}
    QC_PROGRAM->addMethod("getParseOptionStringList", (q_method_n_t)Program_getParseOptionStringList, Public, QCF_CONSTANT, QDOM_DEFAULT, qore_get_complex_list_type(stringTypeInfo));

    // int Program::getParseOptions(){}
    QC_PROGRAM->addMethod("getParseOptions", (q_method_n_t)Program_getParseOptions, Public, QCF_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

    // ProgramControl Program::getProgram(){}
    QC_PROGRAM->addMethod("getProgram", (q_method_n_t)Program_getProgram, Public, QCF_RET_VALUE_ONLY, QDOM_DEBUGGER, QC_PROGRAMCONTROL->getTypeInfo());

    // int Program::getProgramId(){}
    QC_PROGRAM->addMethod("getProgramId", (q_method_n_t)Program_getProgramId, Public, QCF_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

    // *string Program::getScriptDir(){}
    QC_PROGRAM->addMethod("getScriptDir", (q_method_n_t)Program_getScriptDir, Public, QCF_CONSTANT, QDOM_DEFAULT, stringOrNothingTypeInfo);

    // *string Program::getScriptName(){}
    QC_PROGRAM->addMethod("getScriptName", (q_method_n_t)Program_getScriptName, Public, QCF_CONSTANT, QDOM_DEFAULT, stringOrNothingTypeInfo);

    // *string Program::getScriptPath(){}
    QC_PROGRAM->addMethod("getScriptPath", (q_method_n_t)Program_getScriptPath, Public, QCF_CONSTANT, QDOM_DEFAULT, stringOrNothingTypeInfo);

    // list<int> Program::getThreadList(){}
    QC_PROGRAM->addMethod("getThreadList", (q_method_n_t)Program_getThreadList, Public, QCF_CONSTANT, QDOM_DEFAULT, qore_get_complex_list_type(bigIntTypeInfo));

    // TimeZone Program::getTimeZone(){}
    QC_PROGRAM->addMethod("getTimeZone", (q_method_n_t)Program_getTimeZone, Public, QCF_CONSTANT, QDOM_DEFAULT, QC_TIMEZONE->getTypeInfo());

    // softlist<string> Program::getUserFunctionList(){}
    QC_PROGRAM->addMethod("getUserFunctionList", (q_method_n_t)Program_getUserFunctionList, Public, QCF_CONSTANT, QDOM_DEFAULT, qore_get_complex_softlist_type(stringTypeInfo));

    // nothing Program::importClass(string cls, *string new_name, *softbool inject, int module_visibility = CSP_UNCHANGED){}
    QC_PROGRAM->addMethod("importClass", (q_method_n_t)Program_importClass_VsNsnbVi, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 4, stringTypeInfo, QORE_PARAM_NO_ARG, "cls", stringOrNothingTypeInfo, QORE_PARAM_NO_ARG, "new_name", softBoolOrNothingTypeInfo, QORE_PARAM_NO_ARG, "inject", bigIntTypeInfo, QoreSimpleValue().assign((int64)CSP_UNCHANGED), "module_visibility");

    // nothing Program::importFunction(string func){}
    QC_PROGRAM->addMethod("importFunction", (q_method_n_t)Program_importFunction_Vs, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "func");

    // nothing Program::importFunction(string func, string new_name, *softbool inject){}
    QC_PROGRAM->addMethod("importFunction", (q_method_n_t)Program_importFunction_VsVsnb, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 3, stringTypeInfo, QORE_PARAM_NO_ARG, "func", stringTypeInfo, QORE_PARAM_NO_ARG, "new_name", softBoolOrNothingTypeInfo, QORE_PARAM_NO_ARG, "inject");

    // nothing Program::importGlobalVariable(string varname, bool readonly = False){}
    QC_PROGRAM->addMethod("importGlobalVariable", (q_method_n_t)Program_importGlobalVariable_VsVb, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 2, stringTypeInfo, QORE_PARAM_NO_ARG, "varname", boolTypeInfo, QoreSimpleValue().assign(false), "readonly");

    // nothing Program::importHashDecl(string name, *string new_name){}
    QC_PROGRAM->addMethod("importHashDecl", (q_method_n_t)Program_importHashDecl_VsNs, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 2, stringTypeInfo, QORE_PARAM_NO_ARG, "name", stringOrNothingTypeInfo, QORE_PARAM_NO_ARG, "new_name");

    // nothing Program::importSystemApi(){}
    QC_PROGRAM->addMethod("importSystemApi", (q_method_n_t)Program_importSystemApi, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

    // nothing Program::importSystemClasses(){}
    QC_PROGRAM->addMethod("importSystemClasses", (q_method_n_t)Program_importSystemClasses, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

    // nothing Program::importSystemConstants(){}
    QC_PROGRAM->addMethod("importSystemConstants", (q_method_n_t)Program_importSystemConstants, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

    // nothing Program::importSystemFunctions(){}
    QC_PROGRAM->addMethod("importSystemFunctions", (q_method_n_t)Program_importSystemFunctions, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

    // nothing Program::importSystemHashDecls(){}
    QC_PROGRAM->addMethod("importSystemHashDecls", (q_method_n_t)Program_importSystemHashDecls, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

    // bool Program::isDefined(string def){}
    QC_PROGRAM->addMethod("isDefined", (q_method_n_t)Program_isDefined_Vs, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, boolTypeInfo, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "def");

    // nothing Program::issueModuleCmd(string module, string cmd){}
    QC_PROGRAM->addMethod("issueModuleCmd", (q_method_n_t)Program_issueModuleCmd_VsVs, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 2, stringTypeInfo, QORE_PARAM_NO_ARG, "module", stringTypeInfo, QORE_PARAM_NO_ARG, "cmd");

    // nothing Program::loadApplyToPrivateUserModule(string name, int warning_mask = WARN_MODULES){}
    QC_PROGRAM->addMethod("loadApplyToPrivateUserModule", (q_method_n_t)Program_loadApplyToPrivateUserModule_VsVi, Public, QCF_NO_FLAGS, QDOM_MODULES, nothingTypeInfo, 2, stringTypeInfo, QORE_PARAM_NO_ARG, "name", bigIntTypeInfo, QoreSimpleValue().assign((int64)WARN_MODULES), "warning_mask");

    // *hash<ExceptionInfo> Program::loadApplyToPrivateUserModuleWarn(string name, int warning_mask = WARN_MODULES){}
    QC_PROGRAM->addMethod("loadApplyToPrivateUserModuleWarn", (q_method_n_t)Program_loadApplyToPrivateUserModuleWarn_VsVi, Public, QCF_NO_FLAGS, QDOM_MODULES, hashdeclExceptionInfo->getTypeInfo(true), 2, stringTypeInfo, QORE_PARAM_NO_ARG, "name", bigIntTypeInfo, QoreSimpleValue().assign((int64)WARN_MODULES), "warning_mask");

    // nothing Program::loadApplyToUserModule(string name, *softbool reinject, int warning_mask = WARN_MODULES, bool reexport = False){}
    QC_PROGRAM->addMethod("loadApplyToUserModule", (q_method_n_t)Program_loadApplyToUserModule_VsnbViVb, Public, QCF_NO_FLAGS, QDOM_MODULES|QDOM_INJECTION, nothingTypeInfo, 4, stringTypeInfo, QORE_PARAM_NO_ARG, "name", softBoolOrNothingTypeInfo, QORE_PARAM_NO_ARG, "reinject", bigIntTypeInfo, QoreSimpleValue().assign((int64)WARN_MODULES), "warning_mask", boolTypeInfo, QoreSimpleValue().assign(false), "reexport");

    // *hash<ExceptionInfo> Program::loadApplyToUserModuleWarn(string name, *softbool reinject, int warning_mask = WARN_MODULES, bool reexport = False){}
    QC_PROGRAM->addMethod("loadApplyToUserModuleWarn", (q_method_n_t)Program_loadApplyToUserModuleWarn_VsnbViVb, Public, QCF_NO_FLAGS, QDOM_MODULES|QDOM_INJECTION, hashdeclExceptionInfo->getTypeInfo(true), 4, stringTypeInfo, QORE_PARAM_NO_ARG, "name", softBoolOrNothingTypeInfo, QORE_PARAM_NO_ARG, "reinject", bigIntTypeInfo, QoreSimpleValue().assign((int64)WARN_MODULES), "warning_mask", boolTypeInfo, QoreSimpleValue().assign(false), "reexport");

    // nothing Program::loadModule(string name, int warning_mask = WARN_MODULES){}
    QC_PROGRAM->addMethod("loadModule", (q_method_n_t)Program_loadModule_VsVi, Public, QCF_NO_FLAGS, QDOM_MODULES, nothingTypeInfo, 2, stringTypeInfo, QORE_PARAM_NO_ARG, "name", bigIntTypeInfo, QoreSimpleValue().assign((int64)WARN_MODULES), "warning_mask");

    // *hash<ExceptionInfo> Program::loadModuleWarn(string name, int warning_mask = WARN_MODULES){}
    QC_PROGRAM->addMethod("loadModuleWarn", (q_method_n_t)Program_loadModuleWarn_VsVi, Public, QCF_NO_FLAGS, QDOM_MODULES, hashdeclExceptionInfo->getTypeInfo(true), 2, stringTypeInfo, QORE_PARAM_NO_ARG, "name", bigIntTypeInfo, QoreSimpleValue().assign((int64)WARN_MODULES), "warning_mask");

    // nothing Program::loadUserModuleWithProgram(string name, Qore::Program pgm, int warning_mask = WARN_MODULES){}
    QC_PROGRAM->addMethod("loadUserModuleWithProgram", (q_method_n_t)Program_loadUserModuleWithProgram_VsC7ProgramVi, Public, QCF_NO_FLAGS, QDOM_MODULES|QDOM_INJECTION, nothingTypeInfo, 3, stringTypeInfo, QORE_PARAM_NO_ARG, "name", QC_PROGRAM->getTypeInfo(), QORE_PARAM_NO_ARG, "pgm", bigIntTypeInfo, QoreSimpleValue().assign((int64)WARN_MODULES), "warning_mask");

    // *hash<ExceptionInfo> Program::loadUserModuleWithProgramWarn(string name, Qore::Program pgm, int warning_mask = WARN_MODULES){}
    QC_PROGRAM->addMethod("loadUserModuleWithProgramWarn", (q_method_n_t)Program_loadUserModuleWithProgramWarn_VsC7ProgramVi, Public, QCF_NO_FLAGS, QDOM_MODULES|QDOM_INJECTION, hashdeclExceptionInfo->getTypeInfo(true), 3, stringTypeInfo, QORE_PARAM_NO_ARG, "name", QC_PROGRAM->getTypeInfo(), QORE_PARAM_NO_ARG, "pgm", bigIntTypeInfo, QoreSimpleValue().assign((int64)WARN_MODULES), "warning_mask");

    // nothing Program::lockOptions(){}
    QC_PROGRAM->addMethod("lockOptions", (q_method_n_t)Program_lockOptions, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

    // *hash<ExceptionInfo> Program::parse(string code, string label, *softint warning_mask, *string source, *softint offset, softbool format_label = True){}
    QC_PROGRAM->addMethod("parse", (q_method_n_t)Program_parse_VsVsniNsnivb, Public, QCF_NO_FLAGS, QDOM_EMBEDDED_LOGIC, hashdeclExceptionInfo->getTypeInfo(true), 6, stringTypeInfo, QORE_PARAM_NO_ARG, "code", stringTypeInfo, QORE_PARAM_NO_ARG, "label", softBigIntOrNothingTypeInfo, QORE_PARAM_NO_ARG, "warning_mask", stringOrNothingTypeInfo, QORE_PARAM_NO_ARG, "source", softBigIntOrNothingTypeInfo, QORE_PARAM_NO_ARG, "offset", softBoolTypeInfo, QoreSimpleValue().assign(true), "format_label");

    // nothing Program::parseCommit(){}
    QC_PROGRAM->addMethod("parseCommit", (q_method_n_t)Program_parseCommit, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

    // *hash<ExceptionInfo> Program::parseCommit(int warning_mask){}
    QC_PROGRAM->addMethod("parseCommit", (q_method_n_t)Program_parseCommit_Vi, Public, QCF_NO_FLAGS, QDOM_DEFAULT, hashdeclExceptionInfo->getTypeInfo(true), 1, bigIntTypeInfo, QORE_PARAM_NO_ARG, "warning_mask");

    // *hash<ExceptionInfo> Program::parsePending(string code, string label, *softint warning_mask, *string source, *softint offset, softbool format_label = True){}
    QC_PROGRAM->addMethod("parsePending", (q_method_n_t)Program_parsePending_VsVsniNsnivb, Public, QCF_NO_FLAGS, QDOM_EMBEDDED_LOGIC, hashdeclExceptionInfo->getTypeInfo(true), 6, stringTypeInfo, QORE_PARAM_NO_ARG, "code", stringTypeInfo, QORE_PARAM_NO_ARG, "label", softBigIntOrNothingTypeInfo, QORE_PARAM_NO_ARG, "warning_mask", stringOrNothingTypeInfo, QORE_PARAM_NO_ARG, "source", softBigIntOrNothingTypeInfo, QORE_PARAM_NO_ARG, "offset", softBoolTypeInfo, QoreSimpleValue().assign(true), "format_label");

    // nothing Program::parseRollback(){}
    QC_PROGRAM->addMethod("parseRollback", (q_method_n_t)Program_parseRollback, Public, QCF_DEPRECATED, QDOM_DEFAULT, nothingTypeInfo);

    // nothing Program::replaceParseOptions(softint opt){}
    QC_PROGRAM->addMethod("replaceParseOptions", (q_method_n_t)Program_replaceParseOptions_vi, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, softBigIntTypeInfo, QORE_PARAM_NO_ARG, "opt");

    // auto Program::run(){}
    QC_PROGRAM->addMethod("run", (q_method_n_t)Program_run, Public, QCF_NO_FLAGS, QDOM_DEFAULT, autoTypeInfo);

    // nothing Program::setGlobalVarValue(string name, auto value){}
    QC_PROGRAM->addMethod("setGlobalVarValue", (q_method_n_t)Program_setGlobalVarValue_VsVA, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 2, stringTypeInfo, QORE_PARAM_NO_ARG, "name", autoTypeInfo, QORE_PARAM_NO_ARG, "value");

    // nothing Program::setParseOptions(softint opt = PO_DEFAULT){}
    QC_PROGRAM->addMethod("setParseOptions", (q_method_n_t)Program_setParseOptions_vi, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, softBigIntTypeInfo, QoreSimpleValue().assign((int64)PO_DEFAULT), "opt");

    // nothing Program::setScriptPath(*string path){}
    QC_PROGRAM->addMethod("setScriptPath", (q_method_n_t)Program_setScriptPath_Ns, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, stringOrNothingTypeInfo, QORE_PARAM_NO_ARG, "path");

    // bool Program::setThreadInit(*code init){}
    QC_PROGRAM->addMethod("setThreadInit", (q_method_n_t)Program_setThreadInit_Nc, Public, QCF_NO_FLAGS, QDOM_THREAD_CONTROL, boolTypeInfo, 1, codeOrNothingTypeInfo, QORE_PARAM_NO_ARG, "init");

    // nothing Program::setTimeZone(TimeZone zone){}
    QC_PROGRAM->addMethod("setTimeZone", (q_method_n_t)Program_setTimeZone_C8TimeZone, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, QC_TIMEZONE->getTypeInfo(), QORE_PARAM_NO_ARG, "zone");

    // nothing Program::setTimeZoneRegion(string region){}
    QC_PROGRAM->addMethod("setTimeZoneRegion", (q_method_n_t)Program_setTimeZoneRegion_Vs, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "region");

    // nothing Program::setTimeZoneUTCOffset(softint seconds_east){}
    QC_PROGRAM->addMethod("setTimeZoneUTCOffset", (q_method_n_t)Program_setTimeZoneUTCOffset_vi, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, softBigIntTypeInfo, QORE_PARAM_NO_ARG, "seconds_east");

    // nothing Program::undefine(string def){}
    QC_PROGRAM->addMethod("undefine", (q_method_n_t)Program_undefine_Vs, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "def");
# 76 "QC_Program.qpp"
# 85 "QC_Program.qpp"
    ns.addConstant("CSP_SETPRIV", ((int64)CSP_SETPRIV));
    ns.addConstant("CSP_SETPUB", ((int64)CSP_SETPUB));
    ns.addConstant("CSP_UNCHANGED", ((int64)CSP_UNCHANGED));
# 92 "QC_Program.qpp"
# 105 "QC_Program.qpp"
# 112 "QC_Program.qpp"
# 119 "QC_Program.qpp"
# 126 "QC_Program.qpp"
# 133 "QC_Program.qpp"
# 140 "QC_Program.qpp"
# 147 "QC_Program.qpp"
# 154 "QC_Program.qpp"
# 161 "QC_Program.qpp"
# 168 "QC_Program.qpp"
# 175 "QC_Program.qpp"
# 182 "QC_Program.qpp"
# 189 "QC_Program.qpp"
# 194 "QC_Program.qpp"
# 199 "QC_Program.qpp"
# 206 "QC_Program.qpp"
    ns.addConstant("DOM_DATABASE", ((int64)QDOM_DATABASE));
    ns.addConstant("DOM_DEBUGGER", ((int64)QDOM_DEBUGGER));
    ns.addConstant("DOM_EMBEDDED_LOGIC", ((int64)QDOM_EMBEDDED_LOGIC));
    ns.addConstant("DOM_EXTERNAL_INFO", ((int64)QDOM_EXTERNAL_INFO));
    ns.addConstant("DOM_EXTERNAL_PROCESS", ((int64)QDOM_EXTERNAL_PROCESS));
    ns.addConstant("DOM_FILESYSTEM", ((int64)QDOM_FILESYSTEM));
    ns.addConstant("DOM_GUI", ((int64)QDOM_GUI));
    ns.addConstant("DOM_INJECTION", ((int64)QDOM_INJECTION));
    ns.addConstant("DOM_LOCALE_CONTROL", ((int64)QDOM_LOCALE_CONTROL));
    ns.addConstant("DOM_MODULE", ((int64)QDOM_MODULES));
    ns.addConstant("DOM_NETWORK", ((int64)QDOM_NETWORK));
    ns.addConstant("DOM_PROCESS_CONTROL", ((int64)QDOM_PROCESS));
    ns.addConstant("DOM_REFLECTION", ((int64)QDOM_REFLECTION));
    ns.addConstant("DOM_TERMINAL_IO", ((int64)QDOM_TERMINAL_IO));
    ns.addConstant("DOM_THREAD_CLASS", ((int64)QDOM_THREAD_CLASS));
    ns.addConstant("DOM_THREAD_CONTROL", ((int64)QDOM_THREAD_CONTROL));
    ns.addConstant("DOM_THREAD_INFO", ((int64)QDOM_THREAD_INFO));
    ns.addConstant("DOM_UNCONTROLLED_API", ((int64)QDOM_UNCONTROLLED_API));
# 217 "QC_Program.qpp"
# 228 "QC_Program.qpp"
# 235 "QC_Program.qpp"
# 242 "QC_Program.qpp"
# 249 "QC_Program.qpp"
# 256 "QC_Program.qpp"
# 263 "QC_Program.qpp"
# 270 "QC_Program.qpp"
# 277 "QC_Program.qpp"
# 280 "QC_Program.qpp"
# 285 "QC_Program.qpp"
# 290 "QC_Program.qpp"
# 293 "QC_Program.qpp"
# 296 "QC_Program.qpp"
# 302 "QC_Program.qpp"
# 309 "QC_Program.qpp"
# 316 "QC_Program.qpp"
# 322 "QC_Program.qpp"
# 328 "QC_Program.qpp"
# 334 "QC_Program.qpp"
# 340 "QC_Program.qpp"
# 346 "QC_Program.qpp"
# 349 "QC_Program.qpp"
# 356 "QC_Program.qpp"
# 362 "QC_Program.qpp"
# 368 "QC_Program.qpp"
# 374 "QC_Program.qpp"
# 380 "QC_Program.qpp"
# 386 "QC_Program.qpp"
# 393 "QC_Program.qpp"
# 399 "QC_Program.qpp"
# 405 "QC_Program.qpp"
# 411 "QC_Program.qpp"
# 421 "QC_Program.qpp"
# 427 "QC_Program.qpp"
# 433 "QC_Program.qpp"
# 440 "QC_Program.qpp"
# 446 "QC_Program.qpp"
# 453 "QC_Program.qpp"
# 459 "QC_Program.qpp"
# 465 "QC_Program.qpp"
# 471 "QC_Program.qpp"
# 477 "QC_Program.qpp"
# 483 "QC_Program.qpp"
# 489 "QC_Program.qpp"
# 496 "QC_Program.qpp"
# 502 "QC_Program.qpp"
# 509 "QC_Program.qpp"
# 516 "QC_Program.qpp"
# 522 "QC_Program.qpp"
# 531 "QC_Program.qpp"
# 539 "QC_Program.qpp"
# 546 "QC_Program.qpp"
# 553 "QC_Program.qpp"
# 562 "QC_Program.qpp"
# 569 "QC_Program.qpp"
# 576 "QC_Program.qpp"
# 587 "QC_Program.qpp"
# 598 "QC_Program.qpp"
# 614 "QC_Program.qpp"
# 621 "QC_Program.qpp"
# 628 "QC_Program.qpp"
# 635 "QC_Program.qpp"
# 642 "QC_Program.qpp"
# 649 "QC_Program.qpp"
# 656 "QC_Program.qpp"
# 663 "QC_Program.qpp"
# 670 "QC_Program.qpp"
# 677 "QC_Program.qpp"
# 684 "QC_Program.qpp"
# 689 "QC_Program.qpp"
# 694 "QC_Program.qpp"
# 701 "QC_Program.qpp"
# 707 "QC_Program.qpp"
# 713 "QC_Program.qpp"
# 719 "QC_Program.qpp"
# 725 "QC_Program.qpp"
# 732 "QC_Program.qpp"
# 739 "QC_Program.qpp"
# 746 "QC_Program.qpp"
    ns.addConstant("DomainCodeMap", (pomaps.getDomainToStringMap()));
    ns.addConstant("DomainStringMap", (pomaps.getStringToDomainMap()));
    ns.addConstant("PO_ALLOW_BARE_REFS", ((int64)PO_ALLOW_BARE_REFS));
    ns.addConstant("PO_ALLOW_DEBUGGER", ((int64)PO_ALLOW_DEBUGGER));
    ns.addConstant("PO_ALLOW_INJECTION", ((int64)PO_ALLOW_INJECTION));
    ns.addConstant("PO_ALLOW_RETURNS", ((int64)PO_ALLOW_RETURNS));
    ns.addConstant("PO_ALLOW_STATEMENT_NO_EFFECT", ((int64)PO_ALLOW_STATEMENT_NO_EFFECT));
    ns.addConstant("PO_ALLOW_WEAK_REFERENCES", ((int64)PO_ALLOW_WEAK_REFERENCES));
    ns.addConstant("PO_ASSUME_LOCAL", ((int64)PO_ASSUME_LOCAL));
    ns.addConstant("PO_BROKEN_CAST", ((int64)PO_BROKEN_CAST));
    ns.addConstant("PO_BROKEN_INT_ASSIGNMENTS", ((int64)PO_BROKEN_INT_ASSIGNMENTS));
    ns.addConstant("PO_BROKEN_LIST_PARSING", ((int64)PO_BROKEN_LIST_PARSING));
    ns.addConstant("PO_BROKEN_LOGIC_PRECEDENCE", ((int64)PO_BROKEN_LOGIC_PRECEDENCE));
    ns.addConstant("PO_BROKEN_LOOP_STATEMENT", ((int64)PO_BROKEN_LOOP_STATEMENT));
    ns.addConstant("PO_BROKEN_OPERATORS", ((int64)PO_BROKEN_OPERATORS));
    ns.addConstant("PO_BROKEN_RANGE", ((int64)PO_BROKEN_RANGE));
    ns.addConstant("PO_BROKEN_REFERENCES", ((int64)PO_BROKEN_REFERENCES));
    ns.addConstant("PO_BROKEN_SPRINTF", ((int64)PO_BROKEN_SPRINTF));
    ns.addConstant("PO_DEFAULT", ((int64)PO_DEFAULT));
    ns.addConstant("PO_FREE_OPTIONS", ((int64)PO_FREE_OPTIONS));
    ns.addConstant("PO_IN_MODULE", ((int64)PO_IN_MODULE));
    ns.addConstant("PO_LOCKDOWN", ((int64)PO_LOCKDOWN));
    ns.addConstant("PO_LOCK_WARNINGS", ((int64)PO_LOCK_WARNINGS));
    ns.addConstant("PO_NEW_STYLE", ((int64)PO_NEW_STYLE));
    ns.addConstant("PO_NO_API", ((int64)PO_NO_API));
    ns.addConstant("PO_NO_CHILD_PO_RESTRICTIONS", ((int64)PO_NO_CHILD_PO_RESTRICTIONS));
    ns.addConstant("PO_NO_CLASS_DEFS", ((int64)PO_NO_CLASS_DEFS));
    ns.addConstant("PO_NO_CONSTANT_DEFS", ((int64)PO_NO_CONSTANT_DEFS));
    ns.addConstant("PO_NO_DATABASE", ((int64)PO_NO_DATABASE));
    ns.addConstant("PO_NO_DEBUGGING", ((int64)PO_NO_DEBUGGING));
    ns.addConstant("PO_NO_EMBEDDED_LOGIC", ((int64)PO_NO_EMBEDDED_LOGIC));
    ns.addConstant("PO_NO_EXTERNAL_ACCESS", ((int64)PO_NO_EXTERNAL_ACCESS));
    ns.addConstant("PO_NO_EXTERNAL_INFO", ((int64)PO_NO_EXTERNAL_INFO));
    ns.addConstant("PO_NO_EXTERNAL_PROCESS", ((int64)PO_NO_EXTERNAL_PROCESS));
    ns.addConstant("PO_NO_FILESYSTEM", ((int64)PO_NO_FILESYSTEM));
    ns.addConstant("PO_NO_GLOBAL_VARS", ((int64)PO_NO_GLOBAL_VARS));
    ns.addConstant("PO_NO_GUI", ((int64)PO_NO_GUI));
    ns.addConstant("PO_NO_INHERIT_GLOBAL_VARS", ((int64)PO_NO_INHERIT_GLOBAL_VARS));
    ns.addConstant("PO_NO_INHERIT_PROGRAM_DATA", ((int64)PO_NO_INHERIT_PROGRAM_DATA));
    ns.addConstant("PO_NO_INHERIT_SYSTEM_CLASSES", ((int64)PO_NO_INHERIT_SYSTEM_CLASSES));
    ns.addConstant("PO_NO_INHERIT_SYSTEM_CONSTANTS", ((int64)PO_NO_INHERIT_SYSTEM_CONSTANTS));
    ns.addConstant("PO_NO_INHERIT_SYSTEM_FUNC_VARIANTS", ((int64)PO_NO_INHERIT_SYSTEM_FUNC_VARIANTS));
    ns.addConstant("PO_NO_INHERIT_SYSTEM_HASHDECLS", ((int64)PO_NO_INHERIT_SYSTEM_HASHDECLS));
    ns.addConstant("PO_NO_INHERIT_USER_CLASSES", ((int64)PO_NO_INHERIT_USER_CLASSES));
    ns.addConstant("PO_NO_INHERIT_USER_CONSTANTS", ((int64)PO_NO_INHERIT_USER_CONSTANTS));
    ns.addConstant("PO_NO_INHERIT_USER_FUNC_VARIANTS", ((int64)PO_NO_INHERIT_USER_FUNC_VARIANTS));
    ns.addConstant("PO_NO_INHERIT_USER_HASHDECLS", ((int64)PO_NO_INHERIT_USER_HASHDECLS));
    ns.addConstant("PO_NO_IO", ((int64)PO_NO_IO));
    ns.addConstant("PO_NO_LOCALE_CONTROL", ((int64)PO_NO_LOCALE_CONTROL));
    ns.addConstant("PO_NO_MODULES", ((int64)PO_NO_MODULES));
    ns.addConstant("PO_NO_NAMESPACE_DEFS", ((int64)PO_NO_NAMESPACE_DEFS));
    ns.addConstant("PO_NO_NETWORK", ((int64)PO_NO_NETWORK));
    ns.addConstant("PO_NO_NEW", ((int64)PO_NO_NEW));
    ns.addConstant("PO_NO_PROCESS_CONTROL", ((int64)PO_NO_PROCESS_CONTROL));
    ns.addConstant("PO_NO_REFLECTION", ((int64)PO_NO_REFLECTION));
    ns.addConstant("PO_NO_SUBROUTINE_DEFS", ((int64)PO_NO_SUBROUTINE_DEFS));
    ns.addConstant("PO_NO_SYSTEM_API", ((int64)PO_NO_SYSTEM_API));
    ns.addConstant("PO_NO_SYSTEM_CLASSES", ((int64)PO_NO_INHERIT_SYSTEM_CLASSES));
    ns.addConstant("PO_NO_SYSTEM_FUNC_VARIANTS", ((int64)PO_NO_INHERIT_SYSTEM_FUNC_VARIANTS));
    ns.addConstant("PO_NO_TERMINAL_IO", ((int64)PO_NO_TERMINAL_IO));
    ns.addConstant("PO_NO_THREADS", ((int64)PO_NO_THREADS));
    ns.addConstant("PO_NO_THREAD_CLASSES", ((int64)PO_NO_THREAD_CLASSES));
    ns.addConstant("PO_NO_THREAD_CONTROL", ((int64)PO_NO_THREAD_CONTROL));
    ns.addConstant("PO_NO_THREAD_INFO", ((int64)PO_NO_THREAD_INFO));
    ns.addConstant("PO_NO_TOP_LEVEL_STATEMENTS", ((int64)PO_NO_TOP_LEVEL_STATEMENTS));
    ns.addConstant("PO_NO_TRANSIENT", ((int64)PO_NO_TRANSIENT));
    ns.addConstant("PO_NO_UNCONTROLLED_APIS", ((int64)PO_NO_UNCONTROLLED_APIS));
    ns.addConstant("PO_NO_USER_API", ((int64)PO_NO_USER_API));
    ns.addConstant("PO_NO_USER_CLASSES", ((int64)PO_NO_INHERIT_USER_CLASSES));
    ns.addConstant("PO_POSITIVE_OPTIONS", ((int64)PO_POSITIVE_OPTIONS));
    ns.addConstant("PO_REQUIRE_OUR", ((int64)PO_REQUIRE_OUR));
    ns.addConstant("PO_REQUIRE_PROTOTYPES", ((int64)PO_REQUIRE_PROTOTYPES));
    ns.addConstant("PO_REQUIRE_TYPES", ((int64)PO_REQUIRE_TYPES));
    ns.addConstant("PO_STRICT_ARGS", ((int64)PO_STRICT_ARGS));
    ns.addConstant("PO_STRICT_BOOLEAN_EVAL", ((int64)PO_STRICT_BOOLEAN_EVAL));
    ns.addConstant("PO_STRICT_TYPES", ((int64)PO_STRICT_TYPES));
    ns.addConstant("PO_STRONG_ENCAPSULATION", ((int64)PO_STRONG_ENCAPSULATION));
    ns.addConstant("ParseOptionCmdCodeMap", (ParseOptionMap::getCodeToStringMap()));
    ns.addConstant("ParseOptionCmdStringMap", (ParseOptionMap::getStringToCodeMap()));
    ns.addConstant("ParseOptionCodeMap", (pomaps.getCodeToStringMap()));
    ns.addConstant("ParseOptionStringMap", (pomaps.getStringToCodeMap()));
# 759 "QC_Program.qpp"
# 766 "QC_Program.qpp"
# 771 "QC_Program.qpp"
# 787 "QC_Program.qpp"
# 792 "QC_Program.qpp"
# 797 "QC_Program.qpp"
# 802 "QC_Program.qpp"
# 807 "QC_Program.qpp"
# 812 "QC_Program.qpp"
# 817 "QC_Program.qpp"
# 822 "QC_Program.qpp"
# 836 "QC_Program.qpp"
# 839 "QC_Program.qpp"
# 846 "QC_Program.qpp"
# 851 "QC_Program.qpp"
# 856 "QC_Program.qpp"
# 861 "QC_Program.qpp"
# 866 "QC_Program.qpp"
# 871 "QC_Program.qpp"
# 876 "QC_Program.qpp"
    ns.addConstant("WARN_ALL", ((int64)QP_WARN_ALL));
    ns.addConstant("WARN_BROKEN_LOGIC_PRECEDENCE", ((int64)QP_WARN_BROKEN_LOGIC_PRECEDENCE));
    ns.addConstant("WARN_CALL_WITH_TYPE_ERRORS", ((int64)QP_WARN_CALL_WITH_TYPE_ERRORS));
    ns.addConstant("WARN_DEFAULT", ((int64)QP_WARN_DEFAULT));
    ns.addConstant("WARN_DEPRECATED", ((int64)QP_WARN_DEPRECATED));
    ns.addConstant("WARN_DUPLICATE_BLOCK_VARS", ((int64)QP_WARN_DUPLICATE_BLOCK_VARS));
    ns.addConstant("WARN_DUPLICATE_GLOBAL_VARS", ((int64)QP_WARN_DUPLICATE_GLOBAL_VARS));
    ns.addConstant("WARN_DUPLICATE_HASH_KEY", ((int64)QP_WARN_DUPLICATE_HASH_KEY));
    ns.addConstant("WARN_DUPLICATE_LOCAL_VARS", ((int64)QP_WARN_DUPLICATE_LOCAL_VARS));
    ns.addConstant("WARN_EXCESS_ARGS", ((int64)QP_WARN_EXCESS_ARGS));
    ns.addConstant("WARN_INVALID_CATCH", ((int64)QP_WARN_INVALID_CATCH));
    ns.addConstant("WARN_INVALID_OPERATION", ((int64)QP_WARN_INVALID_OPERATION));
    ns.addConstant("WARN_MODULES", ((int64)QP_WARN_MODULES));
    ns.addConstant("WARN_NONE", ((int64)QP_WARN_NONE));
    ns.addConstant("WARN_NONEXISTENT_METHOD_CALL", ((int64)QP_WARN_NONEXISTENT_METHOD_CALL));
    ns.addConstant("WARN_RETURN_VALUE_IGNORED", ((int64)QP_WARN_RETURN_VALUE_IGNORED));
    ns.addConstant("WARN_UNDECLARED_VAR", ((int64)QP_WARN_UNDECLARED_VAR));
    ns.addConstant("WARN_UNKNOWN_WARNING", ((int64)QP_WARN_UNKNOWN_WARNING));
    ns.addConstant("WARN_UNREACHABLE_CODE", ((int64)QP_WARN_UNREACHABLE_CODE));
    ns.addConstant("WARN_UNREFERENCED_VARIABLE", ((int64)QP_WARN_UNREFERENCED_VARIABLE));
    ns.addConstant("WARN_WARNING_MASK_UNCHANGED", ((int64)QP_WARN_WARNING_MASK_UNCHANGED));

    return QC_PROGRAM;
}
