/* -*- mode: c++; indent-tabs-mode: nil -*- */
/*
    QC_ReadOnlyFile.qpp

    Qore Programming Language

    Copyright (C) 2003 - 2023 Qore Technologies, s.r.o.

    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
    to deal in the Software without restriction, including without limitation
    the rights to use, copy, modify, merge, publish, distribute, sublicense,
    and/or sell copies of the Software, and to permit persons to whom the
    Software is furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
    DEALINGS IN THE SOFTWARE.

    Note that the Qore library is released under a choice of three open-source
    licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
    information.
*/

#include <qore/Qore.h>
#include "qore/intern/QC_File.h"
#include "qore/intern/QC_Queue.h"

#include <cerrno>
/* Qore class Qore::ReadOnlyFile */

qore_classid_t CID_READONLYFILE;
QoreClass* QC_READONLYFILE;

// int ReadOnlyFile::close(){}
static QoreValue ReadOnlyFile_close(QoreObject* self, File* f, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 298 "QC_ReadOnlyFile.qpp"
    if (check_terminal_io(self, "ReadOnlyFile::close", xsink))
        return QoreValue();

    return f->close();
}

// ReadOnlyFile::constructor(string path, *string encoding) {}
static void ReadOnlyFile_constructor_VsNs(QoreObject* self, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* path = HARD_QORE_VALUE_STRING(args, 0);
    const QoreStringNode* encoding = get_param_value(args, 1).get<const QoreStringNode>();
# 213 "QC_ReadOnlyFile.qpp"
const QoreEncoding *qe = encoding ? QEM.findCreate(encoding) : QCS_DEFAULT;

    SimpleRefHolder<File> f(new File(qe));
    if (f->isTty() && runtime_check_parse_option(PO_NO_TERMINAL_IO)) {
        xsink->raiseException("ILLEGAL-EXPRESSION", "ReadOnlyFile::constructor() cannot be called with a TTY target "
            "when 'no-terminal-io' is set");
        return;
    }

    if (f->open(path->c_str(), O_RDONLY, 0, qe)) {
        xsink->raiseErrnoException("READONLYFILE-OPEN-ERROR", errno, "cannot open '%s'", path->c_str());
        return;
    }

    self->setPrivate(CID_READONLYFILE, f.release());
}

// ReadOnlyFile::copy() {}
static void ReadOnlyFile_copy(QoreObject* self, QoreObject* old, File* f, ExceptionSink* xsink) {
# 245 "QC_ReadOnlyFile.qpp"
self->setPrivate(CID_READONLYFILE, new File(f->getEncoding()));
}

// string ReadOnlyFile::getEncoding(){}
static QoreValue ReadOnlyFile_getEncoding(QoreObject* self, File* f, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 736 "QC_ReadOnlyFile.qpp"
    return new QoreStringNode(f->getEncoding()->getCode());
}

// *string ReadOnlyFile::getFileName(){}
static QoreValue ReadOnlyFile_getFileName(QoreObject* self, File* f, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 983 "QC_ReadOnlyFile.qpp"
    return f->getFileName();
}

// int ReadOnlyFile::getPos(){}
static QoreValue ReadOnlyFile_getPos(QoreObject* self, File* f, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 773 "QC_ReadOnlyFile.qpp"
    if (check_terminal_io(self, "ReadOnlyFile::getPos", xsink))
        return QoreValue();

    return f->getPos();
}

// *string ReadOnlyFile::getchar(){}
static QoreValue ReadOnlyFile_getchar(QoreObject* self, File* f, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 794 "QC_ReadOnlyFile.qpp"
    if (check_terminal_io(self, "ReadOnlyFile::getchar", xsink))
        return QoreValue();

    return f->getchar(xsink);
}

// hash<StatInfo> ReadOnlyFile::hstat(){}
static QoreValue ReadOnlyFile_hstat(QoreObject* self, File* f, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 912 "QC_ReadOnlyFile.qpp"
    if (check_terminal_io(self, "ReadOnlyFile::hstat", xsink))
      return QoreValue();

   return f->hstat(xsink);
}

// bool ReadOnlyFile::isDataAvailable(timeout timeout_ms = 0){}
static QoreValue ReadOnlyFile_isDataAvailable_Vt(QoreObject* self, File* f, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 0);
# 816 "QC_ReadOnlyFile.qpp"
    if (check_terminal_io(self, "ReadOnlyFile::isDataAvailable", xsink))
        return QoreValue();

    return f->isDataAvailable(timeout_ms, xsink);
}

// bool ReadOnlyFile::isOpen(){}
static QoreValue ReadOnlyFile_isOpen(QoreObject* self, File* f, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 955 "QC_ReadOnlyFile.qpp"
    return f->isOpen();
}

// bool ReadOnlyFile::isTty(){}
static QoreValue ReadOnlyFile_isTty(QoreObject* self, File* f, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 969 "QC_ReadOnlyFile.qpp"
    return f->isTty();
}

// nothing ReadOnlyFile::open(string path, *string encoding){}
static QoreValue ReadOnlyFile_open_VsNs(QoreObject* self, File* f, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* path = HARD_QORE_VALUE_STRING(args, 0);
    const QoreStringNode* encoding = get_param_value(args, 1).get<const QoreStringNode>();
# 275 "QC_ReadOnlyFile.qpp"
    if (check_terminal_io(self, "ReadOnlyFile::open", xsink))
        return QoreValue();

    const QoreEncoding *qe = encoding ? QEM.findCreate(encoding) : f->getEncoding();
    if (f->open(path->c_str(), O_RDONLY, 0, qe))
        xsink->raiseErrnoException("READONLYFILE-OPEN-ERROR", errno, "cannot open '%s'", path->c_str());
    return QoreValue();
}

// *string ReadOnlyFile::read(softint size, timeout timeout_ms = -1){}
static QoreValue ReadOnlyFile_read_viVt(QoreObject* self, File* f, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 size = HARD_QORE_VALUE_INT(args, 0);
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 1);
# 334 "QC_ReadOnlyFile.qpp"
    if (check_terminal_io(self, "ReadOnlyFile::read", xsink))
        return QoreValue();

    if (!size)
        return xsink->raiseException("READONLYFILE-READ-PARAMETER-ERROR", "expecting non-zero size as first parameter of ReadOnlyFile::read()");

    return f->read((qore_offset_t)size, timeout_ms, xsink);
}

// *binary ReadOnlyFile::readBinary(softint size, timeout timeout_ms = -1){}
static QoreValue ReadOnlyFile_readBinary_viVt(QoreObject* self, File* f, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 size = HARD_QORE_VALUE_INT(args, 0);
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 1);
# 664 "QC_ReadOnlyFile.qpp"
    if (check_terminal_io(self, "ReadOnlyFile::readBinary", xsink))
      return QoreValue();

   if (!size)
      return xsink->raiseException("READONLYFILE-READ-BINARY-PARAMETER-ERROR", "expecting size as first parameter of File::readBinary()");

   return f->readBinary((qore_offset_t)size, timeout_ms, xsink);
}

// *string ReadOnlyFile::readLine(bool incl_eol = True, *string eol){}
static QoreValue ReadOnlyFile_readLine_VbNs(QoreObject* self, File* f, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    bool incl_eol = HARD_QORE_VALUE_BOOL(args, 0);
    const QoreStringNode* eol = get_param_value(args, 1).get<const QoreStringNode>();
# 700 "QC_ReadOnlyFile.qpp"
    if (check_terminal_io(self, "ReadOnlyFile::readLine", xsink))
        return QoreValue();

    if (eol && eol->empty())
        eol = 0;

    return eol ? f->readUntil(eol->c_str(), incl_eol, xsink) : f->readLine(incl_eol, xsink);
}

// *int ReadOnlyFile::readi1(){}
static QoreValue ReadOnlyFile_readi1(QoreObject* self, File* f, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 480 "QC_ReadOnlyFile.qpp"
    if (check_terminal_io(self, "ReadOnlyFile::readi1", xsink))
        return QoreValue();

    char c;
    if (f->readi1(&c, xsink))
        return QoreValue();
    return c;
}

// *int ReadOnlyFile::readi2(){}
static QoreValue ReadOnlyFile_readi2(QoreObject* self, File* f, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 504 "QC_ReadOnlyFile.qpp"
    if (check_terminal_io(self, "ReadOnlyFile::readi2", xsink))
        return QoreValue();

    short s;
    if (f->readi2(&s, xsink))
        return QoreValue();
    return s;
}

// *int ReadOnlyFile::readi2LSB(){}
static QoreValue ReadOnlyFile_readi2LSB(QoreObject* self, File* f, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 577 "QC_ReadOnlyFile.qpp"
    if (check_terminal_io(self, "ReadOnlyFile::readi2LSB", xsink))
      return QoreValue();

   short s;
   if (f->readi2LSB(&s, xsink))
      return QoreValue();

   return s;
}

// *int ReadOnlyFile::readi4(){}
static QoreValue ReadOnlyFile_readi4(QoreObject* self, File* f, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 528 "QC_ReadOnlyFile.qpp"
    if (check_terminal_io(self, "ReadOnlyFile::readi4", xsink))
        return QoreValue();

    int i;
    if (f->readi4(&i, xsink))
        return QoreValue();
    return i;
}

// *int ReadOnlyFile::readi4LSB(){}
static QoreValue ReadOnlyFile_readi4LSB(QoreObject* self, File* f, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 602 "QC_ReadOnlyFile.qpp"
    if (check_terminal_io(self, "ReadOnlyFile::readi4LSB", xsink))
      return QoreValue();

   int i;
   if (f->readi4LSB(&i, xsink))
      return QoreValue();
   return i;
}

// *int ReadOnlyFile::readi8(){}
static QoreValue ReadOnlyFile_readi8(QoreObject* self, File* f, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 552 "QC_ReadOnlyFile.qpp"
    if (check_terminal_io(self, "ReadOnlyFile::readi8", xsink))
      return QoreValue();

   int64 i;
   if (f->readi8(&i, xsink))
      return QoreValue();

   return i;
}

// *int ReadOnlyFile::readi8LSB(){}
static QoreValue ReadOnlyFile_readi8LSB(QoreObject* self, File* f, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 626 "QC_ReadOnlyFile.qpp"
    if (check_terminal_io(self, "ReadOnlyFile::readi8LSB", xsink))
      return QoreValue();

   int64 i;
   if (f->readi8LSB(&i, xsink))
      return QoreValue();
   return i;
}

// *int ReadOnlyFile::readu1(){}
static QoreValue ReadOnlyFile_readu1(QoreObject* self, File* f, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 358 "QC_ReadOnlyFile.qpp"
    if (check_terminal_io(self, "ReadOnlyFile::readu1", xsink))
        return QoreValue();

    unsigned char c;
    if (f->readu1(&c, xsink))
        return QoreValue();
    return c;
}

// *int ReadOnlyFile::readu2(){}
static QoreValue ReadOnlyFile_readu2(QoreObject* self, File* f, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 382 "QC_ReadOnlyFile.qpp"
    if (check_terminal_io(self, "ReadOnlyFile::readu2", xsink))
        return QoreValue();

    unsigned short s;
    if (f->readu2(&s, xsink))
        return QoreValue();
    return s;
}

// *int ReadOnlyFile::readu2LSB(){}
static QoreValue ReadOnlyFile_readu2LSB(QoreObject* self, File* f, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 430 "QC_ReadOnlyFile.qpp"
    if (check_terminal_io(self, "ReadOnlyFile::readu2LSB", xsink))
        return QoreValue();

    unsigned short s;
    if (f->readu2LSB(&s, xsink))
        return QoreValue();

    return s;
}

// *int ReadOnlyFile::readu4(){}
static QoreValue ReadOnlyFile_readu4(QoreObject* self, File* f, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 406 "QC_ReadOnlyFile.qpp"
    if (check_terminal_io(self, "ReadOnlyFile::readu4", xsink))
        return QoreValue();

    unsigned int i;
    if (f->readu4(&i, xsink))
        return QoreValue();
    return i;
}

// *int ReadOnlyFile::readu4LSB(){}
static QoreValue ReadOnlyFile_readu4LSB(QoreObject* self, File* f, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 455 "QC_ReadOnlyFile.qpp"
    if (check_terminal_io(self, "ReadOnlyFile::readu4LSB", xsink))
        return QoreValue();

    unsigned int i;
    if (f->readu4LSB(&i, xsink))
        return QoreValue();

    return i;
}

// nothing ReadOnlyFile::setEncoding(*string encoding){}
static QoreValue ReadOnlyFile_setEncoding_Ns(QoreObject* self, File* f, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* encoding = get_param_value(args, 0).get<const QoreStringNode>();
# 720 "QC_ReadOnlyFile.qpp"
    if (check_terminal_io(self, "ReadOnlyFile::setEncoding", xsink))
        return QoreValue();

    const QoreEncoding *qe = encoding ? QEM.findCreate(encoding) : QCS_DEFAULT;
    f->setEncoding(qe);
    return QoreValue();
}

// nothing ReadOnlyFile::setEventQueue(Queue queue, auto arg, *bool with_data){}
static QoreValue ReadOnlyFile_setEventQueue_C5QueueVANb(QoreObject* self, File* f, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    HARD_QORE_VALUE_OBJ_DATA(queue, Queue, args, 0, CID_QUEUE, "ReadOnlyFile::setEventQueue()", "Queue", xsink);
    if (*xsink)
        return 0;
    QoreValue arg = get_param_value(args, 1);
    bool with_data = HARD_QORE_VALUE_BOOL(args, 2);
# 840 "QC_ReadOnlyFile.qpp"
    ReferenceHolder<Queue> q(queue, xsink);
    if (check_terminal_io(self, "ReadOnlyFile::setEventQueue", xsink))
        return QoreValue();

    if (queue->getMax() != -1)
        return xsink->raiseException("QUEUE-ERROR", "the Queue object passed as a maximum size of %d entr%s, which " \
            "could cause internal I/O to freeze in the File object if used; use a Queue object with no maximum " \
            "size for I/O monitoring", queue->getMax(), queue->getMax() == 1 ? "y" : "ies");

    // pass reference from QoreObject::getReferencedPrivateData() to function
    f->setEventQueue(xsink, q.release(), arg.refSelf(), with_data);
    return QoreValue();
}

// nothing ReadOnlyFile::setEventQueue(){}
static QoreValue ReadOnlyFile_setEventQueue(QoreObject* self, File* f, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 864 "QC_ReadOnlyFile.qpp"
    if (check_terminal_io(self, "ReadOnlyFile::setEventQueue", xsink))
        return QoreValue();

    f->setEventQueue(xsink, nullptr, QoreValue(), false);
    return QoreValue();
}

// int ReadOnlyFile::setPos(int pos = 0){}
static QoreValue ReadOnlyFile_setPos_Vi(QoreObject* self, File* f, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 pos = HARD_QORE_VALUE_INT(args, 0);
# 754 "QC_ReadOnlyFile.qpp"
    if (check_terminal_io(self, "ReadOnlyFile::setPos", xsink))
        return QoreValue();

    return f->setPos(pos);
}

// list<auto> ReadOnlyFile::stat(){}
static QoreValue ReadOnlyFile_stat(QoreObject* self, File* f, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 888 "QC_ReadOnlyFile.qpp"
    if (check_terminal_io(self, "ReadOnlyFile::stat", xsink))
      return QoreValue();

   return f->stat(xsink);
}

// hash<FilesystemInfo> ReadOnlyFile::statvfs(){}
static QoreValue ReadOnlyFile_statvfs(QoreObject* self, File* f, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 936 "QC_ReadOnlyFile.qpp"
    #ifdef Q_HAVE_STATVFS
   if (check_terminal_io(self, "ReadOnlyFile::statvfs", xsink))
      return QoreValue();

   return f->statvfs(xsink);
#else
   return missing_method_error("ReadOnlyFile::statvfs", "STATVFS", xsink);
#endif
    return QoreValue();
}

// static hash<StatInfo> ReadOnlyFile::hlstat(string path){}
static QoreValue static_ReadOnlyFile_hlstat_Vs(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* path = HARD_QORE_VALUE_STRING(args, 0);
# 1160 "QC_ReadOnlyFile.qpp"
struct stat sbuf;
#ifdef HAVE_LSTAT
   if (lstat(path->c_str(), &sbuf))
#else
   if (stat(path->c_str(), &sbuf))
#endif
      return xsink->raiseErrnoException("FILE-HLSTAT-ERROR", errno, "lstat() command failed");

   return stat_to_hash(sbuf);
}

// static hash<StatInfo> ReadOnlyFile::hstat(string path){}
static QoreValue static_ReadOnlyFile_hstat_Vs(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* path = HARD_QORE_VALUE_STRING(args, 0);
# 1132 "QC_ReadOnlyFile.qpp"
struct stat sbuf;
   if (stat(path->c_str(), &sbuf))
      return xsink->raiseErrnoException("FILE-HSTAT-ERROR", errno, "stat() command failed");

   return stat_to_hash(sbuf);
}

// static list<auto> ReadOnlyFile::lstat(string path){}
static QoreValue static_ReadOnlyFile_lstat_Vs(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* path = HARD_QORE_VALUE_STRING(args, 0);
# 1101 "QC_ReadOnlyFile.qpp"
struct stat sbuf;
#ifdef HAVE_LSTAT
   if (lstat(path->c_str(), &sbuf))
#else
   if (stat(path->c_str(), &sbuf))
#endif
      return xsink->raiseErrnoException("FILE-LSTAT-ERROR", errno, "lstat() command failed");

   return stat_to_list(sbuf);
}

// static binary ReadOnlyFile::readBinaryFile(string path, int max_file_len = -1){}
static QoreValue static_ReadOnlyFile_readBinaryFile_VsVi(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* path = HARD_QORE_VALUE_STRING(args, 0);
    int64 max_file_len = HARD_QORE_VALUE_INT(args, 1);
# 1043 "QC_ReadOnlyFile.qpp"
QoreFile qf;

    if (qf.open2(xsink, path->c_str())) {
        return QoreValue();
    }

    BinaryNode* b = qf.readBinary(max_file_len, xsink);
    return !b && !*xsink ? new BinaryNode : b;
}

// static string ReadOnlyFile::readTextFile(string path, *string encoding, int max_file_len = -1){}
static QoreValue static_ReadOnlyFile_readTextFile_VsNsVi(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* path = HARD_QORE_VALUE_STRING(args, 0);
    const QoreStringNode* encoding = get_param_value(args, 1).get<const QoreStringNode>();
    int64 max_file_len = HARD_QORE_VALUE_INT(args, 2);
# 1010 "QC_ReadOnlyFile.qpp"
const QoreEncoding *qe = encoding ? QEM.findCreate(encoding) : QCS_DEFAULT;

    QoreFile qf;

    if (qf.open2(xsink, path->c_str(), O_RDONLY, 0777, qe)) {
        return QoreValue();
    }

    QoreStringNode* str = qf.read(max_file_len, xsink);
    return !str && !*xsink ? new QoreStringNode(qe) : str;
}

// static list<auto> ReadOnlyFile::stat(string path){}
static QoreValue static_ReadOnlyFile_stat_Vs(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* path = HARD_QORE_VALUE_STRING(args, 0);
# 1073 "QC_ReadOnlyFile.qpp"
struct stat sbuf;
   if (stat(path->c_str(), &sbuf))
      return xsink->raiseErrnoException("FILE-STAT-ERROR", errno, "stat() command failed");

   return stat_to_list(sbuf);
}

// static hash<FilesystemInfo> ReadOnlyFile::statvfs(string path){}
static QoreValue static_ReadOnlyFile_statvfs_Vs(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* path = HARD_QORE_VALUE_STRING(args, 0);
# 1191 "QC_ReadOnlyFile.qpp"
#ifdef Q_HAVE_STATVFS
   struct statvfs vfs;
   if (statvfs(path->c_str(), &vfs))
      return xsink->raiseErrnoException("FILE-STATVFS-ERROR", errno, "statvfs() call failed");

   return statvfs_to_hash(vfs);
#else
   file_lock_error("statvfs", xsink);
   return QoreValue();
#endif
    return QoreValue();
}

DLLLOCAL void preinitReadOnlyFileClass() {
    QC_READONLYFILE = new QoreClass("ReadOnlyFile", "::Qore::ReadOnlyFile", QDOM_FILESYSTEM);
    CID_READONLYFILE = QC_READONLYFILE->getID();
    QC_READONLYFILE->setSystem();
}

DLLLOCAL QoreClass* initReadOnlyFileClass(QoreNamespace& ns) {
    if (!QC_READONLYFILE)
        preinitReadOnlyFileClass();

    // set parent class
    assert(QC_ABSTRACTPOLLABLEIOOBJECTBASE);
    QC_READONLYFILE->addBuiltinVirtualBaseClass(QC_ABSTRACTPOLLABLEIOOBJECTBASE);

    // int ReadOnlyFile::close(){}
    QC_READONLYFILE->addMethod("close", (q_method_n_t)ReadOnlyFile_close, Public, QCF_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo);

    // ReadOnlyFile::constructor(string path, *string encoding) {}
    QC_READONLYFILE->addConstructor(ReadOnlyFile_constructor_VsNs, Public, QCF_NO_FLAGS, QDOM_DEFAULT, 2, stringTypeInfo, QORE_PARAM_NO_ARG, "path", stringOrNothingTypeInfo, QORE_PARAM_NO_ARG, "encoding");

    // ReadOnlyFile::copy() {}
    QC_READONLYFILE->setCopy((q_copy_t)ReadOnlyFile_copy);

    // string ReadOnlyFile::getEncoding(){}
    QC_READONLYFILE->addMethod("getEncoding", (q_method_n_t)ReadOnlyFile_getEncoding, Public, QCF_CONSTANT, QDOM_DEFAULT, stringTypeInfo);

    // *string ReadOnlyFile::getFileName(){}
    QC_READONLYFILE->addMethod("getFileName", (q_method_n_t)ReadOnlyFile_getFileName, Public, QCF_CONSTANT, QDOM_DEFAULT, stringOrNothingTypeInfo);

    // int ReadOnlyFile::getPos(){}
    QC_READONLYFILE->addMethod("getPos", (q_method_n_t)ReadOnlyFile_getPos, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, bigIntTypeInfo);

    // *string ReadOnlyFile::getchar(){}
    QC_READONLYFILE->addMethod("getchar", (q_method_n_t)ReadOnlyFile_getchar, Public, QCF_NO_FLAGS, QDOM_DEFAULT, stringOrNothingTypeInfo);

    // hash<StatInfo> ReadOnlyFile::hstat(){}
    QC_READONLYFILE->addMethod("hstat", (q_method_n_t)ReadOnlyFile_hstat, Public, QCF_NO_FLAGS, QDOM_DEFAULT, hashdeclStatInfo->getTypeInfo(false));

    // bool ReadOnlyFile::isDataAvailable(timeout timeout_ms = 0){}
    QC_READONLYFILE->addMethod("isDataAvailable", (q_method_n_t)ReadOnlyFile_isDataAvailable_Vt, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, boolTypeInfo, 1, timeoutTypeInfo, QoreSimpleValue().assign((int64)0), "timeout_ms");

    // bool ReadOnlyFile::isOpen(){}
    QC_READONLYFILE->addMethod("isOpen", (q_method_n_t)ReadOnlyFile_isOpen, Public, QCF_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

    // bool ReadOnlyFile::isTty(){}
    QC_READONLYFILE->addMethod("isTty", (q_method_n_t)ReadOnlyFile_isTty, Public, QCF_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

    // nothing ReadOnlyFile::open(string path, *string encoding){}
    QC_READONLYFILE->addMethod("open", (q_method_n_t)ReadOnlyFile_open_VsNs, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 2, stringTypeInfo, QORE_PARAM_NO_ARG, "path", stringOrNothingTypeInfo, QORE_PARAM_NO_ARG, "encoding");

    // *string ReadOnlyFile::read(softint size, timeout timeout_ms = -1){}
    QC_READONLYFILE->addMethod("read", (q_method_n_t)ReadOnlyFile_read_viVt, Public, QCF_NO_FLAGS, QDOM_DEFAULT, stringOrNothingTypeInfo, 2, softBigIntTypeInfo, QORE_PARAM_NO_ARG, "size", timeoutTypeInfo, QoreSimpleValue().assign((int64)-1), "timeout_ms");

    // *binary ReadOnlyFile::readBinary(softint size, timeout timeout_ms = -1){}
    QC_READONLYFILE->addMethod("readBinary", (q_method_n_t)ReadOnlyFile_readBinary_viVt, Public, QCF_NO_FLAGS, QDOM_DEFAULT, binaryOrNothingTypeInfo, 2, softBigIntTypeInfo, QORE_PARAM_NO_ARG, "size", timeoutTypeInfo, QoreSimpleValue().assign((int64)-1), "timeout_ms");

    // *string ReadOnlyFile::readLine(bool incl_eol = True, *string eol){}
    QC_READONLYFILE->addMethod("readLine", (q_method_n_t)ReadOnlyFile_readLine_VbNs, Public, QCF_NO_FLAGS, QDOM_DEFAULT, stringOrNothingTypeInfo, 2, boolTypeInfo, QoreSimpleValue().assign(true), "incl_eol", stringOrNothingTypeInfo, QORE_PARAM_NO_ARG, "eol");

    // *int ReadOnlyFile::readi1(){}
    QC_READONLYFILE->addMethod("readi1", (q_method_n_t)ReadOnlyFile_readi1, Public, QCF_NO_FLAGS, QDOM_DEFAULT, bigIntOrNothingTypeInfo);

    // *int ReadOnlyFile::readi2(){}
    QC_READONLYFILE->addMethod("readi2", (q_method_n_t)ReadOnlyFile_readi2, Public, QCF_NO_FLAGS, QDOM_DEFAULT, bigIntOrNothingTypeInfo);

    // *int ReadOnlyFile::readi2LSB(){}
    QC_READONLYFILE->addMethod("readi2LSB", (q_method_n_t)ReadOnlyFile_readi2LSB, Public, QCF_NO_FLAGS, QDOM_DEFAULT, bigIntOrNothingTypeInfo);

    // *int ReadOnlyFile::readi4(){}
    QC_READONLYFILE->addMethod("readi4", (q_method_n_t)ReadOnlyFile_readi4, Public, QCF_NO_FLAGS, QDOM_DEFAULT, bigIntOrNothingTypeInfo);

    // *int ReadOnlyFile::readi4LSB(){}
    QC_READONLYFILE->addMethod("readi4LSB", (q_method_n_t)ReadOnlyFile_readi4LSB, Public, QCF_NO_FLAGS, QDOM_DEFAULT, bigIntOrNothingTypeInfo);

    // *int ReadOnlyFile::readi8(){}
    QC_READONLYFILE->addMethod("readi8", (q_method_n_t)ReadOnlyFile_readi8, Public, QCF_NO_FLAGS, QDOM_DEFAULT, bigIntOrNothingTypeInfo);

    // *int ReadOnlyFile::readi8LSB(){}
    QC_READONLYFILE->addMethod("readi8LSB", (q_method_n_t)ReadOnlyFile_readi8LSB, Public, QCF_NO_FLAGS, QDOM_DEFAULT, bigIntOrNothingTypeInfo);

    // *int ReadOnlyFile::readu1(){}
    QC_READONLYFILE->addMethod("readu1", (q_method_n_t)ReadOnlyFile_readu1, Public, QCF_NO_FLAGS, QDOM_DEFAULT, bigIntOrNothingTypeInfo);

    // *int ReadOnlyFile::readu2(){}
    QC_READONLYFILE->addMethod("readu2", (q_method_n_t)ReadOnlyFile_readu2, Public, QCF_NO_FLAGS, QDOM_DEFAULT, bigIntOrNothingTypeInfo);

    // *int ReadOnlyFile::readu2LSB(){}
    QC_READONLYFILE->addMethod("readu2LSB", (q_method_n_t)ReadOnlyFile_readu2LSB, Public, QCF_NO_FLAGS, QDOM_DEFAULT, bigIntOrNothingTypeInfo);

    // *int ReadOnlyFile::readu4(){}
    QC_READONLYFILE->addMethod("readu4", (q_method_n_t)ReadOnlyFile_readu4, Public, QCF_NO_FLAGS, QDOM_DEFAULT, bigIntOrNothingTypeInfo);

    // *int ReadOnlyFile::readu4LSB(){}
    QC_READONLYFILE->addMethod("readu4LSB", (q_method_n_t)ReadOnlyFile_readu4LSB, Public, QCF_NO_FLAGS, QDOM_DEFAULT, bigIntOrNothingTypeInfo);

    // nothing ReadOnlyFile::setEncoding(*string encoding){}
    QC_READONLYFILE->addMethod("setEncoding", (q_method_n_t)ReadOnlyFile_setEncoding_Ns, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, stringOrNothingTypeInfo, QORE_PARAM_NO_ARG, "encoding");

    // nothing ReadOnlyFile::setEventQueue(Queue queue, auto arg, *bool with_data){}
    QC_READONLYFILE->addMethod("setEventQueue", (q_method_n_t)ReadOnlyFile_setEventQueue_C5QueueVANb, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 3, QC_QUEUE->getTypeInfo(), QORE_PARAM_NO_ARG, "queue", autoTypeInfo, QORE_PARAM_NO_ARG, "arg", boolOrNothingTypeInfo, QORE_PARAM_NO_ARG, "with_data");

    // nothing ReadOnlyFile::setEventQueue(){}
    QC_READONLYFILE->addMethod("setEventQueue", (q_method_n_t)ReadOnlyFile_setEventQueue, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

    // int ReadOnlyFile::setPos(int pos = 0){}
    QC_READONLYFILE->addMethod("setPos", (q_method_n_t)ReadOnlyFile_setPos_Vi, Public, QCF_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo, 1, bigIntTypeInfo, QoreSimpleValue().assign((int64)0), "pos");

    // list<auto> ReadOnlyFile::stat(){}
    QC_READONLYFILE->addMethod("stat", (q_method_n_t)ReadOnlyFile_stat, Public, QCF_NO_FLAGS, QDOM_DEFAULT, autoListTypeInfo);

    // hash<FilesystemInfo> ReadOnlyFile::statvfs(){}
    QC_READONLYFILE->addMethod("statvfs", (q_method_n_t)ReadOnlyFile_statvfs, Public, QCF_NO_FLAGS, QDOM_DEFAULT, hashdeclFilesystemInfo->getTypeInfo(false));

    // static hash<StatInfo> ReadOnlyFile::hlstat(string path){}
    QC_READONLYFILE->addStaticMethod("hlstat", (q_func_n_t)static_ReadOnlyFile_hlstat_Vs, Public, QCF_NO_FLAGS, QDOM_FILESYSTEM, hashdeclStatInfo->getTypeInfo(false), 1, stringTypeInfo, QORE_PARAM_NO_ARG, "path");

    // static hash<StatInfo> ReadOnlyFile::hstat(string path){}
    QC_READONLYFILE->addStaticMethod("hstat", (q_func_n_t)static_ReadOnlyFile_hstat_Vs, Public, QCF_NO_FLAGS, QDOM_FILESYSTEM, hashdeclStatInfo->getTypeInfo(false), 1, stringTypeInfo, QORE_PARAM_NO_ARG, "path");

    // static list<auto> ReadOnlyFile::lstat(string path){}
    QC_READONLYFILE->addStaticMethod("lstat", (q_func_n_t)static_ReadOnlyFile_lstat_Vs, Public, QCF_NO_FLAGS, QDOM_FILESYSTEM, autoListTypeInfo, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "path");

    // static binary ReadOnlyFile::readBinaryFile(string path, int max_file_len = -1){}
    QC_READONLYFILE->addStaticMethod("readBinaryFile", (q_func_n_t)static_ReadOnlyFile_readBinaryFile_VsVi, Public, QCF_NO_FLAGS, QDOM_FILESYSTEM, binaryTypeInfo, 2, stringTypeInfo, QORE_PARAM_NO_ARG, "path", bigIntTypeInfo, QoreSimpleValue().assign((int64)-1), "max_file_len");

    // static string ReadOnlyFile::readTextFile(string path, *string encoding, int max_file_len = -1){}
    QC_READONLYFILE->addStaticMethod("readTextFile", (q_func_n_t)static_ReadOnlyFile_readTextFile_VsNsVi, Public, QCF_NO_FLAGS, QDOM_FILESYSTEM, stringTypeInfo, 3, stringTypeInfo, QORE_PARAM_NO_ARG, "path", stringOrNothingTypeInfo, QORE_PARAM_NO_ARG, "encoding", bigIntTypeInfo, QoreSimpleValue().assign((int64)-1), "max_file_len");

    // static list<auto> ReadOnlyFile::stat(string path){}
    QC_READONLYFILE->addStaticMethod("stat", (q_func_n_t)static_ReadOnlyFile_stat_Vs, Public, QCF_NO_FLAGS, QDOM_FILESYSTEM, autoListTypeInfo, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "path");

    // static hash<FilesystemInfo> ReadOnlyFile::statvfs(string path){}
    QC_READONLYFILE->addStaticMethod("statvfs", (q_func_n_t)static_ReadOnlyFile_statvfs_Vs, Public, QCF_NO_FLAGS, QDOM_FILESYSTEM, hashdeclFilesystemInfo->getTypeInfo(false), 1, stringTypeInfo, QORE_PARAM_NO_ARG, "path");
# 39 "QC_ReadOnlyFile.qpp"
# 48 "QC_ReadOnlyFile.qpp"
# 51 "QC_ReadOnlyFile.qpp"
# 54 "QC_ReadOnlyFile.qpp"
# 57 "QC_ReadOnlyFile.qpp"
#ifndef S_IFLNK
#define S_IFLNK 0
#endif
# 64 "QC_ReadOnlyFile.qpp"
#ifndef S_IFSOCK
#define S_IFSOCK 0
#endif
# 71 "QC_ReadOnlyFile.qpp"
#ifndef S_IFWHT
#define S_IFWHT S_IFREG
#endif
# 78 "QC_ReadOnlyFile.qpp"
# 81 "QC_ReadOnlyFile.qpp"
# 84 "QC_ReadOnlyFile.qpp"
# 87 "QC_ReadOnlyFile.qpp"
# 90 "QC_ReadOnlyFile.qpp"
#ifndef S_IRWXG
#define S_IRWXG 0
#endif
# 97 "QC_ReadOnlyFile.qpp"
#ifndef S_IRGRP
#define S_IRGRP 0
#endif
# 104 "QC_ReadOnlyFile.qpp"
#ifndef S_IWGRP
#define S_IWGRP 0
#endif
# 111 "QC_ReadOnlyFile.qpp"
#ifndef S_IXGRP
#define S_IXGRP 0
#endif
# 118 "QC_ReadOnlyFile.qpp"
#ifndef S_IRWXO
#define S_IRWXO 0
#endif
# 125 "QC_ReadOnlyFile.qpp"
#ifndef S_IROTH
#define S_IROTH 0
#endif
# 132 "QC_ReadOnlyFile.qpp"
#ifndef S_IWOTH
#define S_IWOTH 0
#endif
# 139 "QC_ReadOnlyFile.qpp"
#ifndef S_IXOTH
#define S_IXOTH 0
#endif
# 146 "QC_ReadOnlyFile.qpp"
#ifndef S_ISUID
#define S_ISUID 0
#endif
# 153 "QC_ReadOnlyFile.qpp"
#ifndef S_ISGID
#define S_ISGID 0
#endif
# 160 "QC_ReadOnlyFile.qpp"
#ifndef S_ISVTX
#define S_ISVTX 0
#endif
    ns.addConstant("S_IFBLK", ((int64)S_IFBLK));
    ns.addConstant("S_IFCHR", ((int64)S_IFCHR));
    ns.addConstant("S_IFDIR", ((int64)S_IFDIR));
    ns.addConstant("S_IFLNK", ((int64)S_IFLNK));
    ns.addConstant("S_IFMT", ((int64)S_IFMT));
    ns.addConstant("S_IFREG", ((int64)S_IFREG));
    ns.addConstant("S_IFSOCK", ((int64)S_IFSOCK));
    ns.addConstant("S_IFWHT", ((int64)S_IFWHT));
    ns.addConstant("S_IRGRP", ((int64)S_IRGRP));
    ns.addConstant("S_IROTH", ((int64)S_IROTH));
    ns.addConstant("S_IRUSR", ((int64)S_IRUSR));
    ns.addConstant("S_IRWXG", ((int64)S_IRWXG));
    ns.addConstant("S_IRWXO", ((int64)S_IRWXO));
    ns.addConstant("S_IRWXU", ((int64)S_IRWXU));
    ns.addConstant("S_ISGID", ((int64)S_ISGID));
    ns.addConstant("S_ISUID", ((int64)S_ISUID));
    ns.addConstant("S_ISVTX", ((int64)S_ISVTX));
    ns.addConstant("S_IWGRP", ((int64)S_IWGRP));
    ns.addConstant("S_IWOTH", ((int64)S_IWOTH));
    ns.addConstant("S_IWUSR", ((int64)S_IWUSR));
    ns.addConstant("S_IXGRP", ((int64)S_IXGRP));
    ns.addConstant("S_IXOTH", ((int64)S_IXOTH));
    ns.addConstant("S_IXUSR", ((int64)S_IXUSR));

    return QC_READONLYFILE;
}
