/* -*- mode: c++; indent-tabs-mode: nil -*- */
/*
    ql_lib.qpp

    Qore Programming Language

    Copyright (C) 2003 - 2023 Qore Technologies, s.r.o.

    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
    to deal in the Software without restriction, including without limitation
    the rights to use, copy, modify, merge, publish, distribute, sublicense,
    and/or sell copies of the Software, and to permit persons to whom the
    Software is furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
    DEALINGS IN THE SOFTWARE.

    Note that the Qore library is released under a choice of three open-source
    licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
    information.
*/

#include <qore/Qore.h>
#include "qore/intern/ql_lib.h"
#include "qore/intern/ExecArgList.h"
#include "qore/intern/QoreSignal.h"
#include "qore/intern/QoreHashNodeIntern.h"
#include <qore/minitest.hpp>

#include <cerrno>
#include <csignal>
#include <cstdio>
#include <cstdlib>
#include <ctime>
#include <fcntl.h>
#include <sys/param.h>
#include <sys/types.h>
#include <unistd.h>

#ifdef HAVE_GRP_H
#include <grp.h>
#endif

#ifdef HAVE_SYS_WAIT_H
#include <sys/wait.h>
#endif

#ifdef HAVE_GLOB_H
#include <glob.h>
#else
#include "qore/intern/qore_glob.h"
#endif

#ifdef HAVE_GETIFADDRS
#ifdef HAVE_ARPA_INET_H
#include <arpa/inet.h>
#endif
#include <ifaddrs.h>
#include <sys/socket.h>
#include <sys/types.h>
#ifdef AF_LINK
#include <net/if_dl.h>
#endif
#ifdef AF_PACKET
#include <linux/if_packet.h>
#endif
#endif

#ifndef SIGHUP
#define SIGHUP 0
#endif

#define MAX_GIDS 100

extern bool threads_initialized;

#ifdef DEBUG_TESTS
// Unsorted unit tests are put here
#  include "tests/ReferenceHolder_tests.cpp"
#endif

AbstractQoreNode* missing_function_error(const char* func, ExceptionSink* xsink) {
   QoreString have(func);
   have.toupr();
   return xsink->raiseException("MISSING-FEATURE-ERROR", "this system does not implement %s(); for maximum portability use the constant Option::HAVE_%s to check if this function is implemented before calling", func, have.getBuffer());
}

AbstractQoreNode* missing_function_error(const char* func, const char* opt, ExceptionSink* xsink) {
   return xsink->raiseException("MISSING-FEATURE-ERROR", "this system does not implement %s(); for maximum portability use the constant Option::HAVE_%s to check if this function is implemented before calling", func, opt);
}

#ifdef DEBUG_X
static AbstractQoreNode* runQoreTests(const QoreListNode* args, ExceptionSink* xsink) {
   minitest::result res = minitest::execute_all_tests();
   if (res.all_tests_succeeded) {
      printf("Qore runtime: %d tests succeeded\n", res.sucessful_tests_count);
      return 0;
   }

   return xsink->raiseException("A Qore test failed", "Qore test in file %s, line %d threw an exception.",
                                res.failed_test_file, res.failed_test_line);
}

static AbstractQoreNode* runRecentQoreTests(const QoreListNode* args, ExceptionSink* xsink) {
   minitest::result res = minitest::test_last_changed_files(3); // 3 last modified files
   if (res.all_tests_succeeded) {
      printf("Qore runtime: %d recent tests succeeded\n", res.sucessful_tests_count);
      return 0;
   }

   return xsink->raiseException("A Qore test failed", "Qore test in file %s, line %d threw an exception.",
                                res.failed_test_file, res.failed_test_line);
}
namespace {
TEST() {
  // just an example of empty test
}
}
#endif
TypedHashDecl* init_hashdecl_NetIfInfo(QoreNamespace& ns) {
    TypedHashDecl* hd = new TypedHashDecl("NetIfInfo", "::Qore::NetIfInfo");
    hd->addMember("address", stringOrNothingTypeInfo, QoreValue());
    hd->addMember("address_desc", stringOrNothingTypeInfo, QoreValue());
    hd->addMember("family", bigIntTypeInfo, QoreValue());
    hd->addMember("familystr", stringTypeInfo, QoreValue());
    hd->addMember("interface", stringTypeInfo, QoreValue());
    ns.addSystemHashDecl(hd);
    return hd;
}

TypedHashDecl* init_hashdecl_ExceptionInfo(QoreNamespace& ns) {
    TypedHashDecl* hd = new TypedHashDecl("ExceptionInfo", "::Qore::ExceptionInfo");
    hd->addMember("arg", autoTypeInfo, QoreValue());
    hd->addMember("callstack", qore_get_complex_list_type(hashdeclCallStackInfo->getTypeInfo(false)), QoreValue());
    hd->addMember("desc", autoTypeInfo, QoreValue());
    hd->addMember("endline", bigIntTypeInfo, QoreValue());
    hd->addMember("err", autoTypeInfo, QoreValue());
    hd->addMember("file", stringTypeInfo, QoreValue());
    hd->addMember("lang", stringTypeInfo, QoreValue());
    hd->addMember("line", bigIntTypeInfo, QoreValue());
    hd->addMember("next", hd->getTypeInfo(false), QoreValue());
    hd->addMember("offset", bigIntTypeInfo, QoreValue());
    hd->addMember("source", stringTypeInfo, QoreValue());
    hd->addMember("type", stringTypeInfo, QoreValue());
    ns.addSystemHashDecl(hd);
    return hd;
}

# 229 "ql_lib.qpp"
# 265 "ql_lib.qpp"
# 280 "ql_lib.qpp"
# 295 "ql_lib.qpp"
# 342 "ql_lib.qpp"
# 348 "ql_lib.qpp"
# 367 "ql_lib.qpp"
# 386 "ql_lib.qpp"
# 405 "ql_lib.qpp"
# 424 "ql_lib.qpp"
# 455 "ql_lib.qpp"
# 493 "ql_lib.qpp"
# 512 "ql_lib.qpp"
# 518 "ql_lib.qpp"
# 539 "ql_lib.qpp"
# 557 "ql_lib.qpp"
# 563 "ql_lib.qpp"
# 575 "ql_lib.qpp"
# 594 "ql_lib.qpp"
# 652 "ql_lib.qpp"
# 678 "ql_lib.qpp"
# 684 "ql_lib.qpp"
# 708 "ql_lib.qpp"
# 731 "ql_lib.qpp"
# 737 "ql_lib.qpp"
# 753 "ql_lib.qpp"
# 768 "ql_lib.qpp"
# 783 "ql_lib.qpp"
# 798 "ql_lib.qpp"
# 816 "ql_lib.qpp"
# 822 "ql_lib.qpp"
# 840 "ql_lib.qpp"
# 846 "ql_lib.qpp"
# 868 "ql_lib.qpp"
# 890 "ql_lib.qpp"
# 912 "ql_lib.qpp"
# 934 "ql_lib.qpp"
# 953 "ql_lib.qpp"
# 971 "ql_lib.qpp"
# 977 "ql_lib.qpp"
# 1000 "ql_lib.qpp"
# 1006 "ql_lib.qpp"
# 1025 "ql_lib.qpp"
# 1031 "ql_lib.qpp"
# 1054 "ql_lib.qpp"
# 1060 "ql_lib.qpp"
# 1081 "ql_lib.qpp"
# 1108 "ql_lib.qpp"
# 1225 "ql_lib.qpp"
# 1239 "ql_lib.qpp"
// nothing abort() {}
static QoreValue f_abort(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 275 "ql_lib.qpp"
   threads_initialized = false;
   abort();
    return QoreValue();
}

// string basename(string path) {}
static QoreValue f_basename_Vs(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* path = HARD_QORE_VALUE_STRING(args, 0);
# 810 "ql_lib.qpp"
   char* p = q_basename(path->getBuffer());
   int len = strlen(p);
   return new QoreStringNode(p, len, len + 1, path->getEncoding());
}

// nothing basename() {}
static QoreValue f_basename(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 819 "ql_lib.qpp"

    return QoreValue();
}

// nothing close_all_fd(*softbool strd) {}
static QoreValue f_close_all_fd_nb(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    bool strd = HARD_QORE_VALUE_BOOL(args, 0);
# 1094 "ql_lib.qpp"
#if defined(HAVE_UNISTD_H) && defined(_SC_OPEN_MAX)
   int start = strd ? 0 : 2;
   // get max file descriptor
   int max = (int)sysconf(_SC_OPEN_MAX);
   printd(5, "close_all_fd() start: %d max: %d\n", start, max);

   for (;start < max; ++start)
      close(start);
#else
   missing_function_error("close_all_fd", xsink);
#endif // HAVE_UNISTD
    return QoreValue();
}

// string dirname(string path) {}
static QoreValue f_dirname_Vs(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* path = HARD_QORE_VALUE_STRING(args, 0);
# 834 "ql_lib.qpp"
   char* p = q_dirname(path->getBuffer());
   int len = strlen(p);
   return new QoreStringNode(p, len, len + 1, path->getEncoding());
}

// nothing dirname() {}
static QoreValue f_dirname(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 843 "ql_lib.qpp"

    return QoreValue();
}

// int errno() {}
static QoreValue f_errno(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 764 "ql_lib.qpp"
   return errno;
}

// nothing exec(string command) {}
static QoreValue f_exec_Vs(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* command = HARD_QORE_VALUE_STRING(args, 0);
# 288 "ql_lib.qpp"
   ExecArgList exec_args(command->getBuffer());
   execvp(exec_args.getFile(), exec_args.getArgs());

   xsink->raiseErrnoException("EXEC-ERROR", errno, "execvp() failed in child process for target '%s'", exec_args.getFile());
    return QoreValue();
}

// nothing exit(softint rc = 0) {}
static QoreValue f_exit_vi(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 rc = HARD_QORE_VALUE_INT(args, 0);
# 261 "ql_lib.qpp"
   qore_exit_process(rc);
    return QoreValue();
}

// int fork() {}
static QoreValue f_fork(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 619 "ql_lib.qpp"
#ifdef HAVE_FORK
#ifdef HAVE_SIGNAL_HANDLING
   int QORE_THREAD_LIMIT = 1 + (QSM.thread_running ? 1 : 0);
#else
#define QORE_THREAD_LIMIT 1
#endif
   if (thread_list.getNumThreads() > (unsigned)(QORE_THREAD_LIMIT))
      return xsink->raiseException("ILLEGAL-FORK", "cannot fork() when other threads are running");

#ifdef HAVE_SIGNAL_HANDLING
   // we may not fork from within a signal handler
   if (QSM.thread_running && q_gettid() == QSM.q_gettid())
      return xsink->raiseException("ILLEGAL-FORK", "cannot fork() within a signal handler");

   // stop signal handling thread and make sure it can't be restarted until fork is done
   QSM.preFork();
#endif

   //printd(5, "stopped signal thread, about to fork pid %d\n", getpid()); fflush(stdout);
   int pid = fork();

#ifdef HAVE_SIGNAL_HANDLING
   // release signal handler lock
   QSM.postFork(!pid, xsink);
#endif

   return pid;
#else
   return missing_function_error("fork", xsink);
#endif
    return QoreValue();
}

// list<hash<NetIfInfo>> get_netif_list() {}
static QoreValue f_get_netif_list(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1158 "ql_lib.qpp"
#ifdef HAVE_GETIFADDRS
    struct ifaddrs* ifap;
    if (getifaddrs(&ifap)) {
        xsink->raiseErrnoException("GETGETIFLIST-ERROR", errno, "getidaddrs() failed");
        return QoreValue();
    }
    ON_BLOCK_EXIT(freeifaddrs, ifap);

    ReferenceHolder<QoreListNode> rv(new QoreListNode(hashdeclNetIfInfo->getTypeInfo()), xsink);

    for (struct ifaddrs* ifa = ifap; ifa; ifa = ifa->ifa_next) {
        // skip interfaces with no address
        if (!ifa->ifa_addr)
            continue;

        ReferenceHolder<QoreHashNode> h(new QoreHashNode(hashdeclNetIfInfo, xsink), xsink);
        qore_hash_private* hh = qore_hash_private::get(**h);

        hh->setKeyValueIntern("interface", new QoreStringNode(ifa->ifa_name));

        switch (ifa->ifa_addr->sa_family) {
            case AF_INET:
            case AF_INET6: {
                // get ipv4 or ipv6 address
                char ifname[INET6_ADDRSTRLEN];
                if (inet_ntop(ifa->ifa_addr->sa_family, qore_get_in_addr(ifa->ifa_addr), ifname, sizeof(ifname))) {
                    QoreStringNode* addrstr = new QoreStringNode(ifname);
                    hh->setKeyValueIntern("address", addrstr);
                    hh->setKeyValueIntern("address_desc", QoreAddrInfo::getAddressDesc(ifa->ifa_addr->sa_family, addrstr->c_str()));
                }
                break;
            }
#ifdef AF_PACKET
            case AF_PACKET: {
                struct sockaddr_ll* s = (struct sockaddr_ll*)ifa->ifa_addr;
                QoreStringNode* addrstr = new QoreStringNode;
                for (unsigned i = 0; i < s->sll_halen; ++i) {
                    addrstr->sprintf("%02x%s", s->sll_addr[i], static_cast<int>(i) < (s->sll_halen - 1) ? ":" : "");
                }
                hh->setKeyValueIntern("address", addrstr);
                hh->setKeyValueIntern("address_desc", QoreAddrInfo::getAddressDesc(ifa->ifa_addr->sa_family, addrstr->c_str()));
                break;
            }
#endif
#ifdef AF_LINK
            case AF_LINK: {
                QoreStringNode* addrstr = new QoreStringNode(link_ntoa((struct sockaddr_dl*)ifa->ifa_addr));
                hh->setKeyValueIntern("address", addrstr);
                hh->setKeyValueIntern("address_desc", QoreAddrInfo::getAddressDesc(ifa->ifa_addr->sa_family, addrstr->c_str()));
                break;
            }
#endif
        }

         hh->setKeyValueIntern("family", ifa->ifa_addr->sa_family);
         hh->setKeyValueIntern("familystr", new QoreStringNode(QoreAddrInfo::getFamilyName(ifa->ifa_addr->sa_family)));

         rv->push(h.release(), xsink);
    }

    return rv.release();
#else
    missing_function_error("getifaddrs", xsink);
#endif
    return QoreValue();
}

// list<hash<auto>> getaddrinfo(*string node, *softstring service, softint family = AF_UNSPEC, softint flags = 0) {}
static QoreValue f_getaddrinfo_Nsnsvivi(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* node = get_param_value(args, 0).get<const QoreStringNode>();
    const QoreStringNode* service = get_param_value(args, 1).get<const QoreStringNode>();
    int64 family = HARD_QORE_VALUE_INT(args, 2);
    int64 flags = HARD_QORE_VALUE_INT(args, 3);
# 1077 "ql_lib.qpp"
   return q_getaddrinfo_to_list(xsink, node ? node->getBuffer() : 0, service ? service->getBuffer() : 0, (int)family, (int)flags);
}

// int getegid() {}
static QoreValue f_getegid(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 416 "ql_lib.qpp"
#ifdef HAVE_GETEGID
   return getgid();
#else
   return missing_function_error("getegid", "UNIX_USERMGT", xsink);
#endif
    return QoreValue();
}

// int geteuid() {}
static QoreValue f_geteuid(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 378 "ql_lib.qpp"
#ifdef HAVE_GETEUID
   return getuid();
#else
   return missing_function_error("geteuid", "UNIX_USERMGT", xsink);
#endif
    return QoreValue();
}

// int getgid() {}
static QoreValue f_getgid(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 397 "ql_lib.qpp"
#ifdef HAVE_GETGID
   return getgid();
#else
   return missing_function_error("getgid", "UNIX_USERMGT", xsink);
#endif
    return QoreValue();
}

// list<int> getgroups() {}
static QoreValue f_getgroups(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 437 "ql_lib.qpp"
#ifdef HAVE_GETGROUPS
   gid_t l[MAX_GIDS];

   int n = getgroups(MAX_GIDS, l);
   if (n == -1)
      return xsink->raiseErrnoException("GETGROUPS-ERROR", errno, "error calling getgroups()");

   QoreListNode* rv = new QoreListNode(bigIntTypeInfo);
   for (int i = 0; i < n; ++i) {
      rv->push(l[i], xsink);
   }
   return rv;
#else
   missing_function_error("getgroups", "UNIX_USERMGT", xsink);
#endif
    return QoreValue();
}

// *string gethostbyaddr(string addr, softint type = AF_INET) {}
static QoreValue f_gethostbyaddr_Vsvi(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* addr = HARD_QORE_VALUE_STRING(args, 0);
    int64 type = HARD_QORE_VALUE_INT(args, 1);
# 996 "ql_lib.qpp"
   return q_gethostbyaddr_to_string(xsink, addr->getBuffer(), type);
}

// nothing gethostbyaddr() {}
static QoreValue f_gethostbyaddr(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1003 "ql_lib.qpp"

    return QoreValue();
}

// *hash<auto> gethostbyaddr_long(string addr, softint type = AF_INET) {}
static QoreValue f_gethostbyaddr_long_Vsvi(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* addr = HARD_QORE_VALUE_STRING(args, 0);
    int64 type = HARD_QORE_VALUE_INT(args, 1);
# 1050 "ql_lib.qpp"
   return q_gethostbyaddr_to_hash(xsink, addr->getBuffer(), type);
}

// nothing gethostbyaddr_long() {}
static QoreValue f_gethostbyaddr_long(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1057 "ql_lib.qpp"

    return QoreValue();
}

// *string gethostbyname(string name) {}
static QoreValue f_gethostbyname_Vs(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* name = HARD_QORE_VALUE_STRING(args, 0);
# 967 "ql_lib.qpp"
   return q_gethostbyname_to_string(name->getBuffer());
}

// nothing gethostbyname() {}
static QoreValue f_gethostbyname(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 974 "ql_lib.qpp"

    return QoreValue();
}

// *hash<auto> gethostbyname_long(string name) {}
static QoreValue f_gethostbyname_long_Vs(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* name = HARD_QORE_VALUE_STRING(args, 0);
# 1021 "ql_lib.qpp"
   return q_gethostbyname_to_hash(name->getBuffer());
}

// nothing gethostbyname_long() {}
static QoreValue f_gethostbyname_long(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1028 "ql_lib.qpp"

    return QoreValue();
}

// string gethostname() {}
static QoreValue f_gethostname(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 745 "ql_lib.qpp"
   char buf[HOSTNAMEBUFSIZE + 1];

   if (gethostname(buf, HOSTNAMEBUFSIZE))
      return xsink->raiseErrnoException("GETHOSTNAME-ERROR", errno, "gethostname() failed");
   return new QoreStringNode(buf);
}

// int getpid() {}
static QoreValue f_getpid(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 571 "ql_lib.qpp"
   return getpid();
}

// int getppid() {}
static QoreValue f_getppid(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 586 "ql_lib.qpp"
#ifdef HAVE_GETPPID
   return getppid();
#else
   return missing_function_error("getppid", xsink);
#endif
    return QoreValue();
}

// int getuid() {}
static QoreValue f_getuid(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 359 "ql_lib.qpp"
#ifdef HAVE_GETUID
   return getuid();
#else
   return missing_function_error("getuid", "UNIX_USERMGT", xsink);
#endif
    return QoreValue();
}

// int kill(softint pid, softint sig = SIGHUP) {}
static QoreValue f_kill_vivi(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 pid = HARD_QORE_VALUE_INT(args, 0);
    int64 sig = HARD_QORE_VALUE_INT(args, 1);
# 670 "ql_lib.qpp"
#ifdef HAVE_KILL
   return kill(pid, sig);
#else
   return missing_function_error("kill", xsink);
#endif
    return QoreValue();
}

// nothing kill() {}
static QoreValue f_kill(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 681 "ql_lib.qpp"

    return QoreValue();
}

// int qore_get_library_options() {}
static QoreValue f_qore_get_library_options(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1246 "ql_lib.qpp"
    return qore_library_options;
}

// int qore_set_library_options(int opts) {}
static QoreValue f_qore_set_library_options_Vi(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 opts = HARD_QORE_VALUE_INT(args, 0);
# 1235 "ql_lib.qpp"
    return qore_set_library_options(opts);
}

// int rand() {}
static QoreValue f_rand(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 699 "ql_lib.qpp"
#ifdef HAVE_RANDOM
   // return a random 64-bit integer by calling random() twice
   return random() | (((int64)random()) << 32);
#else
   return rand() | (((int64)rand()) << 32);
#endif
    return QoreValue();
}

// int setegid(softint gid) {}
static QoreValue f_setegid_vi(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 gid = HARD_QORE_VALUE_INT(args, 0);
# 926 "ql_lib.qpp"
#ifdef HAVE_SETEGID
   return setegid((int)gid);
#else
   return missing_function_error("setegid", xsink);
#endif
    return QoreValue();
}

// int seteuid(softint uid) {}
static QoreValue f_seteuid_vi(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 uid = HARD_QORE_VALUE_INT(args, 0);
# 882 "ql_lib.qpp"
#ifdef HAVE_SETEUID
   return seteuid((int)uid);
#else
   return missing_function_error("seteuid", xsink);
#endif
    return QoreValue();
}

// int setgid(softint gid) {}
static QoreValue f_setgid_vi(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 gid = HARD_QORE_VALUE_INT(args, 0);
# 904 "ql_lib.qpp"
#ifdef HAVE_SETGID
   return setgid((int)gid);
#else
   return missing_function_error("setgid", "UNIX_USERMGT", xsink);
#endif
    return QoreValue();
}

// nothing setgroups(softlist<softint> gids) {}
static QoreValue f_setgroups_C17softlist_softint_(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreListNode* gids = HARD_QORE_VALUE_LIST(args, 0);
# 468 "ql_lib.qpp"
#ifdef HAVE_SETGROUPS
    if (gids->empty())
        return QoreValue();

#ifdef HAVE_LOCAL_VARIADIC_ARRAYS_X
    gid_t l[gids->size()];
#else
    gid_t* l = (gid_t*)malloc(sizeof(gid_t) * gids->size());
    if (!l) {
        xsink->outOfMemory();
        return QoreValue();
    }
    ON_BLOCK_EXIT(free, l);
#endif
    for (size_t i = 0, e = gids->size(); i < e; ++i) {
        l[i] = (int)gids->retrieveEntry(i).getAsBigInt();
    }
    if (setgroups(gids->size(), l))
        xsink->raiseErrnoException("SETGROUPS-ERROR", errno, "error calling setgroups()");
#else
    missing_function_error("setgroups", "UNIX_USERMGT", xsink);
#endif
    return QoreValue();
}

// int setsid() {}
static QoreValue f_setsid(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 945 "ql_lib.qpp"
#ifdef HAVE_SETSID
   return setsid();
#else
   return missing_function_error("setsid", xsink);
#endif
    return QoreValue();
}

// int setuid(softint uid) {}
static QoreValue f_setuid_vi(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 uid = HARD_QORE_VALUE_INT(args, 0);
# 860 "ql_lib.qpp"
#ifdef HAVE_SETUID
   return setuid((int)uid);
#else
   return missing_function_error("setuid", "UNIX_USERMGT", xsink);
#endif
    return QoreValue();
}

// int sleep(softint seconds) {}
static QoreValue f_sleep_vi(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 seconds = HARD_QORE_VALUE_INT(args, 0);
# 505 "ql_lib.qpp"
   if (!seconds)
      return 0;

   return qore_usleep(seconds * 1000000ll);
}

// nothing sleep() {}
static QoreValue f_sleep(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 515 "ql_lib.qpp"

    return QoreValue();
}

// nothing srand(softint seed) {}
static QoreValue f_srand_vi(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 seed = HARD_QORE_VALUE_INT(args, 0);
# 722 "ql_lib.qpp"
#ifdef HAVE_RANDOM
   srandom((int)seed);
#else
   srand((int)seed);
#endif
   return QoreValue();
}

// nothing srand() {}
static QoreValue f_srand(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 734 "ql_lib.qpp"

    return QoreValue();
}

// string strerror(softint err) {}
static QoreValue f_strerror_vi(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 err = HARD_QORE_VALUE_INT(args, 0);
# 779 "ql_lib.qpp"
   return q_strerror(err);
}

// string strerror() {}
static QoreValue f_strerror(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 794 "ql_lib.qpp"
   return q_strerror(errno);
}

// int system(string command) {}
static QoreValue f_system_Vs(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* command = HARD_QORE_VALUE_STRING(args, 0);
# 312 "ql_lib.qpp"
#if defined(HAVE_FORK) && defined(HAVE_SIGNAL_HANDLING)
   // on platforms with fork(2) and signal handling, we need to fork, enable all signals, then execvp()
   pid_t pid;
   if (!(pid = fork())) {
      // enable all signals
      sigset_t mask;
      sigemptyset(&mask);
      sigprocmask(SIG_SETMASK, &mask, 0);

      // exec pgm like system(): sh -c "command"
      execl("/bin/sh", "sh", "-c", command->getBuffer(), NULL);
      fprintf(stderr, "execvp() failed in child process for target '/bin/sh' with error code %d: %s\n", errno, strerror(errno));
      _Exit(-1);
      //qore_exit_process(-1);
   }
   if (pid == -1)
      return -1;

   // wait for child to exit
   int status;
   wait(&status);
   return WIFEXITED(status) ? WEXITSTATUS(status) : -1;
#elif defined(HAVE_SYSTEM)
   return system(command->getBuffer());
#else
   return missing_function_error("system", xsink);
#endif
    return QoreValue();
}

// nothing system() {}
static QoreValue f_system(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 345 "ql_lib.qpp"

    return QoreValue();
}

// int usleep(softint usecs) {}
static QoreValue f_usleep_vi(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 usecs = HARD_QORE_VALUE_INT(args, 0);
# 532 "ql_lib.qpp"
   if (!usecs)
      return 0;

   return qore_usleep(usecs);
}

// int usleep(date d) {}
static QoreValue f_usleep_Vd(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const DateTimeNode* d = HARD_QORE_VALUE_DATE(args, 0);
# 553 "ql_lib.qpp"
   return qore_usleep(d->getRelativeMicroseconds());
}

// nothing usleep() {}
static QoreValue f_usleep(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 560 "ql_lib.qpp"

    return QoreValue();
}


DLLLOCAL void init_lib_functions(QoreNamespace& ns) {
    // nothing abort() {}
    ns.addBuiltinVariant("abort", (q_func_n_t)f_abort, QCF_NO_FLAGS, QDOM_PROCESS, nothingTypeInfo);
    // string basename(string path) {}
    ns.addBuiltinVariant("basename", (q_func_n_t)f_basename_Vs, QCF_CONSTANT, QDOM_DEFAULT, stringTypeInfo, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "path");
    // nothing basename() {}
    ns.addBuiltinVariant("basename", (q_func_n_t)f_basename, QCF_RUNTIME_NOOP, QDOM_DEFAULT, nothingTypeInfo);
    // nothing close_all_fd(*softbool strd) {}
    ns.addBuiltinVariant("close_all_fd", (q_func_n_t)f_close_all_fd_nb, QCF_NO_FLAGS, QDOM_PROCESS, nothingTypeInfo, 1, softBoolOrNothingTypeInfo, QORE_PARAM_NO_ARG, "strd");
    // string dirname(string path) {}
    ns.addBuiltinVariant("dirname", (q_func_n_t)f_dirname_Vs, QCF_CONSTANT, QDOM_DEFAULT, stringTypeInfo, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "path");
    // nothing dirname() {}
    ns.addBuiltinVariant("dirname", (q_func_n_t)f_dirname, QCF_RUNTIME_NOOP, QDOM_DEFAULT, nothingTypeInfo);
    // int errno() {}
    ns.addBuiltinVariant("errno", (q_func_n_t)f_errno, QCF_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo);
    // nothing exec(string command) {}
    ns.addBuiltinVariant("exec", (q_func_n_t)f_exec_Vs, QCF_NO_FLAGS, QDOM_EXTERNAL_PROCESS|QDOM_PROCESS, nothingTypeInfo, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "command");
    // nothing exit(softint rc = 0) {}
    ns.addBuiltinVariant("exit", (q_func_n_t)f_exit_vi, QCF_NO_FLAGS, QDOM_PROCESS, nothingTypeInfo, 1, softBigIntTypeInfo, QoreSimpleValue().assign((int64)0), "rc");
    // int fork() {}
    ns.addBuiltinVariant("fork", (q_func_n_t)f_fork, QCF_NO_FLAGS, QDOM_PROCESS, bigIntTypeInfo);
    // list<hash<NetIfInfo>> get_netif_list() {}
    ns.addBuiltinVariant("get_netif_list", (q_func_n_t)f_get_netif_list, QCF_NO_FLAGS, QDOM_EXTERNAL_INFO, qore_get_complex_list_type(hashdeclNetIfInfo->getTypeInfo(false)));
    // list<hash<auto>> getaddrinfo(*string node, *softstring service, softint family = AF_UNSPEC, softint flags = 0) {}
    ns.addBuiltinVariant("getaddrinfo", (q_func_n_t)f_getaddrinfo_Nsnsvivi, QCF_RET_VALUE_ONLY, QDOM_EXTERNAL_INFO, qore_get_complex_list_type(autoHashTypeInfo), 4, stringOrNothingTypeInfo, QORE_PARAM_NO_ARG, "node", softStringOrNothingTypeInfo, QORE_PARAM_NO_ARG, "service", softBigIntTypeInfo, QoreSimpleValue().assign((int64)AF_UNSPEC), "family", softBigIntTypeInfo, QoreSimpleValue().assign((int64)0), "flags");
    // int getegid() {}
    ns.addBuiltinVariant("getegid", (q_func_n_t)f_getegid, QCF_NO_FLAGS, QDOM_EXTERNAL_INFO, bigIntTypeInfo);
    // int geteuid() {}
    ns.addBuiltinVariant("geteuid", (q_func_n_t)f_geteuid, QCF_NO_FLAGS, QDOM_EXTERNAL_INFO, bigIntTypeInfo);
    // int getgid() {}
    ns.addBuiltinVariant("getgid", (q_func_n_t)f_getgid, QCF_NO_FLAGS, QDOM_EXTERNAL_INFO, bigIntTypeInfo);
    // list<int> getgroups() {}
    ns.addBuiltinVariant("getgroups", (q_func_n_t)f_getgroups, QCF_NO_FLAGS, QDOM_EXTERNAL_INFO, qore_get_complex_list_type(bigIntTypeInfo));
    // *string gethostbyaddr(string addr, softint type = AF_INET) {}
    ns.addBuiltinVariant("gethostbyaddr", (q_func_n_t)f_gethostbyaddr_Vsvi, QCF_CONSTANT, QDOM_EXTERNAL_INFO, stringOrNothingTypeInfo, 2, stringTypeInfo, QORE_PARAM_NO_ARG, "addr", softBigIntTypeInfo, QoreSimpleValue().assign((int64)AF_INET), "type");
    // nothing gethostbyaddr() {}
    ns.addBuiltinVariant("gethostbyaddr", (q_func_n_t)f_gethostbyaddr, QCF_RUNTIME_NOOP, QDOM_DEFAULT, nothingTypeInfo);
    // *hash<auto> gethostbyaddr_long(string addr, softint type = AF_INET) {}
    ns.addBuiltinVariant("gethostbyaddr_long", (q_func_n_t)f_gethostbyaddr_long_Vsvi, QCF_CONSTANT, QDOM_EXTERNAL_INFO, autoHashOrNothingTypeInfo, 2, stringTypeInfo, QORE_PARAM_NO_ARG, "addr", softBigIntTypeInfo, QoreSimpleValue().assign((int64)AF_INET), "type");
    // nothing gethostbyaddr_long() {}
    ns.addBuiltinVariant("gethostbyaddr_long", (q_func_n_t)f_gethostbyaddr_long, QCF_RUNTIME_NOOP, QDOM_DEFAULT, nothingTypeInfo);
    // *string gethostbyname(string name) {}
    ns.addBuiltinVariant("gethostbyname", (q_func_n_t)f_gethostbyname_Vs, QCF_CONSTANT, QDOM_EXTERNAL_INFO, stringOrNothingTypeInfo, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "name");
    // nothing gethostbyname() {}
    ns.addBuiltinVariant("gethostbyname", (q_func_n_t)f_gethostbyname, QCF_RUNTIME_NOOP, QDOM_DEFAULT, nothingTypeInfo);
    // *hash<auto> gethostbyname_long(string name) {}
    ns.addBuiltinVariant("gethostbyname_long", (q_func_n_t)f_gethostbyname_long_Vs, QCF_CONSTANT, QDOM_EXTERNAL_INFO, autoHashOrNothingTypeInfo, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "name");
    // nothing gethostbyname_long() {}
    ns.addBuiltinVariant("gethostbyname_long", (q_func_n_t)f_gethostbyname_long, QCF_RUNTIME_NOOP, QDOM_DEFAULT, nothingTypeInfo);
    // string gethostname() {}
    ns.addBuiltinVariant("gethostname", (q_func_n_t)f_gethostname, QCF_CONSTANT, QDOM_EXTERNAL_INFO, stringTypeInfo);
    // int getpid() {}
    ns.addBuiltinVariant("getpid", (q_func_n_t)f_getpid, QCF_RET_VALUE_ONLY, QDOM_EXTERNAL_INFO, bigIntTypeInfo);
    // int getppid() {}
    ns.addBuiltinVariant("getppid", (q_func_n_t)f_getppid, QCF_RET_VALUE_ONLY, QDOM_EXTERNAL_INFO, bigIntTypeInfo);
    // int getuid() {}
    ns.addBuiltinVariant("getuid", (q_func_n_t)f_getuid, QCF_NO_FLAGS, QDOM_EXTERNAL_INFO, bigIntTypeInfo);
    // int kill(softint pid, softint sig = SIGHUP) {}
    ns.addBuiltinVariant("kill", (q_func_n_t)f_kill_vivi, QCF_NO_FLAGS, QDOM_EXTERNAL_PROCESS, bigIntTypeInfo, 2, softBigIntTypeInfo, QORE_PARAM_NO_ARG, "pid", softBigIntTypeInfo, QoreSimpleValue().assign((int64)SIGHUP), "sig");
    // nothing kill() {}
    ns.addBuiltinVariant("kill", (q_func_n_t)f_kill, QCF_RUNTIME_NOOP, QDOM_EXTERNAL_PROCESS, nothingTypeInfo);
    // int qore_get_library_options() {}
    ns.addBuiltinVariant("qore_get_library_options", (q_func_n_t)f_qore_get_library_options, QCF_NO_FLAGS, QDOM_PROCESS, bigIntTypeInfo);
    // int qore_set_library_options(int opts) {}
    ns.addBuiltinVariant("qore_set_library_options", (q_func_n_t)f_qore_set_library_options_Vi, QCF_NO_FLAGS, QDOM_PROCESS, bigIntTypeInfo, 1, bigIntTypeInfo, QORE_PARAM_NO_ARG, "opts");
    // int rand() {}
    ns.addBuiltinVariant("rand", (q_func_n_t)f_rand, QCF_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);
    // int setegid(softint gid) {}
    ns.addBuiltinVariant("setegid", (q_func_n_t)f_setegid_vi, QCF_NO_FLAGS, QDOM_PROCESS, bigIntTypeInfo, 1, softBigIntTypeInfo, QORE_PARAM_NO_ARG, "gid");
    // int seteuid(softint uid) {}
    ns.addBuiltinVariant("seteuid", (q_func_n_t)f_seteuid_vi, QCF_NO_FLAGS, QDOM_PROCESS, bigIntTypeInfo, 1, softBigIntTypeInfo, QORE_PARAM_NO_ARG, "uid");
    // int setgid(softint gid) {}
    ns.addBuiltinVariant("setgid", (q_func_n_t)f_setgid_vi, QCF_NO_FLAGS, QDOM_PROCESS, bigIntTypeInfo, 1, softBigIntTypeInfo, QORE_PARAM_NO_ARG, "gid");
    // nothing setgroups(softlist<softint> gids) {}
    ns.addBuiltinVariant("setgroups", (q_func_n_t)f_setgroups_C17softlist_softint_, QCF_NO_FLAGS, QDOM_PROCESS, nothingTypeInfo, 1, qore_get_complex_softlist_type(softBigIntTypeInfo), QORE_PARAM_NO_ARG, "gids");
    // int setsid() {}
    ns.addBuiltinVariant("setsid", (q_func_n_t)f_setsid, QCF_NO_FLAGS, QDOM_PROCESS, bigIntTypeInfo);
    // int setuid(softint uid) {}
    ns.addBuiltinVariant("setuid", (q_func_n_t)f_setuid_vi, QCF_NO_FLAGS, QDOM_PROCESS, bigIntTypeInfo, 1, softBigIntTypeInfo, QORE_PARAM_NO_ARG, "uid");
    // int sleep(softint seconds) {}
    ns.addBuiltinVariant("sleep", (q_func_n_t)f_sleep_vi, QCF_NO_FLAGS, QDOM_PROCESS, bigIntTypeInfo, 1, softBigIntTypeInfo, QORE_PARAM_NO_ARG, "seconds");
    // nothing sleep() {}
    ns.addBuiltinVariant("sleep", (q_func_n_t)f_sleep, QCF_RUNTIME_NOOP, QDOM_PROCESS, nothingTypeInfo);
    // nothing srand(softint seed) {}
    ns.addBuiltinVariant("srand", (q_func_n_t)f_srand_vi, QCF_NO_FLAGS, QDOM_PROCESS, nothingTypeInfo, 1, softBigIntTypeInfo, QORE_PARAM_NO_ARG, "seed");
    // nothing srand() {}
    ns.addBuiltinVariant("srand", (q_func_n_t)f_srand, QCF_RUNTIME_NOOP, QDOM_PROCESS, nothingTypeInfo);
    // string strerror(softint err) {}
    ns.addBuiltinVariant("strerror", (q_func_n_t)f_strerror_vi, QCF_CONSTANT, QDOM_DEFAULT, stringTypeInfo, 1, softBigIntTypeInfo, QORE_PARAM_NO_ARG, "err");
    // string strerror() {}
    ns.addBuiltinVariant("strerror", (q_func_n_t)f_strerror, QCF_CONSTANT, QDOM_DEFAULT, stringTypeInfo);
    // int system(string command) {}
    ns.addBuiltinVariant("system", (q_func_n_t)f_system_Vs, QCF_NO_FLAGS, QDOM_EXTERNAL_PROCESS, bigIntTypeInfo, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "command");
    // nothing system() {}
    ns.addBuiltinVariant("system", (q_func_n_t)f_system, QCF_RUNTIME_NOOP, QDOM_EXTERNAL_PROCESS, nothingTypeInfo);
    // int usleep(softint usecs) {}
    ns.addBuiltinVariant("usleep", (q_func_n_t)f_usleep_vi, QCF_NO_FLAGS, QDOM_PROCESS, bigIntTypeInfo, 1, softBigIntTypeInfo, QORE_PARAM_NO_ARG, "usecs");
    // int usleep(date d) {}
    ns.addBuiltinVariant("usleep", (q_func_n_t)f_usleep_Vd, QCF_NO_FLAGS, QDOM_PROCESS, bigIntTypeInfo, 1, dateTypeInfo, QORE_PARAM_NO_ARG, "d");
    // nothing usleep() {}
    ns.addBuiltinVariant("usleep", (q_func_n_t)f_usleep, QCF_RUNTIME_NOOP, QDOM_PROCESS, nothingTypeInfo);

}

DLLLOCAL void init_lib_constants(QoreNamespace& ns) {
# 131 "ql_lib.qpp"
# 139 "ql_lib.qpp"
# 142 "ql_lib.qpp"
# 145 "ql_lib.qpp"
# 148 "ql_lib.qpp"
# 151 "ql_lib.qpp"
# 156 "ql_lib.qpp"
    ns.addConstant("QLO_DISABLE_GARBAGE_COLLECTION", ((int64)QLO_DISABLE_GARBAGE_COLLECTION));
    ns.addConstant("QLO_DISABLE_OPENSSL_CLEANUP", ((int64)QLO_DISABLE_OPENSSL_CLEANUP));
    ns.addConstant("QLO_DISABLE_OPENSSL_INIT", ((int64)QLO_DISABLE_OPENSSL_INIT));
    ns.addConstant("QLO_DISABLE_SIGNAL_HANDLING", ((int64)QLO_DISABLE_SIGNAL_HANDLING));
    ns.addConstant("QLO_DISABLE_TLS_13", ((int64)QLO_DISABLE_TLS_13));
    ns.addConstant("QLO_DO_NOT_SEED_RNG", ((int64)QLO_DO_NOT_SEED_RNG));
    ns.addConstant("QLO_MINIMUM_TLS_13", ((int64)QLO_MINIMUM_TLS_13));
    ns.addConstant("QLO_RUNTIME_OPTS", ((int64)QLO_RUNTIME_OPTS));
# 229 "ql_lib.qpp"
# 265 "ql_lib.qpp"
# 280 "ql_lib.qpp"
# 295 "ql_lib.qpp"
# 342 "ql_lib.qpp"
# 348 "ql_lib.qpp"
# 367 "ql_lib.qpp"
# 386 "ql_lib.qpp"
# 405 "ql_lib.qpp"
# 424 "ql_lib.qpp"
# 455 "ql_lib.qpp"
# 493 "ql_lib.qpp"
# 512 "ql_lib.qpp"
# 518 "ql_lib.qpp"
# 539 "ql_lib.qpp"
# 557 "ql_lib.qpp"
# 563 "ql_lib.qpp"
# 575 "ql_lib.qpp"
# 594 "ql_lib.qpp"
# 652 "ql_lib.qpp"
# 678 "ql_lib.qpp"
# 684 "ql_lib.qpp"
# 708 "ql_lib.qpp"
# 731 "ql_lib.qpp"
# 737 "ql_lib.qpp"
# 753 "ql_lib.qpp"
# 768 "ql_lib.qpp"
# 783 "ql_lib.qpp"
# 798 "ql_lib.qpp"
# 816 "ql_lib.qpp"
# 822 "ql_lib.qpp"
# 840 "ql_lib.qpp"
# 846 "ql_lib.qpp"
# 868 "ql_lib.qpp"
# 890 "ql_lib.qpp"
# 912 "ql_lib.qpp"
# 934 "ql_lib.qpp"
# 953 "ql_lib.qpp"
# 971 "ql_lib.qpp"
# 977 "ql_lib.qpp"
# 1000 "ql_lib.qpp"
# 1006 "ql_lib.qpp"
# 1025 "ql_lib.qpp"
# 1031 "ql_lib.qpp"
# 1054 "ql_lib.qpp"
# 1060 "ql_lib.qpp"
# 1081 "ql_lib.qpp"
# 1108 "ql_lib.qpp"
# 1225 "ql_lib.qpp"
# 1239 "ql_lib.qpp"

}
