/* -*- mode: c++; indent-tabs-mode: nil -*- */
/*
    ql_math.qpp

    Qore Programming Language

    Copyright (C) 2003 - 2023 Qore Technologies, s.r.o.

    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
    to deal in the Software without restriction, including without limitation
    the rights to use, copy, modify, merge, publish, distribute, sublicense,
    and/or sell copies of the Software, and to permit persons to whom the
    Software is furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
    DEALINGS IN THE SOFTWARE.

    Note that the Qore library is released under a choice of three open-source
    licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
    information.
*/

#include <qore/Qore.h>
#include "qore/intern/ql_math.h"
#include "qore/intern/qore_number_private.h"

#include <climits>
#include <cmath>
#include <cstdio>
#include <cstdlib>
#include <string>

using namespace std;

static double minus_infinity() {
#ifdef INFINITY
   return -INFINITY;
#else
   // returns -infinity
   return log10((double)0.0);
#endif
}
typedef enum {ROUND, CEIL, FLOOR} FUN;
long long round_ceil_floor_intern(long long num, int prec, FUN fun) {
    if (prec >= 0)
        return num;

    string s = string(to_string(LLONG_MAX).length() + 4, '0') + to_string(num < 0 ? -num : num);
    int len = s.length();
    if (len < -prec)
        return 0;

    int pos = len + prec;
    unsigned long long n = stoll(s.substr(0, pos));

    switch (fun) {
        case ROUND: if (s[pos] - '0' > 4)                 n++; break;
        case CEIL:  if (num >= 0 && stoll(s.substr(pos))) n++; break;
        case FLOOR: if (num <  0 && stoll(s.substr(pos))) n++;
    }

    unsigned long long res = stoll(to_string(n) + string(-prec, '0'));
    return num < 0 ? -res : res;
}
# 77 "ql_math.qpp"
# 82 "ql_math.qpp"
# 100 "ql_math.qpp"
# 134 "ql_math.qpp"
# 154 "ql_math.qpp"
# 161 "ql_math.qpp"
# 181 "ql_math.qpp"
# 211 "ql_math.qpp"
# 233 "ql_math.qpp"
# 240 "ql_math.qpp"
# 260 "ql_math.qpp"
# 290 "ql_math.qpp"
# 312 "ql_math.qpp"
# 319 "ql_math.qpp"
# 337 "ql_math.qpp"
# 362 "ql_math.qpp"
# 379 "ql_math.qpp"
# 396 "ql_math.qpp"
# 413 "ql_math.qpp"
# 420 "ql_math.qpp"
# 436 "ql_math.qpp"
# 452 "ql_math.qpp"
# 459 "ql_math.qpp"
# 474 "ql_math.qpp"
# 489 "ql_math.qpp"
# 496 "ql_math.qpp"
# 511 "ql_math.qpp"
# 526 "ql_math.qpp"
# 533 "ql_math.qpp"
# 548 "ql_math.qpp"
# 563 "ql_math.qpp"
# 570 "ql_math.qpp"
# 585 "ql_math.qpp"
# 600 "ql_math.qpp"
# 607 "ql_math.qpp"
# 622 "ql_math.qpp"
# 637 "ql_math.qpp"
# 644 "ql_math.qpp"
# 659 "ql_math.qpp"
# 674 "ql_math.qpp"
# 681 "ql_math.qpp"
# 696 "ql_math.qpp"
# 711 "ql_math.qpp"
# 726 "ql_math.qpp"
# 741 "ql_math.qpp"
# 748 "ql_math.qpp"
# 764 "ql_math.qpp"
# 780 "ql_math.qpp"
# 787 "ql_math.qpp"
# 802 "ql_math.qpp"
# 817 "ql_math.qpp"
# 824 "ql_math.qpp"
# 839 "ql_math.qpp"
# 854 "ql_math.qpp"
# 861 "ql_math.qpp"
# 876 "ql_math.qpp"
# 891 "ql_math.qpp"
# 898 "ql_math.qpp"
# 913 "ql_math.qpp"
# 928 "ql_math.qpp"
# 935 "ql_math.qpp"
# 950 "ql_math.qpp"
# 965 "ql_math.qpp"
# 972 "ql_math.qpp"
# 987 "ql_math.qpp"
# 1002 "ql_math.qpp"
# 1009 "ql_math.qpp"
# 1024 "ql_math.qpp"
# 1031 "ql_math.qpp"
# 1046 "ql_math.qpp"
# 1061 "ql_math.qpp"
# 1068 "ql_math.qpp"
# 1083 "ql_math.qpp"
# 1102 "ql_math.qpp"
# 1109 "ql_math.qpp"
# 1124 "ql_math.qpp"
# 1139 "ql_math.qpp"
# 1146 "ql_math.qpp"
# 1169 "ql_math.qpp"
// int abs(int i) {}
static QoreValue f_abs_Vi(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 i = HARD_QORE_VALUE_INT(args, 0);
# 375 "ql_math.qpp"
    return i < 0 ? -i : i;
}

// number abs(number n) {}
static QoreValue f_abs_Vn(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreNumberNode* n = HARD_QORE_VALUE_NUMBER(args, 0);
# 392 "ql_math.qpp"
    return qore_number_private::doUnary(*n, mpfr_abs);
}

// float abs(softfloat f) {}
static QoreValue f_abs_vf(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    double f = HARD_QORE_VALUE_FLOAT(args, 0);
# 409 "ql_math.qpp"
    return fabs(f);
}

// float abs() {}
static QoreValue f_abs(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 416 "ql_math.qpp"
    return 0;
}

// number acos(number n) {}
static QoreValue f_acos_Vn(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreNumberNode* n = HARD_QORE_VALUE_NUMBER(args, 0);
# 692 "ql_math.qpp"
   return qore_number_private::doUnary(*n, mpfr_acos);
}

// float acos(softfloat f) {}
static QoreValue f_acos_vf(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    double f = HARD_QORE_VALUE_FLOAT(args, 0);
# 707 "ql_math.qpp"
   return acos(f);
}

// number asin(number n) {}
static QoreValue f_asin_Vn(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreNumberNode* n = HARD_QORE_VALUE_NUMBER(args, 0);
# 655 "ql_math.qpp"
   return qore_number_private::doUnary(*n, mpfr_asin);
}

// float asin(softfloat f) {}
static QoreValue f_asin_vf(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    double f = HARD_QORE_VALUE_FLOAT(args, 0);
# 670 "ql_math.qpp"
   return asin(f);
}

// float asin() {}
static QoreValue f_asin(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 677 "ql_math.qpp"
   return 0;
}

// number atan(number n) {}
static QoreValue f_atan_Vn(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreNumberNode* n = HARD_QORE_VALUE_NUMBER(args, 0);
# 722 "ql_math.qpp"
   return qore_number_private::doUnary(*n, mpfr_atan);
}

// float atan(softfloat f) {}
static QoreValue f_atan_vf(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    double f = HARD_QORE_VALUE_FLOAT(args, 0);
# 737 "ql_math.qpp"
   return atan(f);
}

// float atan() {}
static QoreValue f_atan(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 744 "ql_math.qpp"
   return 0;
}

// number atan2(number y, number x) {}
static QoreValue f_atan2_VnVn(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreNumberNode* y = HARD_QORE_VALUE_NUMBER(args, 0);
    const QoreNumberNode* x = HARD_QORE_VALUE_NUMBER(args, 1);
# 760 "ql_math.qpp"
   return qore_number_private::doBinary(*y, mpfr_atan2, *x);
}

// float atan2(softfloat y, softfloat x) {}
static QoreValue f_atan2_vfvf(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    double y = HARD_QORE_VALUE_FLOAT(args, 0);
    double x = HARD_QORE_VALUE_FLOAT(args, 1);
# 776 "ql_math.qpp"
   return atan2(y, x);
}

// float atan2() {}
static QoreValue f_atan2(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 783 "ql_math.qpp"
   return 0;
}

// number cbrt(number n) {}
static QoreValue f_cbrt_Vn(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreNumberNode* n = HARD_QORE_VALUE_NUMBER(args, 0);
# 507 "ql_math.qpp"
    return qore_number_private::doUnary(*n, mpfr_cbrt);
}

// float cbrt(softfloat f) {}
static QoreValue f_cbrt_vf(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    double f = HARD_QORE_VALUE_FLOAT(args, 0);
# 522 "ql_math.qpp"
    return cbrt(f);
}

// float cbrt() {}
static QoreValue f_cbrt(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 529 "ql_math.qpp"
    return 0;
}

// int ceil(int num, int prec = 0) {}
static QoreValue f_ceil_ViVi(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 num = HARD_QORE_VALUE_INT(args, 0);
    int64 prec = HARD_QORE_VALUE_INT(args, 1);
# 177 "ql_math.qpp"
    return round_ceil_floor_intern(num, prec, CEIL);
}

// float ceil(softfloat num, int prec = 0) {}
static QoreValue f_ceil_vfVi(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    double num = HARD_QORE_VALUE_FLOAT(args, 0);
    int64 prec = HARD_QORE_VALUE_INT(args, 1);
# 199 "ql_math.qpp"
    if (prec > 0) {
        int c = pow(10, prec);
        return ceil(c*num)/c;
    }
    if (prec < 0) {
        int c = pow(10, -prec);
        return c*ceil(num/c);
    }
    return ceil(num);
}

// number ceil(number num, int prec = 0) {}
static QoreValue f_ceil_VnVi(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreNumberNode* num = HARD_QORE_VALUE_NUMBER(args, 0);
    int64 prec = HARD_QORE_VALUE_INT(args, 1);
# 229 "ql_math.qpp"
    return qore_number_private::doRoundNR(*num, mpfr_ceil, prec);
}

// float ceil() {}
static QoreValue f_ceil(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 236 "ql_math.qpp"
   return 0;
}

// int compare(number n1, number n2, number epsilon) {}
static QoreValue f_compare_VnVnVn(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreNumberNode* n1 = HARD_QORE_VALUE_NUMBER(args, 0);
    const QoreNumberNode* n2 = HARD_QORE_VALUE_NUMBER(args, 1);
    const QoreNumberNode* epsilon = HARD_QORE_VALUE_NUMBER(args, 2);
# 1156 "ql_math.qpp"
    SimpleRefHolder<QoreNumberNode> v(n1->doMinus(*n2));
    if (v->zero()) {
        return 0;
    }
    int rv = v->sign();
    SimpleRefHolder<QoreNumberNode> tmp(qore_number_private::doUnary(**v, mpfr_abs));
    if (tmp->lessThan(*epsilon)) {
        return 0;
    }
    return rv;
}

// int compare(float n1, float n2, float epsilon) {}
static QoreValue f_compare_VfVfVf(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    double n1 = HARD_QORE_VALUE_FLOAT(args, 0);
    double n2 = HARD_QORE_VALUE_FLOAT(args, 1);
    double epsilon = HARD_QORE_VALUE_FLOAT(args, 2);
# 1179 "ql_math.qpp"
    double v = n1 - n2;
    if (v == 0.0) {
        return 0;
    }
    int rv = v < 0 ? -1 : 1;
    if (fabs(v) < epsilon) {
        return 0;
    }
    return rv;
}

// number cos(number n) {}
static QoreValue f_cos_Vn(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreNumberNode* n = HARD_QORE_VALUE_NUMBER(args, 0);
# 581 "ql_math.qpp"
    return qore_number_private::doUnary(*n, mpfr_cos);
}

// float cos(float f) {}
static QoreValue f_cos_Vf(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    double f = HARD_QORE_VALUE_FLOAT(args, 0);
# 596 "ql_math.qpp"
    return cos(f);
}

// float cos() {}
static QoreValue f_cos(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 603 "ql_math.qpp"
    return 1;
}

// number cosh(number n) {}
static QoreValue f_cosh_Vn(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreNumberNode* n = HARD_QORE_VALUE_NUMBER(args, 0);
# 835 "ql_math.qpp"
   return qore_number_private::doUnary(*n, mpfr_cosh);
}

// float cosh(softfloat f) {}
static QoreValue f_cosh_vf(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    double f = HARD_QORE_VALUE_FLOAT(args, 0);
# 850 "ql_math.qpp"
   return cosh(f);
}

// float cosh() {}
static QoreValue f_cosh(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 857 "ql_math.qpp"
   return 1;
}

// number exp(number n) {}
static QoreValue f_exp_Vn(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreNumberNode* n = HARD_QORE_VALUE_NUMBER(args, 0);
# 1042 "ql_math.qpp"
   return qore_number_private::doUnary(*n, mpfr_exp);
}

// float exp(softfloat f) {}
static QoreValue f_exp_vf(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    double f = HARD_QORE_VALUE_FLOAT(args, 0);
# 1057 "ql_math.qpp"
   return exp(f);
}

// float exp() {}
static QoreValue f_exp(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1064 "ql_math.qpp"
   return 1;
}

// number exp2(number n) {}
static QoreValue f_exp2_Vn(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreNumberNode* n = HARD_QORE_VALUE_NUMBER(args, 0);
# 1079 "ql_math.qpp"
   return qore_number_private::doUnary(*n, mpfr_exp2);
}

// float exp2(softfloat f) {}
static QoreValue f_exp2_vf(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    double f = HARD_QORE_VALUE_FLOAT(args, 0);
# 1094 "ql_math.qpp"
#ifdef HAVE_EXP2
   return exp2(f);
#else
   return pow(2, f);
#endif
    return QoreValue();
}

// float exp2() {}
static QoreValue f_exp2(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1105 "ql_math.qpp"
   return 1;
}

// number expm1(number n) {}
static QoreValue f_expm1_Vn(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreNumberNode* n = HARD_QORE_VALUE_NUMBER(args, 0);
# 1120 "ql_math.qpp"
   return qore_number_private::doUnary(*n, mpfr_expm1);
}

// float expm1(softfloat f) {}
static QoreValue f_expm1_vf(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    double f = HARD_QORE_VALUE_FLOAT(args, 0);
# 1135 "ql_math.qpp"
    return expm1(f);
}

// float expm1() {}
static QoreValue f_expm1(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1142 "ql_math.qpp"
    return 0;
}

// int floor(int num, int prec = 0) {}
static QoreValue f_floor_ViVi(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 num = HARD_QORE_VALUE_INT(args, 0);
    int64 prec = HARD_QORE_VALUE_INT(args, 1);
# 256 "ql_math.qpp"
    return round_ceil_floor_intern(num, prec, FLOOR);
}

// float floor(softfloat f, int prec = 0) {}
static QoreValue f_floor_vfVi(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    double f = HARD_QORE_VALUE_FLOAT(args, 0);
    int64 prec = HARD_QORE_VALUE_INT(args, 1);
# 278 "ql_math.qpp"
    if (prec > 0) {
        int c = pow(10, prec);
        return floor(c*f)/c;
    }
    if (prec < 0) {
        int c = pow(10, -prec);
        return c*floor(f/c);
    }
    return floor(f);
}

// number floor(number num, int prec = 0) {}
static QoreValue f_floor_VnVi(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreNumberNode* num = HARD_QORE_VALUE_NUMBER(args, 0);
    int64 prec = HARD_QORE_VALUE_INT(args, 1);
# 308 "ql_math.qpp"
    return qore_number_private::doRoundNR(*num, mpfr_floor, prec);
}

// float floor() {}
static QoreValue f_floor(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 315 "ql_math.qpp"
   return 0;
}

// number hypot(number x, number y) {}
static QoreValue f_hypot_VnVn(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreNumberNode* x = HARD_QORE_VALUE_NUMBER(args, 0);
    const QoreNumberNode* y = HARD_QORE_VALUE_NUMBER(args, 1);
# 432 "ql_math.qpp"
    return qore_number_private::doBinary(*x, mpfr_hypot, *y);
}

// float hypot(softfloat x, softfloat y) {}
static QoreValue f_hypot_vfvf(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    double x = HARD_QORE_VALUE_FLOAT(args, 0);
    double y = HARD_QORE_VALUE_FLOAT(args, 1);
# 448 "ql_math.qpp"
    return hypot(x, y);
}

// float hypot() {}
static QoreValue f_hypot(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 455 "ql_math.qpp"
    return 0;
}

// number log10(number n) {}
static QoreValue f_log10_Vn(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreNumberNode* n = HARD_QORE_VALUE_NUMBER(args, 0);
# 946 "ql_math.qpp"
   return qore_number_private::doUnary(*n, mpfr_log10);
}

// float log10(softfloat f) {}
static QoreValue f_log10_vf(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    double f = HARD_QORE_VALUE_FLOAT(args, 0);
# 961 "ql_math.qpp"
   return log10(f);
}

// float log10() {}
static QoreValue f_log10(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 968 "ql_math.qpp"
   return minus_infinity();
}

// number log1p(number n) {}
static QoreValue f_log1p_Vn(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreNumberNode* n = HARD_QORE_VALUE_NUMBER(args, 0);
# 983 "ql_math.qpp"
   return qore_number_private::doUnary(*n, mpfr_log1p);
}

// float log1p(softfloat f) {}
static QoreValue f_log1p_vf(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    double f = HARD_QORE_VALUE_FLOAT(args, 0);
# 998 "ql_math.qpp"
   return log1p(f);
}

// float log1p() {}
static QoreValue f_log1p(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1005 "ql_math.qpp"
   return 0;
}

// float logb(softfloat f) {}
static QoreValue f_logb_vf(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    double f = HARD_QORE_VALUE_FLOAT(args, 0);
# 1020 "ql_math.qpp"
   return logb(f);
}

// float logb() {}
static QoreValue f_logb(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1027 "ql_math.qpp"
   return minus_infinity();
}

// number nlog(number n) {}
static QoreValue f_nlog_Vn(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreNumberNode* n = HARD_QORE_VALUE_NUMBER(args, 0);
# 909 "ql_math.qpp"
   return qore_number_private::doUnary(*n, mpfr_log);
}

// float nlog(softfloat f) {}
static QoreValue f_nlog_vf(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    double f = HARD_QORE_VALUE_FLOAT(args, 0);
# 924 "ql_math.qpp"
   return log(f);
}

// float nlog() {}
static QoreValue f_nlog(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 931 "ql_math.qpp"
   return minus_infinity();
}

// number pow(number x, number y) {}
static QoreValue f_pow_VnVn(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreNumberNode* x = HARD_QORE_VALUE_NUMBER(args, 0);
    const QoreNumberNode* y = HARD_QORE_VALUE_NUMBER(args, 1);
# 333 "ql_math.qpp"
    return qore_number_private::doBinary(*x, mpfr_pow, *y, xsink);
}

// float pow(softfloat x = 0.0, softfloat y = 0.0) {}
static QoreValue f_pow_vfvf(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    double x = HARD_QORE_VALUE_FLOAT(args, 0);
    double y = HARD_QORE_VALUE_FLOAT(args, 1);
# 352 "ql_math.qpp"
    if (x == 0.0 && y < 0)
        return xsink->raiseException("DIVISION-BY-ZERO", "pow(x, y) if x = 0 then y must be a non-negative value");

    if (x < 0 && y != ceil(y))
        return xsink->raiseException("INVALID-POW-ARGUMENTS", "pow(x, y) x cannot be negative when y is not an integer value");

    return pow(x, y);
}

// int round(softint num, int prec = 0) {}
static QoreValue f_round_viVi(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 num = HARD_QORE_VALUE_INT(args, 0);
    int64 prec = HARD_QORE_VALUE_INT(args, 1);
# 96 "ql_math.qpp"
    return round_ceil_floor_intern(num, prec, ROUND);
}

// float round(softfloat num, int prec = 0) {}
static QoreValue f_round_vfVi(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    double num = HARD_QORE_VALUE_FLOAT(args, 0);
    int64 prec = HARD_QORE_VALUE_INT(args, 1);
# 118 "ql_math.qpp"
#ifdef HAVE_ROUND
    if (prec > 0) {
        int c = pow(10, prec);
        return round(c*num)/c;
    }
    if (prec < 0) {
        int c = pow(10, -prec);
        return c*round(num/c);
    }
    return round(num);
#else
    return missing_function_error("round", "ROUND", xsink);
#endif
    return QoreValue();
}

// number round(number num, int prec = 0) {}
static QoreValue f_round_VnVi(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreNumberNode* num = HARD_QORE_VALUE_NUMBER(args, 0);
    int64 prec = HARD_QORE_VALUE_INT(args, 1);
# 150 "ql_math.qpp"
    return qore_number_private::doRoundNR(*num, mpfr_round, prec);
}

// float round() {}
static QoreValue f_round(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 157 "ql_math.qpp"
   return 0;
}

// number sin(number n) {}
static QoreValue f_sin_Vn(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreNumberNode* n = HARD_QORE_VALUE_NUMBER(args, 0);
# 544 "ql_math.qpp"
    return qore_number_private::doUnary(*n, mpfr_sin);
}

// float sin(softfloat f) {}
static QoreValue f_sin_vf(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    double f = HARD_QORE_VALUE_FLOAT(args, 0);
# 559 "ql_math.qpp"
    return sin(f);
}

// float sin() {}
static QoreValue f_sin(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 566 "ql_math.qpp"
    return 0;
}

// number sinh(number n) {}
static QoreValue f_sinh_Vn(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreNumberNode* n = HARD_QORE_VALUE_NUMBER(args, 0);
# 798 "ql_math.qpp"
   return qore_number_private::doUnary(*n, mpfr_sinh);
}

// float sinh(softfloat f) {}
static QoreValue f_sinh_vf(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    double f = HARD_QORE_VALUE_FLOAT(args, 0);
# 813 "ql_math.qpp"
   return sinh(f);
}

// float sinh() {}
static QoreValue f_sinh(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 820 "ql_math.qpp"
   return 0;
}

// number sqrt(number n) {}
static QoreValue f_sqrt_Vn(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreNumberNode* n = HARD_QORE_VALUE_NUMBER(args, 0);
# 470 "ql_math.qpp"
    return qore_number_private::doUnary(*n, mpfr_sqrt);
}

// float sqrt(softfloat f) {}
static QoreValue f_sqrt_vf(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    double f = HARD_QORE_VALUE_FLOAT(args, 0);
# 485 "ql_math.qpp"
    return sqrt(f);
}

// float sqrt() {}
static QoreValue f_sqrt(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 492 "ql_math.qpp"
   return 0;
}

// number tan(number n) {}
static QoreValue f_tan_Vn(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreNumberNode* n = HARD_QORE_VALUE_NUMBER(args, 0);
# 618 "ql_math.qpp"
    return qore_number_private::doUnary(*n, mpfr_tan);
}

// float tan(softfloat f) {}
static QoreValue f_tan_vf(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    double f = HARD_QORE_VALUE_FLOAT(args, 0);
# 633 "ql_math.qpp"
    return tan(f);
}

// float tan() {}
static QoreValue f_tan(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 640 "ql_math.qpp"
    return 0;
}

// number tanh(number n) {}
static QoreValue f_tanh_Vn(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreNumberNode* n = HARD_QORE_VALUE_NUMBER(args, 0);
# 872 "ql_math.qpp"
   return qore_number_private::doUnary(*n, mpfr_tanh);
}

// float tanh(softfloat f) {}
static QoreValue f_tanh_vf(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    double f = HARD_QORE_VALUE_FLOAT(args, 0);
# 887 "ql_math.qpp"
   return tanh(f);
}

// float tanh() {}
static QoreValue f_tanh(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 894 "ql_math.qpp"
   return 0;
}


DLLLOCAL void init_math_functions(QoreNamespace& ns) {
    // int abs(int i) {}
    ns.addBuiltinVariant("abs", (q_func_n_t)f_abs_Vi, QCF_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo, 1, bigIntTypeInfo, QORE_PARAM_NO_ARG, "i");
    // number abs(number n) {}
    ns.addBuiltinVariant("abs", (q_func_n_t)f_abs_Vn, QCF_CONSTANT, QDOM_DEFAULT, numberTypeInfo, 1, numberTypeInfo, QORE_PARAM_NO_ARG, "n");
    // float abs(softfloat f) {}
    ns.addBuiltinVariant("abs", (q_func_n_t)f_abs_vf, QCF_CONSTANT, QDOM_DEFAULT, floatTypeInfo, 1, softFloatTypeInfo, QORE_PARAM_NO_ARG, "f");
    // float abs() {}
    ns.addBuiltinVariant("abs", (q_func_n_t)f_abs, QCF_RUNTIME_NOOP, QDOM_DEFAULT, floatTypeInfo);
    // number acos(number n) {}
    ns.addBuiltinVariant("acos", (q_func_n_t)f_acos_Vn, QCF_CONSTANT, QDOM_DEFAULT, numberTypeInfo, 1, numberTypeInfo, QORE_PARAM_NO_ARG, "n");
    // float acos(softfloat f) {}
    ns.addBuiltinVariant("acos", (q_func_n_t)f_acos_vf, QCF_CONSTANT, QDOM_DEFAULT, floatTypeInfo, 1, softFloatTypeInfo, QORE_PARAM_NO_ARG, "f");
    // number asin(number n) {}
    ns.addBuiltinVariant("asin", (q_func_n_t)f_asin_Vn, QCF_CONSTANT, QDOM_DEFAULT, numberTypeInfo, 1, numberTypeInfo, QORE_PARAM_NO_ARG, "n");
    // float asin(softfloat f) {}
    ns.addBuiltinVariant("asin", (q_func_n_t)f_asin_vf, QCF_CONSTANT, QDOM_DEFAULT, floatTypeInfo, 1, softFloatTypeInfo, QORE_PARAM_NO_ARG, "f");
    // float asin() {}
    ns.addBuiltinVariant("asin", (q_func_n_t)f_asin, QCF_RUNTIME_NOOP, QDOM_DEFAULT, floatTypeInfo);
    // number atan(number n) {}
    ns.addBuiltinVariant("atan", (q_func_n_t)f_atan_Vn, QCF_CONSTANT, QDOM_DEFAULT, numberTypeInfo, 1, numberTypeInfo, QORE_PARAM_NO_ARG, "n");
    // float atan(softfloat f) {}
    ns.addBuiltinVariant("atan", (q_func_n_t)f_atan_vf, QCF_CONSTANT, QDOM_DEFAULT, floatTypeInfo, 1, softFloatTypeInfo, QORE_PARAM_NO_ARG, "f");
    // float atan() {}
    ns.addBuiltinVariant("atan", (q_func_n_t)f_atan, QCF_RUNTIME_NOOP, QDOM_DEFAULT, floatTypeInfo);
    // number atan2(number y, number x) {}
    ns.addBuiltinVariant("atan2", (q_func_n_t)f_atan2_VnVn, QCF_CONSTANT, QDOM_DEFAULT, numberTypeInfo, 2, numberTypeInfo, QORE_PARAM_NO_ARG, "y", numberTypeInfo, QORE_PARAM_NO_ARG, "x");
    // float atan2(softfloat y, softfloat x) {}
    ns.addBuiltinVariant("atan2", (q_func_n_t)f_atan2_vfvf, QCF_CONSTANT, QDOM_DEFAULT, floatTypeInfo, 2, softFloatTypeInfo, QORE_PARAM_NO_ARG, "y", softFloatTypeInfo, QORE_PARAM_NO_ARG, "x");
    // float atan2() {}
    ns.addBuiltinVariant("atan2", (q_func_n_t)f_atan2, QCF_RUNTIME_NOOP, QDOM_DEFAULT, floatTypeInfo);
    // number cbrt(number n) {}
    ns.addBuiltinVariant("cbrt", (q_func_n_t)f_cbrt_Vn, QCF_CONSTANT, QDOM_DEFAULT, numberTypeInfo, 1, numberTypeInfo, QORE_PARAM_NO_ARG, "n");
    // float cbrt(softfloat f) {}
    ns.addBuiltinVariant("cbrt", (q_func_n_t)f_cbrt_vf, QCF_CONSTANT, QDOM_DEFAULT, floatTypeInfo, 1, softFloatTypeInfo, QORE_PARAM_NO_ARG, "f");
    // float cbrt() {}
    ns.addBuiltinVariant("cbrt", (q_func_n_t)f_cbrt, QCF_RUNTIME_NOOP, QDOM_DEFAULT, floatTypeInfo);
    // int ceil(int num, int prec = 0) {}
    ns.addBuiltinVariant("ceil", (q_func_n_t)f_ceil_ViVi, QCF_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo, 2, bigIntTypeInfo, QORE_PARAM_NO_ARG, "num", bigIntTypeInfo, QoreSimpleValue().assign((int64)0), "prec");
    // float ceil(softfloat num, int prec = 0) {}
    ns.addBuiltinVariant("ceil", (q_func_n_t)f_ceil_vfVi, QCF_CONSTANT, QDOM_DEFAULT, floatTypeInfo, 2, softFloatTypeInfo, QORE_PARAM_NO_ARG, "num", bigIntTypeInfo, QoreSimpleValue().assign((int64)0), "prec");
    // number ceil(number num, int prec = 0) {}
    ns.addBuiltinVariant("ceil", (q_func_n_t)f_ceil_VnVi, QCF_CONSTANT, QDOM_DEFAULT, numberTypeInfo, 2, numberTypeInfo, QORE_PARAM_NO_ARG, "num", bigIntTypeInfo, QoreSimpleValue().assign((int64)0), "prec");
    // float ceil() {}
    ns.addBuiltinVariant("ceil", (q_func_n_t)f_ceil, QCF_RUNTIME_NOOP, QDOM_DEFAULT, floatTypeInfo);
    // int compare(number n1, number n2, number epsilon) {}
    ns.addBuiltinVariant("compare", (q_func_n_t)f_compare_VnVnVn, QCF_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo, 3, numberTypeInfo, QORE_PARAM_NO_ARG, "n1", numberTypeInfo, QORE_PARAM_NO_ARG, "n2", numberTypeInfo, QORE_PARAM_NO_ARG, "epsilon");
    // int compare(float n1, float n2, float epsilon) {}
    ns.addBuiltinVariant("compare", (q_func_n_t)f_compare_VfVfVf, QCF_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo, 3, floatTypeInfo, QORE_PARAM_NO_ARG, "n1", floatTypeInfo, QORE_PARAM_NO_ARG, "n2", floatTypeInfo, QORE_PARAM_NO_ARG, "epsilon");
    // number cos(number n) {}
    ns.addBuiltinVariant("cos", (q_func_n_t)f_cos_Vn, QCF_CONSTANT, QDOM_DEFAULT, numberTypeInfo, 1, numberTypeInfo, QORE_PARAM_NO_ARG, "n");
    // float cos(float f) {}
    ns.addBuiltinVariant("cos", (q_func_n_t)f_cos_Vf, QCF_CONSTANT, QDOM_DEFAULT, floatTypeInfo, 1, floatTypeInfo, QORE_PARAM_NO_ARG, "f");
    // float cos() {}
    ns.addBuiltinVariant("cos", (q_func_n_t)f_cos, QCF_RUNTIME_NOOP, QDOM_DEFAULT, floatTypeInfo);
    // number cosh(number n) {}
    ns.addBuiltinVariant("cosh", (q_func_n_t)f_cosh_Vn, QCF_CONSTANT, QDOM_DEFAULT, numberTypeInfo, 1, numberTypeInfo, QORE_PARAM_NO_ARG, "n");
    // float cosh(softfloat f) {}
    ns.addBuiltinVariant("cosh", (q_func_n_t)f_cosh_vf, QCF_CONSTANT, QDOM_DEFAULT, floatTypeInfo, 1, softFloatTypeInfo, QORE_PARAM_NO_ARG, "f");
    // float cosh() {}
    ns.addBuiltinVariant("cosh", (q_func_n_t)f_cosh, QCF_RUNTIME_NOOP, QDOM_DEFAULT, floatTypeInfo);
    // number exp(number n) {}
    ns.addBuiltinVariant("exp", (q_func_n_t)f_exp_Vn, QCF_CONSTANT, QDOM_DEFAULT, numberTypeInfo, 1, numberTypeInfo, QORE_PARAM_NO_ARG, "n");
    // float exp(softfloat f) {}
    ns.addBuiltinVariant("exp", (q_func_n_t)f_exp_vf, QCF_CONSTANT, QDOM_DEFAULT, floatTypeInfo, 1, softFloatTypeInfo, QORE_PARAM_NO_ARG, "f");
    // float exp() {}
    ns.addBuiltinVariant("exp", (q_func_n_t)f_exp, QCF_RUNTIME_NOOP, QDOM_DEFAULT, floatTypeInfo);
    // number exp2(number n) {}
    ns.addBuiltinVariant("exp2", (q_func_n_t)f_exp2_Vn, QCF_CONSTANT, QDOM_DEFAULT, numberTypeInfo, 1, numberTypeInfo, QORE_PARAM_NO_ARG, "n");
    // float exp2(softfloat f) {}
    ns.addBuiltinVariant("exp2", (q_func_n_t)f_exp2_vf, QCF_CONSTANT, QDOM_DEFAULT, floatTypeInfo, 1, softFloatTypeInfo, QORE_PARAM_NO_ARG, "f");
    // float exp2() {}
    ns.addBuiltinVariant("exp2", (q_func_n_t)f_exp2, QCF_RUNTIME_NOOP, QDOM_DEFAULT, floatTypeInfo);
    // number expm1(number n) {}
    ns.addBuiltinVariant("expm1", (q_func_n_t)f_expm1_Vn, QCF_CONSTANT, QDOM_DEFAULT, numberTypeInfo, 1, numberTypeInfo, QORE_PARAM_NO_ARG, "n");
    // float expm1(softfloat f) {}
    ns.addBuiltinVariant("expm1", (q_func_n_t)f_expm1_vf, QCF_CONSTANT, QDOM_DEFAULT, floatTypeInfo, 1, softFloatTypeInfo, QORE_PARAM_NO_ARG, "f");
    // float expm1() {}
    ns.addBuiltinVariant("expm1", (q_func_n_t)f_expm1, QCF_RUNTIME_NOOP, QDOM_DEFAULT, floatTypeInfo);
    // int floor(int num, int prec = 0) {}
    ns.addBuiltinVariant("floor", (q_func_n_t)f_floor_ViVi, QCF_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo, 2, bigIntTypeInfo, QORE_PARAM_NO_ARG, "num", bigIntTypeInfo, QoreSimpleValue().assign((int64)0), "prec");
    // float floor(softfloat f, int prec = 0) {}
    ns.addBuiltinVariant("floor", (q_func_n_t)f_floor_vfVi, QCF_CONSTANT, QDOM_DEFAULT, floatTypeInfo, 2, softFloatTypeInfo, QORE_PARAM_NO_ARG, "f", bigIntTypeInfo, QoreSimpleValue().assign((int64)0), "prec");
    // number floor(number num, int prec = 0) {}
    ns.addBuiltinVariant("floor", (q_func_n_t)f_floor_VnVi, QCF_CONSTANT, QDOM_DEFAULT, numberTypeInfo, 2, numberTypeInfo, QORE_PARAM_NO_ARG, "num", bigIntTypeInfo, QoreSimpleValue().assign((int64)0), "prec");
    // float floor() {}
    ns.addBuiltinVariant("floor", (q_func_n_t)f_floor, QCF_RUNTIME_NOOP, QDOM_DEFAULT, floatTypeInfo);
    // number hypot(number x, number y) {}
    ns.addBuiltinVariant("hypot", (q_func_n_t)f_hypot_VnVn, QCF_CONSTANT, QDOM_DEFAULT, numberTypeInfo, 2, numberTypeInfo, QORE_PARAM_NO_ARG, "x", numberTypeInfo, QORE_PARAM_NO_ARG, "y");
    // float hypot(softfloat x, softfloat y) {}
    ns.addBuiltinVariant("hypot", (q_func_n_t)f_hypot_vfvf, QCF_CONSTANT, QDOM_DEFAULT, floatTypeInfo, 2, softFloatTypeInfo, QORE_PARAM_NO_ARG, "x", softFloatTypeInfo, QORE_PARAM_NO_ARG, "y");
    // float hypot() {}
    ns.addBuiltinVariant("hypot", (q_func_n_t)f_hypot, QCF_RUNTIME_NOOP, QDOM_DEFAULT, floatTypeInfo);
    // number log10(number n) {}
    ns.addBuiltinVariant("log10", (q_func_n_t)f_log10_Vn, QCF_CONSTANT, QDOM_DEFAULT, numberTypeInfo, 1, numberTypeInfo, QORE_PARAM_NO_ARG, "n");
    // float log10(softfloat f) {}
    ns.addBuiltinVariant("log10", (q_func_n_t)f_log10_vf, QCF_CONSTANT, QDOM_DEFAULT, floatTypeInfo, 1, softFloatTypeInfo, QORE_PARAM_NO_ARG, "f");
    // float log10() {}
    ns.addBuiltinVariant("log10", (q_func_n_t)f_log10, QCF_RUNTIME_NOOP, QDOM_DEFAULT, floatTypeInfo);
    // number log1p(number n) {}
    ns.addBuiltinVariant("log1p", (q_func_n_t)f_log1p_Vn, QCF_CONSTANT, QDOM_DEFAULT, numberTypeInfo, 1, numberTypeInfo, QORE_PARAM_NO_ARG, "n");
    // float log1p(softfloat f) {}
    ns.addBuiltinVariant("log1p", (q_func_n_t)f_log1p_vf, QCF_CONSTANT, QDOM_DEFAULT, floatTypeInfo, 1, softFloatTypeInfo, QORE_PARAM_NO_ARG, "f");
    // float log1p() {}
    ns.addBuiltinVariant("log1p", (q_func_n_t)f_log1p, QCF_RUNTIME_NOOP, QDOM_DEFAULT, floatTypeInfo);
    // float logb(softfloat f) {}
    ns.addBuiltinVariant("logb", (q_func_n_t)f_logb_vf, QCF_CONSTANT, QDOM_DEFAULT, floatTypeInfo, 1, softFloatTypeInfo, QORE_PARAM_NO_ARG, "f");
    // float logb() {}
    ns.addBuiltinVariant("logb", (q_func_n_t)f_logb, QCF_RUNTIME_NOOP, QDOM_DEFAULT, floatTypeInfo);
    // number nlog(number n) {}
    ns.addBuiltinVariant("nlog", (q_func_n_t)f_nlog_Vn, QCF_CONSTANT, QDOM_DEFAULT, numberTypeInfo, 1, numberTypeInfo, QORE_PARAM_NO_ARG, "n");
    // float nlog(softfloat f) {}
    ns.addBuiltinVariant("nlog", (q_func_n_t)f_nlog_vf, QCF_CONSTANT, QDOM_DEFAULT, floatTypeInfo, 1, softFloatTypeInfo, QORE_PARAM_NO_ARG, "f");
    // float nlog() {}
    ns.addBuiltinVariant("nlog", (q_func_n_t)f_nlog, QCF_RUNTIME_NOOP, QDOM_DEFAULT, floatTypeInfo);
    // number pow(number x, number y) {}
    ns.addBuiltinVariant("pow", (q_func_n_t)f_pow_VnVn, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, numberTypeInfo, 2, numberTypeInfo, QORE_PARAM_NO_ARG, "x", numberTypeInfo, QORE_PARAM_NO_ARG, "y");
    // float pow(softfloat x = 0.0, softfloat y = 0.0) {}
    ns.addBuiltinVariant("pow", (q_func_n_t)f_pow_vfvf, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, floatTypeInfo, 2, softFloatTypeInfo, QoreSimpleValue().assign((double)0.0), "x", softFloatTypeInfo, QoreSimpleValue().assign((double)0.0), "y");
    // int round(softint num, int prec = 0) {}
    ns.addBuiltinVariant("round", (q_func_n_t)f_round_viVi, QCF_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo, 2, softBigIntTypeInfo, QORE_PARAM_NO_ARG, "num", bigIntTypeInfo, QoreSimpleValue().assign((int64)0), "prec");
    // float round(softfloat num, int prec = 0) {}
    ns.addBuiltinVariant("round", (q_func_n_t)f_round_vfVi, QCF_CONSTANT, QDOM_DEFAULT, floatTypeInfo, 2, softFloatTypeInfo, QORE_PARAM_NO_ARG, "num", bigIntTypeInfo, QoreSimpleValue().assign((int64)0), "prec");
    // number round(number num, int prec = 0) {}
    ns.addBuiltinVariant("round", (q_func_n_t)f_round_VnVi, QCF_CONSTANT, QDOM_DEFAULT, numberTypeInfo, 2, numberTypeInfo, QORE_PARAM_NO_ARG, "num", bigIntTypeInfo, QoreSimpleValue().assign((int64)0), "prec");
    // float round() {}
    ns.addBuiltinVariant("round", (q_func_n_t)f_round, QCF_RUNTIME_NOOP, QDOM_DEFAULT, floatTypeInfo);
    // number sin(number n) {}
    ns.addBuiltinVariant("sin", (q_func_n_t)f_sin_Vn, QCF_CONSTANT, QDOM_DEFAULT, numberTypeInfo, 1, numberTypeInfo, QORE_PARAM_NO_ARG, "n");
    // float sin(softfloat f) {}
    ns.addBuiltinVariant("sin", (q_func_n_t)f_sin_vf, QCF_CONSTANT, QDOM_DEFAULT, floatTypeInfo, 1, softFloatTypeInfo, QORE_PARAM_NO_ARG, "f");
    // float sin() {}
    ns.addBuiltinVariant("sin", (q_func_n_t)f_sin, QCF_RUNTIME_NOOP, QDOM_DEFAULT, floatTypeInfo);
    // number sinh(number n) {}
    ns.addBuiltinVariant("sinh", (q_func_n_t)f_sinh_Vn, QCF_CONSTANT, QDOM_DEFAULT, numberTypeInfo, 1, numberTypeInfo, QORE_PARAM_NO_ARG, "n");
    // float sinh(softfloat f) {}
    ns.addBuiltinVariant("sinh", (q_func_n_t)f_sinh_vf, QCF_CONSTANT, QDOM_DEFAULT, floatTypeInfo, 1, softFloatTypeInfo, QORE_PARAM_NO_ARG, "f");
    // float sinh() {}
    ns.addBuiltinVariant("sinh", (q_func_n_t)f_sinh, QCF_RUNTIME_NOOP, QDOM_DEFAULT, floatTypeInfo);
    // number sqrt(number n) {}
    ns.addBuiltinVariant("sqrt", (q_func_n_t)f_sqrt_Vn, QCF_CONSTANT, QDOM_DEFAULT, numberTypeInfo, 1, numberTypeInfo, QORE_PARAM_NO_ARG, "n");
    // float sqrt(softfloat f) {}
    ns.addBuiltinVariant("sqrt", (q_func_n_t)f_sqrt_vf, QCF_CONSTANT, QDOM_DEFAULT, floatTypeInfo, 1, softFloatTypeInfo, QORE_PARAM_NO_ARG, "f");
    // float sqrt() {}
    ns.addBuiltinVariant("sqrt", (q_func_n_t)f_sqrt, QCF_RUNTIME_NOOP, QDOM_DEFAULT, floatTypeInfo);
    // number tan(number n) {}
    ns.addBuiltinVariant("tan", (q_func_n_t)f_tan_Vn, QCF_CONSTANT, QDOM_DEFAULT, numberTypeInfo, 1, numberTypeInfo, QORE_PARAM_NO_ARG, "n");
    // float tan(softfloat f) {}
    ns.addBuiltinVariant("tan", (q_func_n_t)f_tan_vf, QCF_CONSTANT, QDOM_DEFAULT, floatTypeInfo, 1, softFloatTypeInfo, QORE_PARAM_NO_ARG, "f");
    // float tan() {}
    ns.addBuiltinVariant("tan", (q_func_n_t)f_tan, QCF_RUNTIME_NOOP, QDOM_DEFAULT, floatTypeInfo);
    // number tanh(number n) {}
    ns.addBuiltinVariant("tanh", (q_func_n_t)f_tanh_Vn, QCF_CONSTANT, QDOM_DEFAULT, numberTypeInfo, 1, numberTypeInfo, QORE_PARAM_NO_ARG, "n");
    // float tanh(softfloat f) {}
    ns.addBuiltinVariant("tanh", (q_func_n_t)f_tanh_vf, QCF_CONSTANT, QDOM_DEFAULT, floatTypeInfo, 1, softFloatTypeInfo, QORE_PARAM_NO_ARG, "f");
    // float tanh() {}
    ns.addBuiltinVariant("tanh", (q_func_n_t)f_tanh, QCF_RUNTIME_NOOP, QDOM_DEFAULT, floatTypeInfo);

}

DLLLOCAL void init_math_constants(QoreNamespace& ns) {
# 54 "ql_math.qpp"
# 61 "ql_math.qpp"
# 66 "ql_math.qpp"
    ns.addConstant("MAXINT", ((int64)LLONG_MAX));
    ns.addConstant("MININT", (QoreValue(-LLONG_MAX - 1)));
    ns.addConstant("M_PI", ((double)3.14159265358979323846));
    ns.addConstant("M_PIn", (pi_number()));
# 77 "ql_math.qpp"
# 82 "ql_math.qpp"
# 100 "ql_math.qpp"
# 134 "ql_math.qpp"
# 154 "ql_math.qpp"
# 161 "ql_math.qpp"
# 181 "ql_math.qpp"
# 211 "ql_math.qpp"
# 233 "ql_math.qpp"
# 240 "ql_math.qpp"
# 260 "ql_math.qpp"
# 290 "ql_math.qpp"
# 312 "ql_math.qpp"
# 319 "ql_math.qpp"
# 337 "ql_math.qpp"
# 362 "ql_math.qpp"
# 379 "ql_math.qpp"
# 396 "ql_math.qpp"
# 413 "ql_math.qpp"
# 420 "ql_math.qpp"
# 436 "ql_math.qpp"
# 452 "ql_math.qpp"
# 459 "ql_math.qpp"
# 474 "ql_math.qpp"
# 489 "ql_math.qpp"
# 496 "ql_math.qpp"
# 511 "ql_math.qpp"
# 526 "ql_math.qpp"
# 533 "ql_math.qpp"
# 548 "ql_math.qpp"
# 563 "ql_math.qpp"
# 570 "ql_math.qpp"
# 585 "ql_math.qpp"
# 600 "ql_math.qpp"
# 607 "ql_math.qpp"
# 622 "ql_math.qpp"
# 637 "ql_math.qpp"
# 644 "ql_math.qpp"
# 659 "ql_math.qpp"
# 674 "ql_math.qpp"
# 681 "ql_math.qpp"
# 696 "ql_math.qpp"
# 711 "ql_math.qpp"
# 726 "ql_math.qpp"
# 741 "ql_math.qpp"
# 748 "ql_math.qpp"
# 764 "ql_math.qpp"
# 780 "ql_math.qpp"
# 787 "ql_math.qpp"
# 802 "ql_math.qpp"
# 817 "ql_math.qpp"
# 824 "ql_math.qpp"
# 839 "ql_math.qpp"
# 854 "ql_math.qpp"
# 861 "ql_math.qpp"
# 876 "ql_math.qpp"
# 891 "ql_math.qpp"
# 898 "ql_math.qpp"
# 913 "ql_math.qpp"
# 928 "ql_math.qpp"
# 935 "ql_math.qpp"
# 950 "ql_math.qpp"
# 965 "ql_math.qpp"
# 972 "ql_math.qpp"
# 987 "ql_math.qpp"
# 1002 "ql_math.qpp"
# 1009 "ql_math.qpp"
# 1024 "ql_math.qpp"
# 1031 "ql_math.qpp"
# 1046 "ql_math.qpp"
# 1061 "ql_math.qpp"
# 1068 "ql_math.qpp"
# 1083 "ql_math.qpp"
# 1102 "ql_math.qpp"
# 1109 "ql_math.qpp"
# 1124 "ql_math.qpp"
# 1139 "ql_math.qpp"
# 1146 "ql_math.qpp"
# 1169 "ql_math.qpp"

}
