/* -*- mode: c++; indent-tabs-mode: nil -*- */
/** @file QC_AbstractMethod.cpp AbstractMethod class definition */
/*
    Qore Programming Language

    Copyright (C) 2003 - 2023 Qore Technologies, s.r.o.

    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
    to deal in the Software without restriction, including without limitation
    the rights to use, copy, modify, merge, publish, distribute, sublicense,
    and/or sell copies of the Software, and to permit persons to whom the
    Software is furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
    DEALINGS IN THE SOFTWARE.

    Note that the Qore library is released under a choice of three open-source
    licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
    information.
*/

#include "qore_reflection.h"
#include "QC_AbstractClass.h"
#include "QC_AbstractMethod.h"
#include "QC_AbstractVariant.h"
#include "QC_AbstractMethodVariant.h"
#include "QC_Type.h"
#include "QC_AbstractClass.h"

int check_call(QoreObject* obj, const QoreMethod& m, ExceptionSink* xsink) {
    if (!obj->validInstanceOf(*m.getClass())) {
        xsink->raiseException("METHOD-ERROR", "object is of class '%s' which is not compatible with method '%s()' of class '%s'", obj->getClassName(), m.getName(), m.getClassName());
        return -1;
    }
    return 0;
}

QoreReflectionMethod::QoreReflectionMethod(const QoreClass* cls, const char* name, ExceptionSink* xsink, method_type_e type) : QoreReflectionFunction(getProgram()) {
    setMethod(cls, name, xsink, type);
}

QoreReflectionMethod::QoreReflectionMethod(const char* cls_path, const char* name, ExceptionSink* xsink, method_type_e type) : QoreReflectionFunction(getProgram()) {
    const QoreClass* cls = pgm->findClass(cls_path, xsink);
    if (!cls) {
        if (!*xsink) {
            xsink->raiseException("UNKNOWN-CLASS", "cannot resolve class '%s'", cls_path);
        }
        return;
    }

    setMethod(cls, name, xsink, type);
}

QoreReflectionMethod::QoreReflectionMethod(QoreProgram* pgm, const QoreMethod* m) : QoreReflectionFunction(pgm, m->getFunction()), m(m) {
    setType();
}

void QoreReflectionMethod::setMethod(const QoreClass* cls, const char* name, ExceptionSink* xsink, method_type_e type) {
    if (type == MT_None || type == MT_Normal) {
        m = cls->findLocalMethod(name);
    }
    if (!m && (type == MT_None || type == MT_Static)) {
        m = cls->findLocalStaticMethod(name);
    }
    setType(cls, name, type, xsink);
}

void QoreReflectionMethod::setType(const QoreClass* cls, const char* name, method_type_e type, ExceptionSink* xsink) {
    if (!m) {
        const char* typestr;
        switch (type) {
            case MT_Normal: typestr = "normal "; break;
            case MT_Static: typestr = "static "; break;
            case MT_None: typestr = ""; break;
            default: assert(false); break;
        }
        xsink->raiseException("UNKNOWN-METHOD", "cannot resolve %smethod '%s' in class '%s'", typestr, name, cls->getName());
    } else {
        f = m->getFunction();
        setType();
    }
}

void QoreReflectionMethod::setType() {
    // set type
    mtype = m->getMethodType();
    if (mtype == MT_Normal && m->getClass()->getName()[0] == '<') {
        mtype = MT_Pseudo;
    }
}

QoreReflectionHierarchyMethod::QoreReflectionHierarchyMethod(const QoreClass* cls, const char* name, ExceptionSink* xsink, method_type_e type) : QoreReflectionMethod() {
    setMethod(cls, name, xsink, type);
}

void QoreReflectionHierarchyMethod::setMethod(const QoreClass* cls, const char* name, ExceptionSink* xsink, method_type_e type) {
    if (type == MT_None || type == MT_Normal) {
        m = cls->findMethod(name, class_access);
    }
    if (!m && (type == MT_None || type == MT_Static)) {
        m = cls->findStaticMethod(name, class_access);
    }
    setType(cls, name, type, xsink);
}
/* Qore class Qore::Reflection::AbstractMethod */

qore_classid_t CID_ABSTRACTMETHOD;
QoreClass* QC_ABSTRACTMETHOD;

// AbstractMethod::constructor() {}
static void AbstractMethod_constructor(QoreObject* self, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 129 "QC_AbstractMethod.qpp"
xsink->raiseException("ABSTRACTMETHOD-CONSTRUCTOR-ERROR", "this class is an abstract class and cannot be constructed directly or inherited directly by a user-defined class");
}

// AbstractClass AbstractMethod::getClass(){}
static QoreValue AbstractMethod_getClass(QoreObject* self, QoreReflectionMethod* m, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 266 "QC_AbstractMethod.qpp"
    return get_class_object(m->pgm, m->m->getClass());
}

// string AbstractMethod::getMethodTypeName(){}
static QoreValue AbstractMethod_getMethodTypeName(QoreObject* self, QoreReflectionMethod* m, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 164 "QC_AbstractMethod.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, m->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // the method ptr must be valid
    return new QoreStringNode(m->getType());
}

// AbstractMethodVariant AbstractMethod::getVariant(...){}
static QoreValue AbstractMethod_getVariant_VV(QoreObject* self, QoreReflectionMethod* m, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 194 "QC_AbstractMethod.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, m->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // the method ptr must be valid
    return find_function_variant(m->pgm, m->f, args, 0, m->mtype, xsink);
}

// AbstractMethodVariant AbstractMethod::getVariantArgs(softlist<Type> argv){}
static QoreValue AbstractMethod_getVariantArgs_C14softlist_Type_(QoreObject* self, QoreReflectionMethod* m, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreListNode* argv = HARD_QORE_VALUE_LIST(args, 0);
# 224 "QC_AbstractMethod.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, m->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // the method ptr must be valid
    return find_function_variant(m->pgm, m->f, argv, 0, m->mtype, xsink);
}

// list<AbstractMethodVariant> AbstractMethod::getVariants(){}
static QoreValue AbstractMethod_getVariants(QoreObject* self, QoreReflectionMethod* m, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 245 "QC_AbstractMethod.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, m->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // the method ptr must be valid
    ReferenceHolder<QoreListNode> rv(new QoreListNode(QC_ABSTRACTMETHODVARIANT->getTypeInfo()), xsink);
    append_variant_objects(**rv, m->pgm, m->f, m->mtype, xsink);
    return rv.release();
}

// bool AbstractMethod::isStatic(){}
static QoreValue AbstractMethod_isStatic(QoreObject* self, QoreReflectionMethod* m, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 143 "QC_AbstractMethod.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, m->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // the method ptr must be valid
    return m->m->isStatic();
}

DLLLOCAL void preinitAbstractMethodClass() {
    QC_ABSTRACTMETHOD = new QoreClass("AbstractMethod", "::Qore::Reflection::AbstractMethod", QDOM_REFLECTION);
    CID_ABSTRACTMETHOD = QC_ABSTRACTMETHOD->getID();
    QC_ABSTRACTMETHOD->setSystem();
}

DLLLOCAL QoreClass* initAbstractMethodClass(QoreNamespace& ns) {
    if (!QC_ABSTRACTMETHOD)
        preinitAbstractMethodClass();

    // set parent class
    assert(QC_ABSTRACTREFLECTIONFUNCTION);
    QC_ABSTRACTMETHOD->addBuiltinVirtualBaseClass(QC_ABSTRACTREFLECTIONFUNCTION);

    // AbstractMethod::constructor() {}
    QC_ABSTRACTMETHOD->addConstructor(AbstractMethod_constructor, Public, QCF_NO_FLAGS, QDOM_DEFAULT);

    // AbstractClass AbstractMethod::getClass(){}
    QC_ABSTRACTMETHOD->addMethod("getClass", (q_method_n_t)AbstractMethod_getClass, Public, QCF_CONSTANT, QDOM_DEFAULT, QC_ABSTRACTCLASS->getTypeInfo());

    // string AbstractMethod::getMethodTypeName(){}
    QC_ABSTRACTMETHOD->addMethod("getMethodTypeName", (q_method_n_t)AbstractMethod_getMethodTypeName, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, stringTypeInfo);

    // AbstractMethodVariant AbstractMethod::getVariant(...){}
    QC_ABSTRACTMETHOD->addMethod("getVariant", (q_method_n_t)AbstractMethod_getVariant_VV, Public, QCF_RET_VALUE_ONLY|QCF_USES_EXTRA_ARGS, QDOM_DEFAULT, QC_ABSTRACTMETHODVARIANT->getTypeInfo());

    // AbstractMethodVariant AbstractMethod::getVariantArgs(softlist<Type> argv){}
    QC_ABSTRACTMETHOD->addMethod("getVariantArgs", (q_method_n_t)AbstractMethod_getVariantArgs_C14softlist_Type_, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, QC_ABSTRACTMETHODVARIANT->getTypeInfo(), 1, qore_get_complex_softlist_type(QC_TYPE->getTypeInfo()), QORE_PARAM_NO_ARG, "argv");

    // list<AbstractMethodVariant> AbstractMethod::getVariants(){}
    QC_ABSTRACTMETHOD->addMethod("getVariants", (q_method_n_t)AbstractMethod_getVariants, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, qore_get_complex_list_type(QC_ABSTRACTMETHODVARIANT->getTypeInfo()));

    // bool AbstractMethod::isStatic(){}
    QC_ABSTRACTMETHOD->addMethod("isStatic", (q_method_n_t)AbstractMethod_isStatic, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, boolTypeInfo);

    return QC_ABSTRACTMETHOD;
}
