/* -*- mode: c++; indent-tabs-mode: nil -*- */
/** @file QC_AbstractReflectionFunction.cpp AbstractReflectionFunction class definition */
/*
    Qore Programming Language

    Copyright (C) 2003 - 2023 Qore Technologies, s.r.o.

    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
    to deal in the Software without restriction, including without limitation
    the rights to use, copy, modify, merge, publish, distribute, sublicense,
    and/or sell copies of the Software, and to permit persons to whom the
    Software is furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
    DEALINGS IN THE SOFTWARE.

    Note that the Qore library is released under a choice of three open-source
    licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
    information.
*/

#include "qore_reflection.h"
#include "QC_AbstractReflectionFunction.h"
#include "QC_AbstractMethod.h"
#include "QC_AbstractVariant.h"
#include "QC_NormalMethodVariant.h"
#include "QC_StaticMethodVariant.h"
#include "QC_ConstructorMethodVariant.h"
#include "QC_DestructorMethodVariant.h"
#include "QC_CopyMethodVariant.h"
#include "QC_PseudoMethodVariant.h"
#include "QC_FunctionVariant.h"
#include "QC_AbstractClass.h"
#include "QC_Type.h"
#include "QC_Function.h"

QoreObject* QoreReflectionFunction::getFunctionObject(ExceptionSink* xsink) const {
    const QoreClass* c = f->getClass();
    if (c) {
        const QoreExternalMethodFunction* mf = static_cast<const QoreExternalMethodFunction*>(f);
        // must find the QoreMethod corresponding to this function by searching the class by name by method type
        const QoreMethod* m = mf->getMethod();
        ReferenceHolder<QoreReflectionMethod> meth(new QoreReflectionMethod(pgm, m), xsink);
        return get_method_object(meth, xsink);
    }
    return new QoreObject(QC_FUNCTION, pgm, new QoreReflectionFunction(pgm, f));
}

QoreObject* find_function_variant(QoreProgram* pgm, const QoreExternalFunction* func, const QoreListNode* args, size_t offset, method_type_e mtype, ExceptionSink* xsink) {
    // create argument list
    type_vec_t type_vec;

    for (size_t i = offset; i < (args ? args->size() : 0); ++i) {
        const QoreValue v = args->retrieveEntry(i);
        if (v.getType() != NT_OBJECT) {
            xsink->raiseException("GETVARIANT-ERROR", "expecting a Type argument describing the parameter type; got type '%s' instead", v.getTypeName());
            return nullptr;
        }

        ReferenceHolder<QoreType> t(static_cast<QoreType*>(v.get<const QoreObject>()->tryGetReferencedPrivateData(CID_TYPE, xsink)), xsink);
        if (!t) {
            if (!*xsink) {
                xsink->raiseException("GETVARIANT-ERROR", "expecting a Type argument describing the parameter type; got object of class '%s' instead", v.get<const QoreObject>()->getClassName());
            }
            return nullptr;
        }
        type_vec.push_back(t->typeInfo);
    }

    const QoreExternalVariant* v = func->findVariant(type_vec, xsink);

    if (*xsink) {
        return nullptr;
    }
    assert(v);

    return get_variant_object(pgm, func, mtype, v, xsink);
}

QoreObject* get_variant_object(QoreProgram* pgm, const QoreExternalFunction* f, method_type_e mtype, const QoreExternalVariant* v, ExceptionSink* xsink) {
    switch (mtype) {
        case MT_None:
            return new QoreObject(QC_FUNCTIONVARIANT, pgm, new QoreReflectionVariant(pgm, f, v));
        case MT_Static:
            return new QoreObject(QC_STATICMETHODVARIANT, pgm, new QoreReflectionVariant(pgm, f, v));
        case MT_Normal:
            return new QoreObject(QC_NORMALMETHODVARIANT, pgm, new QoreReflectionVariant(pgm, f, v));
        case MT_Constructor:
            return new QoreObject(QC_CONSTRUCTORMETHODVARIANT, pgm, new QoreReflectionVariant(pgm, f, v));
        case MT_Destructor:
            return new QoreObject(QC_DESTRUCTORMETHODVARIANT, pgm, new QoreReflectionVariant(pgm, f, v));
        case MT_Copy:
            return new QoreObject(QC_COPYMETHODVARIANT, pgm, new QoreReflectionVariant(pgm, f, v));
        case MT_Pseudo:
            return new QoreObject(QC_PSEUDOMETHODVARIANT, pgm, new QoreReflectionVariant(pgm, f, v));
        default:
            break;
    }

    assert(false);
    return nullptr;
}

void append_variant_objects(QoreListNode& l, QoreProgram* pgm, const QoreExternalFunction* f, method_type_e mtype, ExceptionSink* xsink) {
    QoreExternalFunctionIterator i(*f);
    while (i.next()) {
        l.push(get_variant_object(pgm, f, mtype, i.getVariant(), xsink), xsink);
    }
}
/* Qore class Qore::Reflection::AbstractReflectionFunction */

qore_classid_t CID_ABSTRACTREFLECTIONFUNCTION;
QoreClass* QC_ABSTRACTREFLECTIONFUNCTION;

// AbstractReflectionFunction::constructor() {}
static void AbstractReflectionFunction_constructor(QoreObject* self, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 132 "QC_AbstractReflectionFunction.qpp"
xsink->raiseException("ABSTRACTREFLECTIONFUNCTION-CONSTRUCTOR-ERROR", "this class is an abstract class and cannot be constructed directly or inherited directly by a user-defined class");
}

// list<string> AbstractReflectionFunction::getCodeFlagList(){}
static QoreValue AbstractReflectionFunction_getCodeFlagList(QoreObject* self, QoreReflectionFunction* f, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 372 "QC_AbstractReflectionFunction.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, f->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // the function ptr must be valid
    return get_code_flag_list(f->f->getCodeFlags());
}

// int AbstractReflectionFunction::getCodeFlags(){}
static QoreValue AbstractReflectionFunction_getCodeFlags(QoreObject* self, QoreReflectionFunction* f, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 341 "QC_AbstractReflectionFunction.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, f->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // the function ptr must be valid
    return f->f->getCodeFlags();
}

// int AbstractReflectionFunction::getDomain(){}
static QoreValue AbstractReflectionFunction_getDomain(QoreObject* self, QoreReflectionFunction* f, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 397 "QC_AbstractReflectionFunction.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, f->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // the function ptr must be valid
    return f->f->getDomain();
}

// list<string> AbstractReflectionFunction::getDomainStringList(){}
static QoreValue AbstractReflectionFunction_getDomainStringList(QoreObject* self, QoreReflectionFunction* f, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 422 "QC_AbstractReflectionFunction.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, f->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // the function ptr must be valid
    return domain_bitfield_to_string_list(f->f->getDomain(), xsink);
}

// *string AbstractReflectionFunction::getModuleName(){}
static QoreValue AbstractReflectionFunction_getModuleName(QoreObject* self, QoreReflectionFunction* f, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 186 "QC_AbstractReflectionFunction.qpp"
    QoreExternalProgramContextHelper tch(xsink, f->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // the function ptr must be valid
    const char* mod = f->f->getModuleName();
    return mod ? new QoreStringNode(mod) : QoreValue();
}

// string AbstractReflectionFunction::getName(){}
static QoreValue AbstractReflectionFunction_getName(QoreObject* self, QoreReflectionFunction* f, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 165 "QC_AbstractReflectionFunction.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, f->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // the function ptr must be valid
    return new QoreStringNode(f->f->getName());
}

// bool AbstractReflectionFunction::isBuiltin(){}
static QoreValue AbstractReflectionFunction_isBuiltin(QoreObject* self, QoreReflectionFunction* f, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 209 "QC_AbstractReflectionFunction.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, f->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // the function ptr must be valid
    return f->f->isBuiltin();
}

// bool AbstractReflectionFunction::isEqual(AbstractReflectionFunction func){}
static QoreValue AbstractReflectionFunction_isEqual_C26AbstractReflectionFunction(QoreObject* self, QoreReflectionFunction* f, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    HARD_QORE_VALUE_OBJ_DATA(func, QoreReflectionFunction, args, 0, CID_ABSTRACTREFLECTIONFUNCTION, "AbstractReflectionFunction::isEqual()", "AbstractReflectionFunction", xsink);
    if (*xsink)
        return 0;
# 149 "QC_AbstractReflectionFunction.qpp"
    ReferenceHolder<QoreReflectionFunction> holder(func, xsink);

    return f->f == func->f;
}

// bool AbstractReflectionFunction::isInjected(){}
static QoreValue AbstractReflectionFunction_isInjected(QoreObject* self, QoreReflectionFunction* f, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 253 "QC_AbstractReflectionFunction.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, f->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // the function ptr must be valid
    return f->f->isInjected();
}

// bool AbstractReflectionFunction::isMethod(){}
static QoreValue AbstractReflectionFunction_isMethod(QoreObject* self, QoreReflectionFunction* f, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 274 "QC_AbstractReflectionFunction.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, f->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // the function ptr must be valid
    return (bool)(f->f->getClass() && !f->f->getClass()->isPseudoClass());
}

// bool AbstractReflectionFunction::isPseudoMethod(){}
static QoreValue AbstractReflectionFunction_isPseudoMethod(QoreObject* self, QoreReflectionFunction* f, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 295 "QC_AbstractReflectionFunction.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, f->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // the function ptr must be valid
    return (bool)(f->f->getClass() && f->f->getClass()->isPseudoClass());
}

// bool AbstractReflectionFunction::isUser(){}
static QoreValue AbstractReflectionFunction_isUser(QoreObject* self, QoreReflectionFunction* f, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 232 "QC_AbstractReflectionFunction.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, f->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // the function ptr must be valid
    return !f->f->isBuiltin();
}

// int AbstractReflectionFunction::numVariants(){}
static QoreValue AbstractReflectionFunction_numVariants(QoreObject* self, QoreReflectionFunction* f, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 316 "QC_AbstractReflectionFunction.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, f->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // the function ptr must be valid
    return f->f->numVariants();
}

DLLLOCAL void preinitAbstractReflectionFunctionClass() {
    QC_ABSTRACTREFLECTIONFUNCTION = new QoreClass("AbstractReflectionFunction", "::Qore::Reflection::AbstractReflectionFunction", QDOM_REFLECTION);
    CID_ABSTRACTREFLECTIONFUNCTION = QC_ABSTRACTREFLECTIONFUNCTION->getID();
    QC_ABSTRACTREFLECTIONFUNCTION->setSystem();
}

DLLLOCAL QoreClass* initAbstractReflectionFunctionClass(QoreNamespace& ns) {
    if (!QC_ABSTRACTREFLECTIONFUNCTION)
        preinitAbstractReflectionFunctionClass();

    // AbstractReflectionFunction::constructor() {}
    QC_ABSTRACTREFLECTIONFUNCTION->addConstructor(AbstractReflectionFunction_constructor, Public, QCF_NO_FLAGS, QDOM_DEFAULT);

    // list<string> AbstractReflectionFunction::getCodeFlagList(){}
    QC_ABSTRACTREFLECTIONFUNCTION->addMethod("getCodeFlagList", (q_method_n_t)AbstractReflectionFunction_getCodeFlagList, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, qore_get_complex_list_type(stringTypeInfo));

    // int AbstractReflectionFunction::getCodeFlags(){}
    QC_ABSTRACTREFLECTIONFUNCTION->addMethod("getCodeFlags", (q_method_n_t)AbstractReflectionFunction_getCodeFlags, Public, QCF_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

    // int AbstractReflectionFunction::getDomain(){}
    QC_ABSTRACTREFLECTIONFUNCTION->addMethod("getDomain", (q_method_n_t)AbstractReflectionFunction_getDomain, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, bigIntTypeInfo);

    // list<string> AbstractReflectionFunction::getDomainStringList(){}
    QC_ABSTRACTREFLECTIONFUNCTION->addMethod("getDomainStringList", (q_method_n_t)AbstractReflectionFunction_getDomainStringList, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, qore_get_complex_list_type(stringTypeInfo));

    // *string AbstractReflectionFunction::getModuleName(){}
    QC_ABSTRACTREFLECTIONFUNCTION->addMethod("getModuleName", (q_method_n_t)AbstractReflectionFunction_getModuleName, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, stringOrNothingTypeInfo);

    // string AbstractReflectionFunction::getName(){}
    QC_ABSTRACTREFLECTIONFUNCTION->addMethod("getName", (q_method_n_t)AbstractReflectionFunction_getName, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, stringTypeInfo);

    // bool AbstractReflectionFunction::isBuiltin(){}
    QC_ABSTRACTREFLECTIONFUNCTION->addMethod("isBuiltin", (q_method_n_t)AbstractReflectionFunction_isBuiltin, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, boolTypeInfo);

    // bool AbstractReflectionFunction::isEqual(AbstractReflectionFunction func){}
    QC_ABSTRACTREFLECTIONFUNCTION->addMethod("isEqual", (q_method_n_t)AbstractReflectionFunction_isEqual_C26AbstractReflectionFunction, Public, QCF_CONSTANT, QDOM_DEFAULT, boolTypeInfo, 1, QC_ABSTRACTREFLECTIONFUNCTION->getTypeInfo(), QORE_PARAM_NO_ARG, "func");

    // bool AbstractReflectionFunction::isInjected(){}
    QC_ABSTRACTREFLECTIONFUNCTION->addMethod("isInjected", (q_method_n_t)AbstractReflectionFunction_isInjected, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, boolTypeInfo);

    // bool AbstractReflectionFunction::isMethod(){}
    QC_ABSTRACTREFLECTIONFUNCTION->addMethod("isMethod", (q_method_n_t)AbstractReflectionFunction_isMethod, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, boolTypeInfo);

    // bool AbstractReflectionFunction::isPseudoMethod(){}
    QC_ABSTRACTREFLECTIONFUNCTION->addMethod("isPseudoMethod", (q_method_n_t)AbstractReflectionFunction_isPseudoMethod, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, boolTypeInfo);

    // bool AbstractReflectionFunction::isUser(){}
    QC_ABSTRACTREFLECTIONFUNCTION->addMethod("isUser", (q_method_n_t)AbstractReflectionFunction_isUser, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, boolTypeInfo);

    // int AbstractReflectionFunction::numVariants(){}
    QC_ABSTRACTREFLECTIONFUNCTION->addMethod("numVariants", (q_method_n_t)AbstractReflectionFunction_numVariants, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, bigIntTypeInfo);

    return QC_ABSTRACTREFLECTIONFUNCTION;
}
