/* -*- mode: c++; indent-tabs-mode: nil -*- */
/** @file QC_PseudoMethod.cpp PseudoMethod class definition */
/*
    Qore Programming Language

    Copyright (C) 2003 - 2023 Qore Technologies, s.r.o.

    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
    to deal in the Software without restriction, including without limitation
    the rights to use, copy, modify, merge, publish, distribute, sublicense,
    and/or sell copies of the Software, and to permit persons to whom the
    Software is furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
    DEALINGS IN THE SOFTWARE.

    Note that the Qore library is released under a choice of three open-source
    licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
    information.
*/

#include <qore_reflection.h>
#include "QC_PseudoMethod.h"
#include "QC_Class.h"

int check_pseudo_call(QoreValue& val, const QoreMethod& m, ExceptionSink* xsink) {
    const QoreClass* cls = m.getClass();
    assert(cls->isPseudoClass());
    qore_type_t t = cls->getPseudoClassType();
    if (t == -1 || t == val.getType()) {
        return 0;
    }
    xsink->raiseException("METHOD-ERROR", "value has type '%s' which is not compatible with method '%s()' of pseudo-class '%s'", val.getTypeName(), m.getName(), m.getClassName());
    return -1;
}
/* Qore class Qore::Reflection::PseudoMethod */

qore_classid_t CID_PSEUDOMETHOD;
QoreClass* QC_PSEUDOMETHOD;

// auto PseudoMethod::call(auto val, ...){}
static QoreValue PseudoMethod_call_VAVV(QoreObject* self, QoreReflectionMethod* m, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    QoreValue val = get_param_value(args, 0);
# 77 "QC_PseudoMethod.qpp"
    // we have to explictly test value compatibility here, otherwise QoreClass::evalPseudoMethod() will throw
    // a confusing exception
    if (check_pseudo_call(val, *m->m, xsink)) {
        return QoreValue();
    }

    ReferenceHolder<QoreListNode> call_args(xsink);

    // if there are arguments to pass
    if (num_args(args) > 1) {
        // create argument list by copying current list
        call_args = args->copyListFrom(1);
    }

    // pseudo-classes and pseudo-methods are always valid
    return m->m->getClass()->evalPseudoMethod(m->m, nullptr, val, *call_args, xsink);
}

// auto PseudoMethod::callArgs(auto val, *softlist<auto> argv){}
static QoreValue PseudoMethod_callArgs_VAC15_softlist_auto_(QoreObject* self, QoreReflectionMethod* m, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    QoreValue val = get_param_value(args, 0);
    const QoreListNode* argv = get_param_value(args, 1).get<const QoreListNode>();
# 111 "QC_PseudoMethod.qpp"
    // we have to explictly test value compatibility here, otherwise QoreClass::evalPseudoMethod() will throw
    // a confusing exception
    if (check_pseudo_call(val, *m->m, xsink)) {
        return QoreValue();
    }

    // pseudo-classes and pseudo-methods are always valid
    return m->m->getClass()->evalPseudoMethod(m->m, nullptr, val, argv, xsink);
}

// PseudoMethod::constructor() {}
static void PseudoMethod_constructor(QoreObject* self, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 58 "QC_PseudoMethod.qpp"
xsink->raiseException("PSEUDOMETHOD-CONSTRUCTOR-ERROR", "this class is an abstract class and cannot be constructed directly or inherited directly by a user-defined class");
}

DLLLOCAL void preinitPseudoMethodClass() {
    QC_PSEUDOMETHOD = new QoreClass("PseudoMethod", "::Qore::Reflection::PseudoMethod", QDOM_REFLECTION);
    CID_PSEUDOMETHOD = QC_PSEUDOMETHOD->getID();
    QC_PSEUDOMETHOD->setSystem();
}

DLLLOCAL QoreClass* initPseudoMethodClass(QoreNamespace& ns) {
    if (!QC_PSEUDOMETHOD)
        preinitPseudoMethodClass();

    // set parent class
    assert(QC_ABSTRACTMETHOD);
    QC_PSEUDOMETHOD->addBuiltinVirtualBaseClass(QC_ABSTRACTMETHOD);

    // auto PseudoMethod::call(auto val, ...){}
    QC_PSEUDOMETHOD->addMethod("call", (q_method_n_t)PseudoMethod_call_VAVV, Public, QCF_USES_EXTRA_ARGS, QDOM_DEFAULT, autoTypeInfo, 1, autoTypeInfo, QORE_PARAM_NO_ARG, "val");

    // auto PseudoMethod::callArgs(auto val, *softlist<auto> argv){}
    QC_PSEUDOMETHOD->addMethod("callArgs", (q_method_n_t)PseudoMethod_callArgs_VAC15_softlist_auto_, Public, QCF_NO_FLAGS, QDOM_DEFAULT, autoTypeInfo, 2, autoTypeInfo, QORE_PARAM_NO_ARG, "val", softAutoListOrNothingTypeInfo, QORE_PARAM_NO_ARG, "argv");

    // PseudoMethod::constructor() {}
    QC_PSEUDOMETHOD->addConstructor(PseudoMethod_constructor, Public, QCF_NO_FLAGS, QDOM_DEFAULT);

    return QC_PSEUDOMETHOD;
}
