/* -*- mode: c++; indent-tabs-mode: nil -*- */
/** @file QC_PseudoMethodVariant.cpp PseudoMethodVariant class definition */
/*
    Qore Programming Language

    Copyright (C) 2003 - 2023 Qore Technologies, s.r.o.

    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
    to deal in the Software without restriction, including without limitation
    the rights to use, copy, modify, merge, publish, distribute, sublicense,
    and/or sell copies of the Software, and to permit persons to whom the
    Software is furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
    DEALINGS IN THE SOFTWARE.

    Note that the Qore library is released under a choice of three open-source
    licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
    information.
*/

#include "qore_reflection.h"
#include "QC_PseudoMethodVariant.h"
#include "QC_PseudoMethod.h"
/* Qore class Qore::Reflection::PseudoMethodVariant */

qore_classid_t CID_PSEUDOMETHODVARIANT;
QoreClass* QC_PSEUDOMETHODVARIANT;

// auto PseudoMethodVariant::call(auto val, ...){}
static QoreValue PseudoMethodVariant_call_VAVV(QoreObject* self, QoreReflectionVariant* v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    QoreValue val = get_param_value(args, 0);
# 69 "QC_PseudoMethodVariant.qpp"
    // we have to explictly test object compatibility here, otherwise QoreObject::evalMethod() will throw
    // a confusing exception
    const QoreExternalMethodVariant* mv = reinterpret_cast<const QoreExternalMethodVariant*>(v->variant);
    const QoreMethod* m = mv->getMethod();
    if (check_pseudo_call(val, *m, xsink)) {
        return QoreValue();
    }

    ReferenceHolder<QoreListNode> call_args(xsink);

    // if there are arguments to pass
    if (num_args(args) > 1) {
        // create argument list by copying current list
        call_args = args->copyListFrom(1);
    }

    // pseudo-classes and pseudo-methods are always valid
    return m->getClass()->evalPseudoMethod(m, mv, val, *call_args, xsink);
}

// auto PseudoMethodVariant::callArgs(auto val, *softlist<auto> argv){}
static QoreValue PseudoMethodVariant_callArgs_VAC15_softlist_auto_(QoreObject* self, QoreReflectionVariant* v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    QoreValue val = get_param_value(args, 0);
    const QoreListNode* argv = get_param_value(args, 1).get<const QoreListNode>();
# 106 "QC_PseudoMethodVariant.qpp"
    // we have to explictly test object compatibility here, otherwise QoreObject::evalMethod() will throw
    // a confusing exception
    const QoreExternalMethodVariant* mv = reinterpret_cast<const QoreExternalMethodVariant*>(v->variant);
    const QoreMethod* m = mv->getMethod();
    if (check_pseudo_call(val, *m, xsink)) {
        return QoreValue();
    }

    // pseudo-classes and pseudo-methods are always valid
    return m->getClass()->evalPseudoMethod(m, mv, val, argv, xsink);
}

// PseudoMethodVariant::constructor() {}
static void PseudoMethodVariant_constructor(QoreObject* self, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 49 "QC_PseudoMethodVariant.qpp"
xsink->raiseException("PSEUDOMETHODVARIANT-CONSTRUCTOR-ERROR", "this class is an abstract class and cannot be constructed directly or inherited directly by a user-defined class");
}

DLLLOCAL void preinitPseudoMethodVariantClass() {
    QC_PSEUDOMETHODVARIANT = new QoreClass("PseudoMethodVariant", "::Qore::Reflection::PseudoMethodVariant", QDOM_REFLECTION);
    CID_PSEUDOMETHODVARIANT = QC_PSEUDOMETHODVARIANT->getID();
    QC_PSEUDOMETHODVARIANT->setSystem();
}

DLLLOCAL QoreClass* initPseudoMethodVariantClass(QoreNamespace& ns) {
    if (!QC_PSEUDOMETHODVARIANT)
        preinitPseudoMethodVariantClass();

    // set parent class
    assert(QC_ABSTRACTMETHODVARIANT);
    QC_PSEUDOMETHODVARIANT->addBuiltinVirtualBaseClass(QC_ABSTRACTMETHODVARIANT);

    // auto PseudoMethodVariant::call(auto val, ...){}
    QC_PSEUDOMETHODVARIANT->addMethod("call", (q_method_n_t)PseudoMethodVariant_call_VAVV, Public, QCF_USES_EXTRA_ARGS, QDOM_DEFAULT, autoTypeInfo, 1, autoTypeInfo, QORE_PARAM_NO_ARG, "val");

    // auto PseudoMethodVariant::callArgs(auto val, *softlist<auto> argv){}
    QC_PSEUDOMETHODVARIANT->addMethod("callArgs", (q_method_n_t)PseudoMethodVariant_callArgs_VAC15_softlist_auto_, Public, QCF_NO_FLAGS, QDOM_DEFAULT, autoTypeInfo, 2, autoTypeInfo, QORE_PARAM_NO_ARG, "val", softAutoListOrNothingTypeInfo, QORE_PARAM_NO_ARG, "argv");

    // PseudoMethodVariant::constructor() {}
    QC_PSEUDOMETHODVARIANT->addConstructor(PseudoMethodVariant_constructor, Public, QCF_NO_FLAGS, QDOM_DEFAULT);

    return QC_PSEUDOMETHODVARIANT;
}
