/* -*- mode: c++; indent-tabs-mode: nil -*- */
/** @file QC_TypedHash.cpp typed hash (hashdecl) class definition */
/*
    Qore Programming Language

    Copyright (C) 2003 - 2023 Qore Technologies, s.r.o.

    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
    to deal in the Software without restriction, including without limitation
    the rights to use, copy, modify, merge, publish, distribute, sublicense,
    and/or sell copies of the Software, and to permit persons to whom the
    Software is furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
    DEALINGS IN THE SOFTWARE.

    Note that the Qore library is released under a choice of three open-source
    licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
    information.
*/

#include "qore_reflection.h"
#include "QC_TypedHash.h"
#include "QC_TypedHashMember.h"
#include "QC_Class.h"
#include "QC_Type.h"
#include "QC_Namespace.h"
#include "QC_Constant.h"
#include "QC_Function.h"
#include "QC_GlobalVar.h"
#include "QC_TypedHash.h"
#include "QC_FunctionVariant.h"

QoreReflectionTypedHash::QoreReflectionTypedHash(ExceptionSink* xsink, const char* path, QoreProgram* pgm)
    : AbstractReflectionObject(pgm) {
    th = pgm->findHashDecl(path, ns);

    if (!th) {
        xsink->raiseException("UNKNOWN-TYPED-HASH", "cannot resolve typed hash (hashdecl) '%s'", path);
        return;
    }
}

static QoreValue find_all_typedhashes(const QoreString& pattern, int re_opts, QoreProgram* pgm, ExceptionSink* xsink) {
    hashdecl_vec_t hashdecl_vec = pgm->findAllHashDeclsRegex(pattern, re_opts, xsink);
    if (*xsink) {
        return QoreValue();
    }

    ReferenceHolder<QoreListNode> l(new QoreListNode(QC_TYPEDHASH->getTypeInfo()), xsink);
    for (auto& i : hashdecl_vec) {
        l->push(new QoreObject(QC_TYPEDHASH, pgm, new QoreReflectionTypedHash(pgm, i.first, i.second)), xsink);
    }

    return l.release();
}
/* Qore class Qore::Reflection::TypedHash */

qore_classid_t CID_TYPEDHASH;
QoreClass* QC_TYPEDHASH;

// TypedHash::constructor(string path) {}
static void TypedHash_constructor_Vs(QoreObject* self, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* path = HARD_QORE_VALUE_STRING(args, 0);
# 87 "QC_TypedHash.qpp"
TempEncodingHelper npath(path, QCS_DEFAULT, xsink);
    if (*xsink) {
        return;
    }

    ReferenceHolder<QoreReflectionTypedHash> th(new QoreReflectionTypedHash(xsink, npath->c_str()), xsink);
    if (*xsink) {
        return;
    }
    self->setPrivate(CID_TYPEDHASH, th.release());
}

// hash<auto> TypedHash::doCast(hash<auto> h){}
static QoreValue TypedHash_doCast_C10hash_auto_(QoreObject* self, QoreReflectionTypedHash* thd, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreHashNode* h = HARD_QORE_VALUE_HASH(args, 0);
# 381 "QC_TypedHash.qpp"
    return thd->th->doRuntimeCast(h, xsink);
}

// TypedHashMember TypedHash::getMember(string name){}
static QoreValue TypedHash_getMember_Vs(QoreObject* self, QoreReflectionTypedHash* thd, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* name = HARD_QORE_VALUE_STRING(args, 0);
# 324 "QC_TypedHash.qpp"
    TempEncodingHelper tname(name, QCS_DEFAULT, xsink);
    if (*xsink) {
        return QoreValue();
    }

    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, thd->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // the typed hash ptr must be valid
    const QoreExternalMemberBase* mi = thd->th->findLocalMember(tname->c_str());
    if (!mi) {
        xsink->raiseException("UNKNOWN-MEMBER", "member '%s' is not defined in typed hash (hashdecl) '%s'", tname->c_str(), thd->th->getName());
        return QoreValue();
    }

    return new QoreObject(QC_TYPEDHASHMEMBER, thd->pgm, new QoreReflectionHashDeclMember(thd->pgm, thd->th, tname->c_str(), mi));
}

// list<TypedHashMember> TypedHash::getMembers(){}
static QoreValue TypedHash_getMembers(QoreObject* self, QoreReflectionTypedHash* thd, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 356 "QC_TypedHash.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, thd->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // the typed hash ptr must be valid
    ReferenceHolder<QoreListNode> l(new QoreListNode(QC_TYPEDHASHMEMBER->getTypeInfo()), xsink);

    TypedHashDeclMemberIterator i(*thd->th);
    while (i.next()) {
        l->push(new QoreObject(QC_TYPEDHASHMEMBER, thd->pgm, new QoreReflectionHashDeclMember(thd->pgm, thd->th, i.getName(), &i.getMember())), xsink);
    }

    return l.release();
}

// *string TypedHash::getModuleName(){}
static QoreValue TypedHash_getModuleName(QoreObject* self, QoreReflectionTypedHash* thd, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 175 "QC_TypedHash.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, thd->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // the class ptr must be valid
    const char* mod = thd->th->getModuleName();
    return mod ? new QoreStringNode(mod) : QoreValue();
}

// string TypedHash::getName(){}
static QoreValue TypedHash_getName(QoreObject* self, QoreReflectionTypedHash* thd, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 131 "QC_TypedHash.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, thd->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // the typed hash (hashdecl) ptr must be valid
    return new QoreStringNode(thd->th->getName());
}

// Namespace TypedHash::getNamespace(){}
static QoreValue TypedHash_getNamespace(QoreObject* self, QoreReflectionTypedHash* thd, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 308 "QC_TypedHash.qpp"
    return new QoreObject(QC_NAMESPACE, thd->pgm, new QoreReflectionNamespace(thd->pgm, thd->ns));
}

// Type TypedHash::getOrNothingType(){}
static QoreValue TypedHash_getOrNothingType(QoreObject* self, QoreReflectionTypedHash* thd, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 289 "QC_TypedHash.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, thd->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // the typed hash ptr must be valid
    return new QoreObject(QC_TYPE, thd->pgm, new QoreType(thd->th->getTypeInfo(true)));
}

// string TypedHash::getPathName(){}
static QoreValue TypedHash_getPathName(QoreObject* self, QoreReflectionTypedHash* thd, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 154 "QC_TypedHash.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, thd->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // the typed hash (hashdecl) ptr must be valid
    SimpleRefHolder<QoreStringNode> rv(new QoreStringNode);
    rv->concat(thd->th->getNamespacePath());
    return rv.release();
}

// hash<SourceLocationInfo> TypedHash::getSourceLocation(){}
static QoreValue TypedHash_getSourceLocation(QoreObject* self, QoreReflectionTypedHash* thd, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 243 "QC_TypedHash.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, thd->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // the typed hash ptr must be valid
    return thd->th->getSourceLocation()->getHash();
}

// Type TypedHash::getType(){}
static QoreValue TypedHash_getType(QoreObject* self, QoreReflectionTypedHash* thd, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 266 "QC_TypedHash.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, thd->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // the typed hash ptr must be valid
    return new QoreObject(QC_TYPE, thd->pgm, new QoreType(thd->th->getTypeInfo()));
}

// bool TypedHash::isBuiltin(){}
static QoreValue TypedHash_isBuiltin(QoreObject* self, QoreReflectionTypedHash* thd, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 199 "QC_TypedHash.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, thd->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // the typed hash (hashdecl) ptr must be valid
    return thd->th->isSystem();
}

// bool TypedHash::isEqual(TypedHash th){}
static QoreValue TypedHash_isEqual_C9TypedHash(QoreObject* self, QoreReflectionTypedHash* thd, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    HARD_QORE_VALUE_OBJ_DATA(th, QoreReflectionTypedHash, args, 0, CID_TYPEDHASH, "TypedHash::isEqual()", "TypedHash", xsink);
    if (*xsink)
        return 0;
# 113 "QC_TypedHash.qpp"
    ReferenceHolder<QoreReflectionTypedHash> holder(th, xsink);

    return thd->th == th->th;
}

// bool TypedHash::isUser(){}
static QoreValue TypedHash_isUser(QoreObject* self, QoreReflectionTypedHash* thd, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 222 "QC_TypedHash.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, thd->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // the typed hash (hashdecl) ptr must be valid
    return !thd->th->isSystem();
}

// static list<TypedHash> TypedHash::findAllRegex(string pattern, *int re_opts){}
static QoreValue static_TypedHash_findAllRegex_VsNi(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* pattern = HARD_QORE_VALUE_STRING(args, 0);
    int64 re_opts = HARD_QORE_VALUE_INT(args, 1);
# 456 "QC_TypedHash.qpp"
return find_all_typedhashes(*pattern, re_opts, getProgram(), xsink);
}

// static list<TypedHash> TypedHash::findAllRegex(Program pgm, string pattern, *int re_opts){}
static QoreValue static_TypedHash_findAllRegex_C7ProgramVsNi(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    HARD_QORE_VALUE_OBJ_DATA(pgm, QoreProgram, args, 0, CID_PROGRAM, "TypedHash::findAllRegex()", "Program", xsink);
    if (*xsink)
        return 0;
    const QoreStringNode* pattern = HARD_QORE_VALUE_STRING(args, 1);
    int64 re_opts = HARD_QORE_VALUE_INT(args, 2);
# 473 "QC_TypedHash.qpp"
ReferenceHolder<QoreProgram> pgm_holder(pgm, xsink);
    return find_all_typedhashes(*pattern, re_opts, pgm, xsink);
}

// static TypedHash TypedHash::forName(string path){}
static QoreValue static_TypedHash_forName_Vs(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* path = HARD_QORE_VALUE_STRING(args, 0);
# 398 "QC_TypedHash.qpp"
TempEncodingHelper npath(path, QCS_DEFAULT, xsink);
    if (*xsink) {
        return QoreValue();
    }

    ReferenceHolder<QoreReflectionTypedHash> th(new QoreReflectionTypedHash(xsink, npath->c_str()), xsink);
    if (*xsink) {
        return QoreValue();
    }
    assert(*th);

    QoreProgram* hash_pgm = th->pgm;
    return new QoreObject(QC_TYPEDHASH, hash_pgm, th.release());
}

// static TypedHash TypedHash::forName(Program pgm, string path){}
static QoreValue static_TypedHash_forName_C7ProgramVs(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    HARD_QORE_VALUE_OBJ_DATA(pgm, QoreProgram, args, 0, CID_PROGRAM, "TypedHash::forName()", "Program", xsink);
    if (*xsink)
        return 0;
    const QoreStringNode* path = HARD_QORE_VALUE_STRING(args, 1);
# 428 "QC_TypedHash.qpp"
ReferenceHolder<QoreProgram> pgm_holder(pgm, xsink);
    TempEncodingHelper npath(path, QCS_DEFAULT, xsink);
    if (*xsink) {
        return QoreValue();
    }

    ReferenceHolder<QoreReflectionTypedHash> th(new QoreReflectionTypedHash(xsink, npath->c_str(), pgm), xsink);
    if (*xsink) {
        return QoreValue();
    }

    QoreProgram* hash_pgm = th->pgm;
    return new QoreObject(QC_TYPEDHASH, hash_pgm, th.release());
}

DLLLOCAL void preinitTypedHashClass() {
    QC_TYPEDHASH = new QoreClass("TypedHash", "::Qore::Reflection::TypedHash", QDOM_REFLECTION);
    CID_TYPEDHASH = QC_TYPEDHASH->getID();
    QC_TYPEDHASH->setSystem();
}

DLLLOCAL QoreClass* initTypedHashClass(QoreNamespace& ns) {
    if (!QC_TYPEDHASH)
        preinitTypedHashClass();

    // TypedHash::constructor(string path) {}
    QC_TYPEDHASH->addConstructor(TypedHash_constructor_Vs, Public, QCF_NO_FLAGS, QDOM_DEFAULT, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "path");

    // hash<auto> TypedHash::doCast(hash<auto> h){}
    QC_TYPEDHASH->addMethod("doCast", (q_method_n_t)TypedHash_doCast_C10hash_auto_, Public, QCF_NO_FLAGS, QDOM_DEFAULT, autoHashTypeInfo, 1, autoHashTypeInfo, QORE_PARAM_NO_ARG, "h");

    // TypedHashMember TypedHash::getMember(string name){}
    QC_TYPEDHASH->addMethod("getMember", (q_method_n_t)TypedHash_getMember_Vs, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, QC_TYPEDHASHMEMBER->getTypeInfo(), 1, stringTypeInfo, QORE_PARAM_NO_ARG, "name");

    // list<TypedHashMember> TypedHash::getMembers(){}
    QC_TYPEDHASH->addMethod("getMembers", (q_method_n_t)TypedHash_getMembers, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, qore_get_complex_list_type(QC_TYPEDHASHMEMBER->getTypeInfo()));

    // *string TypedHash::getModuleName(){}
    QC_TYPEDHASH->addMethod("getModuleName", (q_method_n_t)TypedHash_getModuleName, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, stringOrNothingTypeInfo);

    // string TypedHash::getName(){}
    QC_TYPEDHASH->addMethod("getName", (q_method_n_t)TypedHash_getName, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, stringTypeInfo);

    // Namespace TypedHash::getNamespace(){}
    QC_TYPEDHASH->addMethod("getNamespace", (q_method_n_t)TypedHash_getNamespace, Public, QCF_CONSTANT, QDOM_DEFAULT, QC_NAMESPACE->getTypeInfo());

    // Type TypedHash::getOrNothingType(){}
    QC_TYPEDHASH->addMethod("getOrNothingType", (q_method_n_t)TypedHash_getOrNothingType, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, QC_TYPE->getTypeInfo());

    // string TypedHash::getPathName(){}
    QC_TYPEDHASH->addMethod("getPathName", (q_method_n_t)TypedHash_getPathName, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, stringTypeInfo);

    // hash<SourceLocationInfo> TypedHash::getSourceLocation(){}
    QC_TYPEDHASH->addMethod("getSourceLocation", (q_method_n_t)TypedHash_getSourceLocation, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, hashdeclSourceLocationInfo->getTypeInfo(false));

    // Type TypedHash::getType(){}
    QC_TYPEDHASH->addMethod("getType", (q_method_n_t)TypedHash_getType, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, QC_TYPE->getTypeInfo());

    // bool TypedHash::isBuiltin(){}
    QC_TYPEDHASH->addMethod("isBuiltin", (q_method_n_t)TypedHash_isBuiltin, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, boolTypeInfo);

    // bool TypedHash::isEqual(TypedHash th){}
    QC_TYPEDHASH->addMethod("isEqual", (q_method_n_t)TypedHash_isEqual_C9TypedHash, Public, QCF_CONSTANT, QDOM_DEFAULT, boolTypeInfo, 1, QC_TYPEDHASH->getTypeInfo(), QORE_PARAM_NO_ARG, "th");

    // bool TypedHash::isUser(){}
    QC_TYPEDHASH->addMethod("isUser", (q_method_n_t)TypedHash_isUser, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, boolTypeInfo);

    // static list<TypedHash> TypedHash::findAllRegex(string pattern, *int re_opts){}
    QC_TYPEDHASH->addStaticMethod("findAllRegex", (q_func_n_t)static_TypedHash_findAllRegex_VsNi, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, qore_get_complex_list_type(QC_TYPEDHASH->getTypeInfo()), 2, stringTypeInfo, QORE_PARAM_NO_ARG, "pattern", bigIntOrNothingTypeInfo, QORE_PARAM_NO_ARG, "re_opts");

    // static list<TypedHash> TypedHash::findAllRegex(Program pgm, string pattern, *int re_opts){}
    QC_TYPEDHASH->addStaticMethod("findAllRegex", (q_func_n_t)static_TypedHash_findAllRegex_C7ProgramVsNi, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, qore_get_complex_list_type(QC_TYPEDHASH->getTypeInfo()), 3, QC_PROGRAM->getTypeInfo(), QORE_PARAM_NO_ARG, "pgm", stringTypeInfo, QORE_PARAM_NO_ARG, "pattern", bigIntOrNothingTypeInfo, QORE_PARAM_NO_ARG, "re_opts");

    // static TypedHash TypedHash::forName(string path){}
    QC_TYPEDHASH->addStaticMethod("forName", (q_func_n_t)static_TypedHash_forName_Vs, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, QC_TYPEDHASH->getTypeInfo(), 1, stringTypeInfo, QORE_PARAM_NO_ARG, "path");

    // static TypedHash TypedHash::forName(Program pgm, string path){}
    QC_TYPEDHASH->addStaticMethod("forName", (q_func_n_t)static_TypedHash_forName_C7ProgramVs, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, QC_TYPEDHASH->getTypeInfo(), 2, QC_PROGRAM->getTypeInfo(), QORE_PARAM_NO_ARG, "pgm", stringTypeInfo, QORE_PARAM_NO_ARG, "path");

    return QC_TYPEDHASH;
}
